; ***********************************************************************************
;+
; NAME
; STEREOPOLE
;
; Stereopole is a program for pole figure analysis of Philips PCTEXTURE *.RWA or ASCII
; column data files of the format "PHI [Tab or Blanks] PSI [Tab or Blanks] INTENSITY"
; in x-ray diffraction. The purpose of this software is to easily compare your
; measured data with the expected results of crystallography for two crystal systems
; at a time, offering the possibility to rotate the pole figure and the stereographic
; projections to match which yields the epitaxial relationships as a result.
; Export to .TIF, .JPG, and .PS is included.
;
; The program is written in IDL 6.0 Win32 (x86) of Research systems Inc. and should
; run on any version greater than v5.0. To run it, you need at least the IDL Runtime
; Enviroment "IDL Virtual Machine", for more details see: http://www.rsinc.com
;
; AUTHOR:
;
;   Ingo Salzmann
;   Institut fr Physik
;	Humboldt universitt zu Berlin
;	Newtonstr. 15
;	D-12489 Berlin
;   Germany
;
;	This work was done at:
;	----------------------
;   Institute of Solid State Physics
;   Advanced Materials Devision
;	Graz University of Technology
;	Petersgasse 16/I
;	A-8010 Graz
;	Austria
;
;	ingo.salzmann@physik.hu-berlin.de
;	roland.resel@tugraz.at
;	http://www.if.tugraz.at/amd/stereopole/index.html
;   Phone: +43(0)316/873-8461 or 8476
;
; ***********************************************************************************
;
; Copyright  2002-2005 by Ingo Salzmann
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;
; ***********************************************************************************
;
; 						LICENSE
;
;			GNU GENERAL PUBLIC LICENSE
;		       Version 2, June 1991
;
; Copyright (C) 1989, 1991 Free Software Foundation, Inc.
; 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
; Everyone is permitted to copy and distribute verbatim copies
; of this license document, but changing it is not allowed.
;
; ******* Preamble
;
;  The licenses for most software are designed to take away your
;freedom to share and change it.  By contrast, the GNU General Public
;License is intended to guarantee your freedom to share and change free
;software--to make sure the software is free for all its users.  This
;General Public License applies to most of the Free Software
;Foundation's software and to any other program whose authors commit to
;using it.  (Some other Free Software Foundation software is covered by
;the GNU Library General Public License instead.)  You can apply it to
;your programs, too.
;
;  When we speak of free software, we are referring to freedom, not
;price.  Our General Public Licenses are designed to make sure that you
;have the freedom to distribute copies of free software (and charge for
;this service if you wish), that you receive source code or can get it
;if you want it, that you can change the software or use pieces of it
;in new free programs; and that you know you can do these things.
;
;  To protect your rights, we need to make restrictions that forbid
;anyone to deny you these rights or to ask you to surrender the rights.
;These restrictions translate to certain responsibilities for you if you
;distribute copies of the software, or if you modify it.
;
;  For example, if you distribute copies of such a program, whether
;gratis or for a fee, you must give the recipients all the rights that
;you have.  You must make sure that they, too, receive or can get the
;source code.  And you must show them these terms so they know their
;rights.
;
;  We protect your rights with two steps: (1) copyright the software, and
;(2) offer you this license which gives you legal permission to copy,
;distribute and/or modify the software.
;
;  Also, for each author's protection and ours, we want to make certain
;that everyone understands that there is no warranty for this free
;software.  If the software is modified by someone else and passed on, we
;want its recipients to know that what they have is not the original, so
;that any problems introduced by others will not reflect on the original
;authors' reputations.
;
;  Finally, any free program is threatened constantly by software
;patents.  We wish to avoid the danger that redistributors of a free
;program will individually obtain patent licenses, in effect making the
;program proprietary.  To prevent this, we have made it clear that any
;patent must be licensed for everyone's free use or not licensed at all.
;
;  The precise terms and conditions for copying, distribution and
;modification follow.
;
; ******* GNU GENERAL PUBLIC LICENSE
; ******* TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
;
;  0. This License applies to any program or other work which contains
;a notice placed by the copyright holder saying it may be distributed
;under the terms of this General Public License.  The "Program", below,
;refers to any such program or work, and a "work based on the Program"
;means either the Program or any derivative work under copyright law:
;that is to say, a work containing the Program or a portion of it,
;either verbatim or with modifications and/or translated into another
;language.  (Hereinafter, translation is included without limitation in
;the term "modification".)  Each licensee is addressed as "you".
;
;Activities other than copying, distribution and modification are not
;covered by this License; they are outside its scope.  The act of
;running the Program is not restricted, and the output from the Program
;is covered only if its contents constitute a work based on the
;Program (independent of having been made by running the Program).
;Whether that is true depends on what the Program does.
;
;  1. You may copy and distribute verbatim copies of the Program's
;source code as you receive it, in any medium, provided that you
;conspicuously and appropriately publish on each copy an appropriate
;copyright notice and disclaimer of warranty; keep intact all the
;notices that refer to this License and to the absence of any warranty;
;and give any other recipients of the Program a copy of this License
;along with the Program.
;
;You may charge a fee for the physical act of transferring a copy, and
;you may at your option offer warranty protection in exchange for a fee.
;
;  2. You may modify your copy or copies of the Program or any portion
;of it, thus forming a work based on the Program, and copy and
;distribute such modifications or work under the terms of Section 1
;above, provided that you also meet all of these conditions:
;
;    a) You must cause the modified files to carry prominent notices
;    stating that you changed the files and the date of any change.
;
;    b) You must cause any work that you distribute or publish, that in
;    whole or in part contains or is derived from the Program or any
;    part thereof, to be licensed as a whole at no charge to all third
;    parties under the terms of this License.
;
;    c) If the modified program normally reads commands interactively
;    when run, you must cause it, when started running for such
;    interactive use in the most ordinary way, to print or display an
;    announcement including an appropriate copyright notice and a
;    notice that there is no warranty (or else, saying that you provide
;    a warranty) and that users may redistribute the program under
;    these conditions, and telling the user how to view a copy of this
;    License.  (Exception: if the Program itself is interactive but
;    does not normally print such an announcement, your work based on
;    the Program is not required to print an announcement.)
;
;These requirements apply to the modified work as a whole.  If
;identifiable sections of that work are not derived from the Program,
;and can be reasonably considered independent and separate works in
;themselves, then this License, and its terms, do not apply to those
;sections when you distribute them as separate works.  But when you
;distribute the same sections as part of a whole which is a work based
;on the Program, the distribution of the whole must be on the terms of
;this License, whose permissions for other licensees extend to the
;entire whole, and thus to each and every part regardless of who wrote it.
;
;Thus, it is not the intent of this section to claim rights or contest
;your rights to work written entirely by you; rather, the intent is to
;exercise the right to control the distribution of derivative or
;collective works based on the Program.
;
;In addition, mere aggregation of another work not based on the Program
;with the Program (or with a work based on the Program) on a volume of
;a storage or distribution medium does not bring the other work under
;the scope of this License.
;
;  3. You may copy and distribute the Program (or a work based on it,
;under Section 2) in object code or executable form under the terms of
;Sections 1 and 2 above provided that you also do one of the following:
;
;    a) Accompany it with the complete corresponding machine-readable
;    source code, which must be distributed under the terms of Sections
;    1 and 2 above on a medium customarily used for software interchange; or,
;
;    b) Accompany it with a written offer, valid for at least three
;    years, to give any third party, for a charge no more than your
;    cost of physically performing source distribution, a complete
;    machine-readable copy of the corresponding source code, to be
;    distributed under the terms of Sections 1 and 2 above on a medium
;    customarily used for software interchange; or,
;
;    c) Accompany it with the information you received as to the offer
;    to distribute corresponding source code.  (This alternative is
;    allowed only for noncommercial distribution and only if you
;    received the program in object code or executable form with such
;    an offer, in accord with Subsection b above.)
;
;The source code for a work means the preferred form of the work for
;making modifications to it.  For an executable work, complete source
;code means all the source code for all modules it contains, plus any
;associated interface definition files, plus the scripts used to
;control compilation and installation of the executable.  However, as a
;special exception, the source code distributed need not include
;anything that is normally distributed (in either source or binary
;form) with the major components (compiler, kernel, and so on) of the
;operating system on which the executable runs, unless that component
;itself accompanies the executable.
;
;If distribution of executable or object code is made by offering
;access to copy from a designated place, then offering equivalent
;access to copy the source code from the same place counts as
;distribution of the source code, even though third parties are not
;compelled to copy the source along with the object code.
;
;  4. You may not copy, modify, sublicense, or distribute the Program
;except as expressly provided under this License.  Any attempt
;otherwise to copy, modify, sublicense or distribute the Program is
;void, and will automatically terminate your rights under this License.
;However, parties who have received copies, or rights, from you under
;this License will not have their licenses terminated so long as such
;parties remain in full compliance.
;
;  5. You are not required to accept this License, since you have not
;signed it.  However, nothing else grants you permission to modify or
;distribute the Program or its derivative works.  These actions are
;prohibited by law if you do not accept this License.  Therefore, by
;modifying or distributing the Program (or any work based on the
;Program), you indicate your acceptance of this License to do so, and
;all its terms and conditions for copying, distributing or modifying
;the Program or works based on it.
;
;  6. Each time you redistribute the Program (or any work based on the
;Program), the recipient automatically receives a license from the
;original licensor to copy, distribute or modify the Program subject to
;these terms and conditions.  You may not impose any further
;restrictions on the recipients' exercise of the rights granted herein.
;You are not responsible for enforcing compliance by third parties to
;this License.
;
;  7. If, as a consequence of a court judgment or allegation of patent
;infringement or for any other reason (not limited to patent issues),
;conditions are imposed on you (whether by court order, agreement or
;otherwise) that contradict the conditions of this License, they do not
;excuse you from the conditions of this License.  If you cannot
;distribute so as to satisfy simultaneously your obligations under this
;License and any other pertinent obligations, then as a consequence you
;may not distribute the Program at all.  For example, if a patent
;license would not permit royalty-free redistribution of the Program by
;all those who receive copies directly or indirectly through you, then
;the only way you could satisfy both it and this License would be to
;refrain entirely from distribution of the Program.
;
;If any portion of this section is held invalid or unenforceable under
;any particular circumstance, the balance of the section is intended to
;apply and the section as a whole is intended to apply in other
;circumstances.
;
;It is not the purpose of this section to induce you to infringe any
;patents or other property right claims or to contest validity of any
;such claims; this section has the sole purpose of protecting the
;integrity of the free software distribution system, which is
;implemented by public license practices.  Many people have made
;generous contributions to the wide range of software distributed
;through that system in reliance on consistent application of that
;system; it is up to the author/donor to decide if he or she is willing
;to distribute software through any other system and a licensee cannot
;impose that choice.
;
;This section is intended to make thoroughly clear what is believed to
;be a consequence of the rest of this License.
;
;  8. If the distribution and/or use of the Program is restricted in
;certain countries either by patents or by copyrighted interfaces, the
;original copyright holder who places the Program under this License
;may add an explicit geographical distribution limitation excluding
;those countries, so that distribution is permitted only in or among
;countries not thus excluded.  In such case, this License incorporates
;the limitation as if written in the body of this License.
;
;  9. The Free Software Foundation may publish revised and/or new versions
;of the General Public License from time to time.  Such new versions will
;be similar in spirit to the present version, but may differ in detail to
;address new problems or concerns.
;
;Each version is given a distinguishing version number.  If the Program
;specifies a version number of this License which applies to it and "any
;later version", you have the option of following the terms and conditions
;either of that version or of any later version published by the Free
;Software Foundation.  If the Program does not specify a version number of
;this License, you may choose any version ever published by the Free Software
;Foundation.
;
;  10. If you wish to incorporate parts of the Program into other free
;programs whose distribution conditions are different, write to the author
;to ask for permission.  For software which is copyrighted by the Free
;Software Foundation, write to the Free Software Foundation; we sometimes
;make exceptions for this.  Our decision will be guided by the two goals
;of preserving the free status of all derivatives of our free software and
;of promoting the sharing and reuse of software generally.
;
; ******* NO WARRANTY
;
;  11. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
;FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
;OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
;PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
;OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
;MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
;TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
;PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
;REPAIR OR CORRECTION.
;
;  12. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
;WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
;REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
;INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
;OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
;TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
;YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
;PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
;POSSIBILITY OF SUCH DAMAGES.
;
; ***********************************************************************************
;
; IDL Standalone file...
;
; To create a new standalone IDL file:
; (1) Compile stereopole.pro
; (2) Type at the command line: Resolve_All
; (3) Type: Save, /Routines, Filename='stereopole.sav'
;
; *************************************** Events *********************************************

FUNCTION STARTUPFILE
IF !version.os_family eq 'unix' THEN device, true_color=24
window, /free, /pixmap, colors=-10
wdelete, !d.window
device, decomposed=0, retain=2
END

; *********************************** Read in RWA-Data ***************************************
Pro ReadDataFile_FirstPlot, event

;-> In order to use stereopole for your specific ASCII data files other than Philips .RWA
;-> you will have to alter this subroutine.

;-> Data from user value of top-level base (mainwindow) is copied to local variable info.
;-> User Value is now undefined (!) by this statement ...
widget_control, event.top, get_uvalue = info, /NO_COPY
thisPath=info.path
;-> Error handling for Data input

Catch, theError
IF theError NE 0 THEN BEGIN
	Catch, /Cancel
	errormsg = dialog_message('No Pole Figure loaded.', /ERROR)
	info.path=thisPath
	Widget_Control, event.top, Set_UValue=info, /No_Copy
	RETURN
ENDIF

;-> Read in file with standard IDL Procedure
file = Dialog_Pickfile(/Read, Filter="*.rwa", DIALOG_PARENT=event.top, PATH=info.path, GET_PATH=thePath)
*info.infilename = file
info.path = thePath ; Store current File Path for next data input

OpenR, rwa, file, /Get_Lun
Point_Lun, rwa, 0 ;Set file pointer on beginning

;-> File Info as string

header		= StrArr(20)
headerline	= '';StrArr(1)

;-> *.RWA files use to have 9 header lines in whic important information is to be read out
;-> Here is a sample header file:
;-> -----------------------------
;-> FileName, HH20-3.RWA, FileDate, 19-Oct-2002, FileTime,  5:10
;-> SampleName, HAL HIGHTEMP, HKL,  2 0 3
;-> TubeType, Cr, FocusType, LFF, Wavelength, 2.28970, GenkVmA,  40,  30
;-> ReceivingSlit, 2.00, 0.60, Geometry, Reflection
;-> InstrumentType, ATC3, TwoTheta, 34.610, Oscillation, 0
;-> PSI, 0.0, 75.0, 1.0
;-> PHI, 0.0, 360.0, 3.0
;-> TimePerStep, 7.00, NumberDataPoints,   9120
;-> ScanData
;-> -----------------------------

FOR j=1,9 DO BEGIN
	ReadF, rwa, headerline
	header[j]=headerline
ENDFOR ;-> File pointer now on final position for data read-in

;-> Generating variables out of header, if used for other data formats, you will have t
;-> alter this statements

twothetaarray = STRSPLIT(header[5], ',', /EXTRACT)
twotheta = twothetaarray[3]

psistringarray = STRSPLIT(header[6], ',', /EXTRACT)
psistart = psistringarray[1]
psiend = psistringarray[2]
psistep = psistringarray[3]

phistringarray = STRSPLIT(header[7], ',', /EXTRACT)
phistart = phistringarray[1]
phiend = phistringarray[2]
phistep = phistringarray[3]

numberarray = STRSPLIT(header[8], ',', /EXTRACT)
timeperstep = numberarray[1]
nop = numberarray[3]

twotheta	= FLOAT(twotheta)
psistart	= FLOAT(psistart)
psiend		= FLOAT(psiend)
psistep		= FLOAT(psistep)
phistart	= FLOAT(phistart)
phiend		= FLOAT(phiend)
phistep		= FLOAT(phistep)
timeperstep	= FLOAT(timeperstep)
nop			= FLOAT(nop) ; number of data points

;-> Read in data of file, where no problem occurs in IDL with linebreaks and commas
;
;-> Here are some sample data lines:
;-> --------------------------------
;-> 237, 216, 204, 216, 188, 210, 180, 169, 166, 177, 190, 172, 172, 169
;-> 154, 164, 161, 144, 142, 166, 146, 132, 121, 169, 154, 137, 164, 166
;-> 151, 161, 190, 177, 180, 180, 207, 202, 185, 193, 210, 185, 177, 216
;-> --------------------------------

intensity	= FltArr(nop)
ReadF, rwa, intensity

;-> Convert to 3x3 array (phi, psi, intensity)

daten	= FltArr(3,nop)
jend	= LONG((psiend-psistart)/psistep)+1
kend	= LONG((phiend-phistart)/phistep)-1
phi		= phistart
psi		= psistart
i		= 0

FOR j=1,jend DO BEGIN ;PSILOOP

	FOR k=0,kend DO BEGIN ;PHILOOP

;-> we have to add phistep/2 because the first point is stored at 1.5 when step 3 is chosen

	phi=phistart+phistep*k+phistep/2
	daten[*,i]=[phi,psi,intensity[i]]
	i=i+1

	ENDFOR

psi=psistart+psistep*j

ENDFOR

Free_Lun, rwa

;-> Copying values into local variable info

*info.header = header
*info.infilename = file
*info.psistart = psistart
*info.psiend = psiend
*info.psistep = psistep
*info.phistart = phistart
*info.phiend = phiend
*info.nop = nop
*info.phistep = phistep
*info.daten = daten
*info.datenreset = daten ;-> save initial data for reset without reloading data
*info.twotheta = twotheta
angle0 = info.nettoangle[0] ;-> store angle that possible previous pole figure has been rotated bevore

;-> User Value has to be defined again now, because info is a local variable ...
;-> before exiting this event handler module, the user value has to be set.

widget_control, event.top, set_uvalue = info, /NO_Copy

;-> Now prepare pole figure data for the first plot
PrepareData, event

widget_control, event.top, get_uvalue = info, /NO_Copy
*info.xmin = -MAX(*info.X)
*info.xmax = MAX(*info.X)
*info.ymin = -MAX(*info.Y)
*info.ymax = MAX(*info.Y)

widget_Control, info.minintID, set_value = *info.minint
widget_Control, info.maxintID, set_value = *info.maxint

IF (N_ELEMENTS(*info.header) ne 0) THEN BEGIN
	header = *info.header
	headerline1 = STRSPLIT(header[1], ',', /EXTRACT)
	headerline3 = STRSPLIT(header[2], ',', /EXTRACT)
		filename = headerline1[1]
		date = headerline1[3]
		SampleName = headerline3[1]
	label0 = string('Filename: ', strcompress(filename))
	label1 = string('File Date: ', strcompress(date))
	label3 = string('Sample Name: ', strcompress(SampleName))
	widget_control, (info.labelfile).labelfilemain, set_value = label0
	widget_control, (info.labelfile).labeldatemain, set_value = label1
	widget_control, (info.labelfile).labelnamemain, set_value = label3
ENDIF ELSE BEGIN
  	label3 = string('No pole figure file loaded.')
  	widget_control, (info.labelfile).labelname, set_value = label3
ENDELSE

;-> If crystal data is already set ... recalculate crystal [triggered by motion event in plotwindow]

IF ((N_ELEMENTS(*info.allindices1) GT 0) OR $
	(N_ELEMENTS(*info.allindices2) GT 0) OR $
	(N_ELEMENTS(*info.allindices3) GT 0) OR $
	(N_ELEMENTS(*info.allindices4) GT 0) OR $
	(N_ELEMENTS(*info.allindices5) GT 0)) THEN BEGIN
		info.updatetrigger = 3
		info.showbitmask[0] = 0
		widget_control, event.top, set_uvalue = info, /NO_Copy
		UpdateFromTableCleanup, event
ENDIF ELSE BEGIN

info.showbitmask[0] = 1
info.rwatrigger = 1 ;-> it is a Philips RWA file

;-> User Value is redefined again by this statement ...
widget_control, event.top, set_uvalue = info, /NO_COPY
ENDELSE

;-> Orientation of Pole Figure represents the labratory system
;-> Therefore a new Pole Figure should be rotated by the same angle as the
;-> previous has already been ...

IF angle0 NE 0 THEN BEGIN
	IF angle0 LT 0 THEN BEGIN
		RotatePoleFigure, event, /left
	ENDIF ELSE BEGIN
		RotatePoleFigure, event, /right
	ENDELSE
ENDIF
ShowMain, event
END

; *************************** Load data from ASCII File *********************************
PRO LoadASCII, event
widget_control, event.top, get_uvalue = info
thisPath=info.path

Catch, theError
IF theError NE 0 THEN BEGIN
	Catch, /Cancel
	errormsg = dialog_message('No Pole Figure loaded.', /ERROR)
	info.path=thisPath
	Widget_Control, event.top, Set_UValue=info, /No_Copy
	RETURN
ENDIF

;-> Read in ASCII file with standard IDL Procedure.
;-> Values can be INTEGER or FLOAT with "." as comma...
;-> Values must be equally spaced data sets ...
;-> The file has to be like this:
;-> phi [Tab or Blanks] psi [Tab or Blanks] intensity
;
;      1.50000     0.000000      110.000
;      4.50000     0.000000      125.000
;      7.50000     0.000000      104.000
;      10.5000     0.000000      123.000
;      13.5000     0.000000      102.000
;      16.5000     0.000000      114.000
;      19.5000     0.000000      128.000
;      22.5000     0.000000      128.000
;      25.5000     0.000000      117.000
;      28.5000     0.000000      139.000
;      31.5000     0.000000      98.0000
;      34.5000     0.000000      121.000
;      37.5000     0.000000      98.0000
;      40.5000     0.000000      102.000
; [...]
;      340.500     0.000000      177.000
;      343.500     0.000000      169.000
;      346.500     0.000000      149.000
;      349.500     0.000000      130.000
;      352.500     0.000000      146.000
;      355.500     0.000000      132.000
;      358.500     0.000000      125.000
;      1.50000      1.00000      130.000
;      4.50000      1.00000      135.000
;      7.50000      1.00000      146.000
;      10.5000      1.00000      149.000
;      13.5000      1.00000      156.000
;      16.5000      1.00000      128.000
;      19.5000      1.00000      135.000
;      22.5000      1.00000      139.000
;      25.5000      1.00000      132.000
;      28.5000      1.00000      139.000
;      31.5000      1.00000      130.000
;      34.5000      1.00000      144.000
;      37.5000      1.00000      164.000
;      40.5000      1.00000      123.000
; [...]

file = Dialog_Pickfile(/Read, Filter="*.*", DIALOG_PARENT=event.top, PATH=info.path, GET_PATH=thePath)
*info.infilename = file
info.path = thePath ; Store current File Path for next data input

OpenR, lun, file, /Get_Lun
Point_Lun, lun, 0 ;Set file pointer on beginning

;-> Length of file is unknown ... so read the first line

dummyline = FltArr(3)
dataline = FltArr(3)
ReadF, lun, dataline

;-> and then read on to the end of file

WHILE(eof(lun) NE 1) DO BEGIN
	ReadF, lun, dummyline
	dataline = [[dataline],[dummyline]]
ENDWHILE

*info.infilename = file
*info.psistart = dataline[1,0]
*info.psiend = dataline[1,N_ELEMENTS(dataline[0,*])-1]
*info.psistep = Abs(dataline[1,1]-dataline[1,0]-1)      ;-> Must be qually spaced in psi [!!!]
*info.phistart = dataline[0,0]
*info.phiend = dataline[0,N_ELEMENTS(dataline[0,*])-1]
*info.nop = N_ELEMENTS(dataline[0,*]-1)					;-> Must be qually spaced in phi [!!!]
*info.phistep = Abs(dataline[0,1]-dataline[0,0])
*info.daten = dataline
*info.datenreset = dataline ;-> save initial data for reset without reloading data
*info.twotheta = 50.0 ;-> You will have to set 2 Theta manually in Crystal Data setup

widget_control, event.top, set_uvalue = info, /NO_Copy

;-> Now prepare pole figure data for the first plot
PrepareData, event

widget_control, event.top, get_uvalue = info, /NO_Copy
*info.xmin = -MAX(*info.X)
*info.xmax = MAX(*info.X)
*info.ymin = -MAX(*info.Y)
*info.ymax = MAX(*info.Y)

;-> Set minimum and maximum intensity values in widget
widget_Control, info.minintID, set_value = *info.minint
widget_Control, info.maxintID, set_value = *info.maxint

widget_control, (info.labelfile).labelfilemain, set_value = '(No *.RWA file loaded)'
widget_control, (info.labelfile).labeldatemain, set_value = ''
widget_control, (info.labelfile).labelnamemain, set_value = ''

Free_Lun, lun
info.rwatrigger = 0 ;-> it is not a Philips RWA file
info.showbitmask[0] = 1 ;-> Show it!
angle0 = info.nettoangle[0]  ;-> store angle that possible previous pole figure has been rotated bevore

;-> If crystal data is already set ... recalculate crystal [triggered by motion event in plotwindow]
IF ((N_ELEMENTS(*info.allindices1) GT 0) OR $
	(N_ELEMENTS(*info.allindices2) GT 0) OR $
	(N_ELEMENTS(*info.allindices3) GT 0) OR $
	(N_ELEMENTS(*info.allindices4) GT 0) OR $
	(N_ELEMENTS(*info.allindices5) GT 0)) THEN BEGIN
		info.updatetrigger = 3
		widget_control, event.top, set_uvalue = info, /NO_Copy
		UpdateFromTableCleanup, event
ENDIF ELSE BEGIN

;-> User Value is redefined again by this statement ...
widget_control, event.top, set_uvalue = info, /NO_COPY
ENDELSE

;-> Orientation of Pole Figure represents the labratory system
;-> Therefore a new Pole Figure should be rotated by the same angle as the
;-> previous has already been ...

IF angle0 NE 0 THEN BEGIN
	IF angle0 LT 0 THEN BEGIN
		RotatePoleFigure, event, /left
	ENDIF ELSE BEGIN
		RotatePoleFigure, event, /right
	ENDELSE
ENDIF

;-> Redraw all
ShowMain, event

END

; -> ******************* Stereographic Projection **********************

FUNCTION Project_to_2d, x,y,z
xr2 = x / (1 + z)
yr2 = y / (1 + z)
a = [xr2, yr2]
RETURN, a
END

; -> ***********
FUNCTION Project_to_3d, x,y
RETURN,	[(2 * x) / (1 + x^2 + y^2),$
		 (2 * y) / (1 + x^2 + y^2),$
		 (1 - x^2 - y^2) / (1 + x^2 + y^2)]
END

; -> ********************** Derive a Unit Vector ************************
FUNCTION Unit_Vector, x

length = SQRT(x[0,*]^2+x[1,*]^2+x[2,*]^2)
rule = WHERE(length LE 0)
IF rule GT 0 THEN length[rule] = 1

FOR i=0,N_ELEMENTS(length)-1 DO BEGIN
	x[*,i] = x[*,i] / length[i]
ENDFOR

RETURN, x

END

; -> *************** Lowest Common Multiple in quality range ************
FUNCTION LCM, x,quality

x = Unit_Vector(x)

x_orig = x
i=1LL
tmp = ABS(x-ROUND(x))

WHILE ((tmp[0] GT quality) OR (tmp[1] GT quality) OR (tmp[2] GT quality)) DO BEGIN
i++
x=i*x_orig
tmp = ABS(x-ROUND(x))
ENDWHILE

RETURN, Round(x, /L64)

END

; -> ************* Devision by the Greatest Common Devisor of indices  ************

FUNCTION GCD_THREE, x

m = MAX(ABS(x))
p = PRIMES(100)

IF (m LT 2) THEN RETURN,x

p = p[WHERE(p LE m)]

REPEAT BEGIN
div = 1
	FOR i=0,N_ELEMENTS(p)-1 DO BEGIN
		rule = WHERE(TOTAL(ABS(x) MOD p[i],1) EQ 0.0)
		IF rule[0] NE (-1) THEN BEGIN
			div = p[i]
			x[*,rule] = x[*,rule]/div
		ENDIF
	ENDFOR
ENDREP UNTIL (div EQ 1)
RETURN, x
END

; **************************** Calculate (hkl) from (xyz) *****************************
FUNCTION CalculateBack, info, vector, quality, k

;-> info ... info structure
;-> vector ... [x,y,z] in orthogonal coordinates to be converted to corresponding (hkl) of crystal k
;-> k ... 0 to 5: current crystal

rtrans = (info.rec_rot_info).rtrans[*,*,k]		;-> Tensor of orthogonalization for reciprocal lattice
dtrans = (info.dir_rot_info).dtrans[*,*,k]		;-> Tensor of orthogonalization for direct lattice
rottensor = info.rottensor[*,*,k]				;-> Tensor of integrated rotation
rottensorhkl = info.rottensor_orth_hkl[*,*,k]	;-> Tensor of rotation after orthogonalization for reciprocal lattice
rottensoruvw = info.rottensor_orth_uvw[*,*,k]	;-> Tensor of rotation after orthogonalization for direct lattice

temp = (Invert(rottensor) # vector)
temp = (Invert(rottensorhkl) # temp) ;-> Rotate selected orthogonal axis back where it was after the transformation from the crystal system to the orthogonal system
hkl = Invert(rtrans) # temp 				;-> Transform selected orthogonal axis back to the crystal system

temp_1 = LCM(hkl,quality) 							;-> Convert it back to Miller Indices within a cerrtain quality range [10%]
temp_2 = GCD_THREE(temp_1)

RETURN, temp_2									;-> Return Miller Indices
END

; ******************* Dummy Plot for proper scaling of epitaxial plot *******************
FUNCTION DUMMYPLOT, xmin, xmax, ymin, ymax

d = [[-1,-1],[-1,1],[1,1],[1,-1]]
IF (!D.Name EQ 'PS') THEN BEGIN
plot, d[0,*], d[1,*], xstyle = 4, ystyle=4, xrange=[xmin, xmax], yrange=[ymin, ymax], /ISOTROPIC, color = 1
polyfill, d[0,*], d[1,*], color = 1
ENDIF ELSE BEGIN
	plot, d[0,*], d[1,*], xstyle = 4, ystyle=4, xrange=[xmin, xmax], yrange=[ymin, ymax], /ISOTROPIC
ENDELSE
ERASE
END

; **************************** Set colors for crystal 1 and 2 ***************************
FUNCTION SetColor, c ;-> Array of 5 RGB values [15 integer]

;-> Color definition in IDL is tricky ... defining a color with index i
;-> by TVLCT, r,g,b,i where r,g,b are the color indices of the desired
;-> color. To define say 4 colors [here for postscript output]:
;-> TVLCT, [r1,r2,r3,r4],[g1,g2,g3,g4],[b1,b2,b3,b4], !D.Table_size-3
;-> where the colors are being placed at the end of the color table ...

TVLCT, [c[0],c[3],c[6],c[9],c[12],0,255],[c[1],c[4],c[7],c[10],c[13],0,255],[c[2],c[5],c[8],c[11],c[14],0,255],!D.TABLE_SIZE-6
redColor = !D.TABLE_SIZE-6
blueColor = !D.TABLE_SIZE-5
greenColor = !D.TABLE_SIZE-4
orangeColor = !D.TABLE_SIZE-3
magentaColor = !D.TABLE_SIZE-2
blackColor = !D.TABLE_SIZE-1
whiteColor = !D.TABLE_SIZE
colorlist = [redColor, blueColor, greenColor, orangeColor, magentaColor, blackColor, whiteColor]
RETURN, colorlist
END

; ********************************* Drawing Circles **************************************

FUNCTION CIRCLE, xcenter, ycenter, radius
points = (2 * !PI / 999) * FINDGEN(1000)
x = xcenter + radius * COS(points)
y = ycenter + radius * SIN(points)

RETURN, TRANSPOSE([[x],[y]])

tempx = [0.0]
tempy = [0.0]

END

; ***************
FUNCTION AutoGenerateHKL, info

i = info.crystaltrigger - 1

dummy = 0

	FOR h = info.hmin[i],info.hmax[i] DO BEGIN
		FOR k = info.kmin[i],info.kmax[i] DO BEGIN
			FOR l = info.lmin[i],info.lmax[i] DO BEGIN
				IF (dummy EQ 0) THEN BEGIN
					dummy = 1
					allindices_local = [info.hmin[i],info.kmin[i],info.lmin[i],1,info.higherharm[i]]
				ENDIF ELSE BEGIN
					IF (NOT((h EQ 0) AND (k EQ 0) AND (l EQ 0)))$
					THEN allindices_local = [[allindices_local], [h,k,l,1,info.higherharm[i]]]
				ENDELSE
			ENDFOR
		ENDFOR
	ENDFOR

RETURN, allindices_local
END

; **************************** Create Indices **********************************
FUNCTION ManualGenerateHKL, info

j=1
i = info.crystaltrigger - 1
; IF (i EQ 0) THEN man = info.manualindex1 ELSE man = info.manualindex2 ; V1
man = info.manualindex[*,*,i] ; V2
;-> First line stored
	IF	((man[0,0] NE 0) OR (man[1,0] NE 0) OR (man[2,0] NE 0)) THEN BEGIN
		allindices = [man[0,0],man[1,0],man[2,0],man[3,0],man[4,0]]
	ENDIF ELSE BEGIN
		allindices = [info.middlehkl[*,i],1]
	ENDELSE

;-> Read in indices from table until (0,0,0) encountered

	WHILE ((man[0,j] NE 0) OR (man[1,j] NE 0) OR (man[2,j] NE 0)) DO BEGIN
		allindices = [[allindices], [man[0,j],man[1,j],man[2,j],man[3,j],man[4,j]]]
		j=j+1
	ENDWHILE

RETURN, allindices
END

; -> ***************** Delete "out of 2Theta Range" HKL *********************

FUNCTION ValidateHKLRange, info, temp

i = info.crystaltrigger - 1
allindices = temp[0:3,*] ; (h,k,l,foldness)
allharmonies = temp[4,*] ; (harmony)

IF (N_ELEMENTS(*info.twotheta) NE 0) THEN twotheta = *info.twotheta ELSE twotheta = 0

;-> If no 2T angle is set or in epitaxial mode, nothing has to be done...
IF ((N_ELEMENTS(*info.twotheta) EQ 0) OR (info.modetrigger EQ 1)) THEN BEGIN

;->	Discard higher multiples if desired
	IF info.apply_gcd EQ 1 THEN	temp[0:2,*] = GCD_THREE(temp[0:2,*])

	set_to_one = INTARR(1,N_ELEMENTS(temp[0,*]))+1
	IndicesStruct = {IndicesInRange:temp[0:3,*], harmony:set_to_one, all_twotheta:set_to_one}
	RETURN, IndicesStruct
ENDIF

;-> If twotheta is set to zero, no validation is desired ... set it to save borders
IF ((N_ELEMENTS(*info.twotheta) NE 0) AND (twotheta EQ 0)) THEN BEGIN
	twothetha_low = 1
	twothetha_high = 175

ENDIF ELSE BEGIN
	;-> d of lower and higher bound
	;-> twotheta has to be between 0 and 180, say 175 for d to be positive
	twothetarange = info.twothetarange[i]

	twothetha_low = *info.twotheta - twothetarange * 0.5
	twothetha_high = twothetarange * 0.5 + *info.twotheta
	IF ((*info.twotheta - twothetarange * 0.5) LE 0) THEN  twothetha_low = 1
	IF ((twothetarange * 0.5 + *info.twotheta) GE 175) THEN  twothetha_high = 175

ENDELSE

a = info.a[i]
b = info.b[i]
c = info.c[i]
alpha = info.alpha[i] * !DPI/180
beta = info.beta[i] * !DPI/180
gamma = info.gamma[i] * !DPI/180

;-> Reciprocal Lattice - angles

alpharec = ACos((Cos(beta) * Cos(gamma) -  Cos(alpha)) / (Sin(beta) * Sin(gamma)))
betarec  = ACos((Cos(gamma) * Cos(alpha) -  Cos(beta)) / (Sin(gamma) * Sin(alpha)))
gammarec = ACos((Cos(alpha) * Cos(beta) -  Cos(gamma)) / (Sin(alpha) * Sin(beta)))

;-> Reciprocal Lattice - unit cell params

arec = 1 / (a * Sin(beta) * Sin(gammarec))
brec = 1 / (b * Sin(gamma) * Sin(alpharec))
crec = 1 / (c * Sin(alpha) * Sin(betarec))

;-> Metric Tensor is used for calculating interplanar distances
recmetric =	TRANSPOSE([[arec^2, arec * brec * Cos(gammarec), arec * crec * Cos(betarec)],$
			[arec * brec * Cos(gammarec), brec^2, brec * crec * Cos(alpharec)],$
			[arec * crec * Cos(betarec), brec * crec * Cos(alpharec), crec^2]])

info.recmetric[*,*,i] = recmetric

k=0
IndicesInRange = [0,0,0,1]
harmony = 1

size_of_temp = SIZE(temp) ;-> avoid problems if only a single index is comming fom temp ...

IF size_of_temp[0] EQ 1 THEN BEGIN
	temp = [[temp],[temp]]
ENDIF

temp_higher_harm = WHERE(temp(4,*) GT 1) ;-> Row indices of (hkl) with higher harmonics

in_range = temp	;-> Stays unchanged if there is no higher harmonic

IF temp_higher_harm[0] NE (-1) THEN BEGIN
	in_range_higherharm = temp(*,temp_higher_harm) 	;-> Array subscript of (hkl) with higher harmonics
	in_range[4,*] = 1									;-> Set them to one and add higher harmonics to the end of the list
	FOR j = 0,N_ELEMENTS(in_range_higherharm[0,*])-1 DO BEGIN
		FOR k = 2,MAX(in_range_higherharm(4,j)) DO BEGIN
			temp_1 = in_range_higherharm[*,j]
			temp_1[4] = k
			in_range = [[in_range], [temp_1]]		;-> in_range now consists of all indices of any harmonic
		ENDFOR
	ENDFOR
ENDIF

temp_1 = (recmetric # in_range[0:2,*]) * in_range[0:2,*]
temp_2 = info.wavelength/(2*SQRT(TOTAL(temp_1,1)^(-1)))
temp_2 = temp_2 / in_range[4,*]

temp_3 = WHERE(temp_2 LT 0.9999)	;-> Argument for the calculation of 2Theta has to be less that one (ASIN)

IF temp_3[0] EQ (-1) THEN BEGIN		;-> If no element of temp_2 is LT 0.9999 then create dummy (hkl) and trigger temp_4 to -1
	IndicesInRange = [0,0,0,1]
	harmony = 1
	all_twotheta = 0
	temp_4 = -1
ENDIF ELSE BEGIN
	in_range = in_range[*,temp_3]					;-> Those that are ok for Asin --> means LT 0.9999 ...
	all_twotheta = 2*Asin(temp_2[temp_3])*!RADEG 	;-> Are used for 2Theta calculation ...
	temp_4 = WHERE((all_twotheta LE twothetha_high) AND (all_twotheta GE twothetha_low))  ;-> In range: (h,k,l, foldness, harmony)
ENDELSE

IF temp_4[0] EQ (-1) THEN BEGIN
	IndicesInRange = [0,0,0,1]
	harmony = 1
	all_twotheta = 0
ENDIF ELSE BEGIN
	IndicesInRange = in_range[0:3,temp_4]
	harmony = in_range[4,temp_4]
	all_twotheta = all_twotheta[temp_4]
ENDELSE

;-> If none in range ... show middle-hkl in  hkl-list ...

IF (TOTAL(IndicesInRange[*,0] NE [0,0,0,1]) EQ 0.0) THEN BEGIN
	IndicesInRange = [info.middlehkl[*,i],1]
	IF info.wavelength/(2 * SQRT(TOTAL((recmetric # IndicesInRange[0:2,0]) * IndicesInRange[0:2,0])^(-1))) LT 0.9999 THEN BEGIN
		all_twotheta = 2*Asin(info.wavelength/(2 * SQRT(TOTAL((recmetric # IndicesInRange[0:2,0]) * IndicesInRange[0:2,0])^(-1)) ))*!RADEG
	ENDIF ELSE BEGIN
		all_twotheta = 'Out of Range'
	ENDELSE
ENDIF

;-> Discard higher multiples if desired
IF info.apply_gcd EQ 1 THEN	IndicesInRange[0:2,*] = GCD_THREE(IndicesInRange[0:2,*])

IndicesStruct = {IndicesInRange:IndicesInRange, harmony:harmony, all_twotheta:all_twotheta} ;-> Harmony and foldness have to be calculated later on

RETURN, IndicesStruct
END

; -> ***************** Rotation tensors as functions *********************

FUNCTION XROT, a
tensor = [[1, 0, 0],[0, Cos(a), - Sin(a)],[0, Sin(a), Cos(a)]]
RETURN, TRANSPOSE(tensor)
END
; -> ***********
FUNCTION YROT, a
tensor = [[Cos(a), 0, - Sin(a)],[0, 1, 0],[Sin(a), 0, Cos(a)]]
RETURN, TRANSPOSE(tensor)
END
; -> ***********
FUNCTION ZROT, a
tensor = [[Cos(a), - Sin(a), 0],[Sin(a), - Cos(a), 0],[0, 0, 1]]
RETURN, TRANSPOSE(tensor)
END
; -> ***********
; -> Rotation about an arbitrary axis (x,y,z) with unit length

FUNCTION General_ROT, x, y, z, a
s = Sin(a)
c = Cos(a)
t = (1 - Cos(a))
tensor =	[[t * x^2 + c, t * x * y + s * z, t * x * z - s * y],$
			[t * x * y - s * z, t * y^2 + c, t * y * z + s * x],$
			[t * x * z + s * y, t * y * z - s * x, t * z^2 + c]]
RETURN, TRANSPOSE(tensor)
END


; ********************** Calculate angle between two (hkl) **************************
FUNCTION InterplanarAngle, info, temp, i

a = info.a[i]
b = info.b[i]
c = info.c[i]
alpha = info.alpha[i] * !DPI/180
beta = info.beta[i] * !DPI/180
gamma = info.gamma[i] * !DPI/180

;-> Reciprocal Lattice - angles

alpharec = ACos((Cos(beta) * Cos(gamma) -  Cos(alpha)) / (Sin(beta) * Sin(gamma)))
betarec  = ACos((Cos(gamma) * Cos(alpha) -  Cos(beta)) / (Sin(gamma) * Sin(alpha)))
gammarec = ACos((Cos(alpha) * Cos(beta) -  Cos(gamma)) / (Sin(alpha) * Sin(beta)))

;-> Reciprocal Lattice - unit cell params

arec = 1 / (a * Sin(beta) * Sin(gammarec))
brec = 1 / (b * Sin(gamma) * Sin(alpharec))
crec = 1 / (c * Sin(alpha) * Sin(betarec))

;-> Metric Tensor is used for calculating interplanar distances
recmetric =	([[arec^2, arec * brec * Cos(gammarec), arec * crec * Cos(betarec)],$
			[arec * brec * Cos(gammarec), brec^2, brec * crec * Cos(alpharec)],$
			[arec * crec * Cos(betarec), brec * crec * Cos(alpharec), crec^2]])

;-> Calculate it

hkl1 = temp[*,0]
hkl2 = temp[*,1]

innerproduct = TOTAL(hkl1 * (recmetric # hkl2))
len3 = Sqrt(TOTAL((hkl1 # recmetric) * hkl1))
len4 = Sqrt(TOTAL((hkl2 # recmetric) * hkl2))
angle = ACos((innerproduct)/(len3 * len4))*180/!Pi

RETURN, angle

END

; **************************** Build Crystal Vectors 3D *****************************
FUNCTION BuildVectors3D, info

j = info.crystaltrigger - 1

a = DOUBLE(info.a[j])
b = DOUBLE(info.b[j])
c = DOUBLE(info.c[j])
alpha = DOUBLE(info.alpha[j]) * !DPI/180
beta = DOUBLE(info.beta[j]) * !DPI/180
gamma = DOUBLE(info.gamma[j]) * !DPI/180

CASE info.crystaltrigger OF
	1:	BEGIN
			allindices = *info.allindices1
			allharmonies = *info.allharmonies1
			alltwotheta = *info.alltwotheta1
		END
	2:	BEGIN
			allindices = *info.allindices2
			allharmonies = *info.allharmonies2
			alltwotheta = *info.alltwotheta2
		END
	3:	BEGIN
			allindices = *info.allindices3
			allharmonies = *info.allharmonies3
			alltwotheta = *info.alltwotheta3
		END
	4:	BEGIN
			allindices = *info.allindices4
			allharmonies = *info.allharmonies4
			alltwotheta = *info.alltwotheta4
		END
	5:	BEGIN
			allindices = *info.allindices5
			allharmonies = *info.allharmonies5
			alltwotheta = *info.alltwotheta5
		END
ENDCASE

middleuvw = info.middleuvw[*,j]
middlehkl = info.middlehkl[*,j]

;-> Reciprocal Lattice - angles

alpharec = ACos((Cos(beta) * Cos(gamma) -  Cos(alpha)) / (Sin(beta) * Sin(gamma)))
betarec  = ACos((Cos(gamma) * Cos(alpha) -  Cos(beta)) / (Sin(gamma) * Sin(alpha)))
gammarec = ACos((Cos(alpha) * Cos(beta) -  Cos(gamma)) / (Sin(alpha) * Sin(beta)))

;-> Reciprocal Lattice - unit cell params
;-> Could also be calculated by inverting the metric tensor

arec = 1 / (a * Sin(beta) * Sin(gammarec))
brec = 1 / (b * Sin(gamma) * Sin(alpharec))
crec = 1 / (c * Sin(alpha) * Sin(betarec))

;-> Transformation Matrix for real space to orthogonal coordinates

;-> Caution! IDL uses (column, row) ordering convention ...
;-> That means: A(3,5) # B(5,2) is a proper matrix multiplication ...
;-> Due to its origin in image processing, IDL always displays the
;-> transpose of a matrix or vector ...

dtrans = TRANSPOSE([$
			[a * Sin(beta) * Sin(gammarec), 0, 0],$
			[- a * Sin(beta) * Cos(gammarec), b * Sin(alpha), 0],$
			[a * Cos(beta), b * Cos(alpha), c]])

;-> Transformation Matrix for reciprocal space to orthogonal coordinates

rtrans = TRANSPOSE([$
			[arec, brec * Cos(gammarec), crec * Cos(betarec)],$
			[0, brec * Sin(gammarec), - crec * Sin(betarec) * Cos(alpha)],$
			[0, 0, crec * Sin(betarec) * Sin(alpha)]])

;-> Calculate 3D Coordinates of the relevant vectors in an orthogonal system
;-> for direct and rec. space (they are ROW vectors with 3 COLUMNS!)
;
;-> At this point everything is prepared for further extention of the program,
;-> i.e. orientating the epitaxial plot according to an orientation [uvw] and not
;-> according to an orientation (hkl). For this, the following two lines have
;-> to be uncommented (also see: (*),(**))

;middleuvw3D = dtrans # middleuvw
;	middleuvw3D = Unit_Vector(middleuvw3D)

middlehkl3D = rtrans # middlehkl
	middlehkl3D = Unit_Vector(middlehkl3D)

	middleuvw3D = middlehkl3D ; (*) comment this in the case expained above

;-> Rotating the desired orientational (hkl) or (uvw) 3D vector to match
;-> with (0,0,1) and therefore with (0,0) of the stereographic projection

;-> ***** direct space (**) the
axisuvw = CROSSP([0,0,1], middleuvw3D) ;rotaxis
axisuvw = Unit_Vector(axisuvw)
angleuvw = ACOS(TOTAL([0,0,1] * middleuvw3D))
rottensoruvw = General_Rot(axisuvw[0],axisuvw[1],axisuvw[2],angleuvw)
info.rottensor_orth_uvw[*,*,j] = rottensoruvw 	;-> Store tensor of rotation (middleuvw || (001)ortho )
info.dir_rot_info.dtrans[*,*,j] = dtrans		;-> Store tensor of direct orthogonalization for further use

;-> ***** reciprocal space
axishkl = CROSSP([0,0,1], middlehkl3D) ;rotaxis
axishkl = Unit_Vector(axishkl)
anglehkl = ACOS(TOTAL([0,0,1] * middlehkl3D))
rottensorhkl = General_Rot(axishkl[0],axishkl[1],axishkl[2],anglehkl)
info.rottensor_orth_hkl[*,*,j] = rottensorhkl 	;-> Store tensor of rotation (middlehkl || (001)ortho )
info.rec_rot_info.rtrans[*,*,j] = rtrans		;-> Store tensor of reciprocal orthogonalization for further use

;-> ALLINDICES is an INT Array[3, n] ...
;-> in each row of dirvec3d and recvec3d there is one (x,y,z) so that
;-> in (col, row) convention (*,0) yields the first, (*,1) the second point...
;-> All vectors are rotated in the same way as the orientational vectors were ...

dirvec3d = Unit_Vector(dtrans # allindices[0:2,*])
dirvec3d = (rottensoruvw # dirvec3d)
recvec3d = Unit_Vector(rtrans # allindices[0:2,*])
recvec3d = (rottensorhkl # recvec3d)

;-> If there is a wish for foldness >1 [only in pole figure mode] ...
;-> If the foldness of a reflex is set to be greater than 1, which can only
;-> occur unless automatic mode was chosen, allindices[*,3] represents this
;-> foldness. Then souch an index has to be rotated around orthogonal [0,0,1]
;-> by an angle of 360/foldness degree. The resulting vectors are appended to
;-> the two arrays dirvec3d, recvec3d
;-> This is also done for the vectors of direct space for the resulting arrays to
;-> be eqal sized.

;->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> only possible in manual input mode ...

IF (info.modetrigger EQ 0) THEN BEGIN

	dummy = intarr(N_ELEMENTS(allindices[1,*]))+1
	IF (ARRAY_EQUAL(dummy, allindices[3,*]) NE 1) THEN BEGIN

		orig_dim = N_ELEMENTS(allindices[0,*])
		rule = WHERE(allindices[3,*] GT 1)

		to_be_rotated = allindices[*,rule]
		to_be_rotated_harmony = allharmonies[*,rule]
		to_be_rotated_twotheta = alltwotheta[rule]
		to_be_rotated_dir = dirvec3d[*,rule]
		to_be_rotated_rec = recvec3d[*,rule]

		angles = DBLARR(N_ELEMENTS(rule))
		angles = 360.0D / to_be_rotated[3,*]

		number_to_be_added = TOTAL(to_be_rotated[3,*]) - N_ELEMENTS(to_be_rotated[3,*])
		result_array_indices = INTARR(4,number_to_be_added + orig_dim)
		result_array_harmony = INTARR(1,number_to_be_added + orig_dim)
		result_array_twotheta = DBLARR(number_to_be_added + orig_dim)
		result_array_dir = DBLARR(3,number_to_be_added + orig_dim)
		result_array_rec = DBLARR(3,number_to_be_added + orig_dim)

		result_array_indices[*,0:orig_dim-1] = allindices
		IF (SIZE(to_be_rotated_harmony))[0] EQ 1 THEN result_array_harmony[0,0] = allharmonies ELSE result_array_harmony[0,0:orig_dim-1] = allharmonies
		result_array_twotheta[0:orig_dim-1]  = alltwotheta[0:orig_dim-1] ; >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> TWOTHETA WRONG FOR LIST
		result_array_dir[*,0:orig_dim-1] = dirvec3d
		result_array_rec[*,0:orig_dim-1] = recvec3d

		line = orig_dim

		FOR i=0,N_ELEMENTS(to_be_rotated[0,*])-1 DO BEGIN
			FOR k=1,to_be_rotated[3,i]-1 DO BEGIN
				rottensor = General_ROT(0, 0, 1, k * angles[i] * !DTOR)

				result_array_dir[*,line] = rottensor # Unit_Vector(to_be_rotated_dir[*,i])
				result_array_rec[*,line] = rottensor # Unit_Vector(to_be_rotated_rec[*,i])

				result_array_indices[*,line] = to_be_rotated[*,i]
				result_array_harmony[0,line] = to_be_rotated_harmony[*,i]
				result_array_twotheta[0:line]  = to_be_rotated_twotheta[i]

				line ++
			ENDFOR
		ENDFOR

		dirvec3d = result_array_dir
		recvec3d = result_array_rec
		allindices = result_array_indices
		allharmonies = result_array_harmony
		alltwotheta = result_array_twotheta

	ENDIF
ENDIF

;-> At this point the information about foldness can be discarded ...
allindices = allindices[0:2,*]

;-> Do different things for the two different crystals ...

CASE info.crystaltrigger OF
	1:	BEGIN
			*info.allindices1 = allindices
			*info.allharmonies1 = allharmonies
			*info.alltwotheta1 = alltwotheta
		END
	2:	BEGIN
			*info.allindices2 = allindices
			*info.allharmonies2 = allharmonies
			*info.alltwotheta2 = alltwotheta
		END
	3:	BEGIN
			*info.allindices3 = allindices
			*info.allharmonies3 = allharmonies
			*info.alltwotheta3 = alltwotheta
		END
	4:	BEGIN
			*info.allindices4 = allindices
			*info.allharmonies4 = allharmonies
			*info.alltwotheta4 = alltwotheta
		END
	5:	BEGIN
			*info.allindices5 = allindices
			*info.allharmonies5 = allharmonies
			*info.alltwotheta5 = alltwotheta
		END
ENDCASE

Vectors3D = {dirvec3d:dirvec3d, recvec3d:recvec3d, allindices:allindices}

RETURN, Vectors3D
END

; **************************** Build Crystal Vectors 2D *****************************
FUNCTION BuildVectors2D, info

j = info.crystaltrigger - 1

;-> Now the stereographic projection has to be performed ...
;-> Bad things can happen there with vectors very close to (0,0,1)
;-> which lead to 'Floating divide by 0' errors ...'

buildhkl = 0

CASE info.crystaltrigger OF
	1:	BEGIN
			recvec3D = (*info.Vectors3D_1).recvec3d
			dirvec3D = (*info.Vectors3D_1).dirvec3d
			allindices = (*info.Vectors3D_1).allindices
			harmony = *info.allharmonies1
			alltwotheta = *info.alltwotheta1
		END
	2:	BEGIN
			recvec3D = (*info.Vectors3D_2).recvec3d
			dirvec3D = (*info.Vectors3D_2).dirvec3d
			allindices = (*info.Vectors3D_2).allindices
			harmony = *info.allharmonies2
			alltwotheta = *info.alltwotheta2
		END
	3:	BEGIN
			recvec3D = (*info.Vectors3D_3).recvec3d
			dirvec3D = (*info.Vectors3D_3).dirvec3d
			allindices = (*info.Vectors3D_3).allindices
			harmony = *info.allharmonies3
			alltwotheta = *info.alltwotheta3
		END
	4:	BEGIN
			recvec3D = (*info.Vectors3D_4).recvec3d
			dirvec3D = (*info.Vectors3D_4).dirvec3d
			allindices = (*info.Vectors3D_4).allindices
			harmony = *info.allharmonies4
			alltwotheta = *info.alltwotheta4
		END
	5:	BEGIN
			recvec3D = (*info.Vectors3D_5).recvec3d
			dirvec3D = (*info.Vectors3D_5).dirvec3d
			allindices = (*info.Vectors3D_5).allindices
			harmony = *info.allharmonies5
			alltwotheta = *info.alltwotheta5
		END
ENDCASE

; VECTORS

savelimit = -0.9999 ;-> avoid devision by zero

rule_dir = WHERE(dirvec3d[2,*] GT savelimit, COMPLEMENT = set_it_dir)
rule_rec = WHERE(recvec3d[2,*] GT savelimit, COMPLEMENT = set_it_rec)

IF set_it_dir[0] GT -1 THEN dirvec3d[2, set_it_dir] = savelimit
IF set_it_rec[0] GT -1 THEN recvec3d[2, set_it_rec] = savelimit

dirvec2d = Project_to_2d(dirvec3d[0,*], dirvec3d[1,*], dirvec3d[2,*])
recvec2d = Project_to_2d(recvec3d[0,*], recvec3d[1,*], recvec3d[2,*])

temp_dir = dirvec3d
temp_rec = recvec3d

; INDICES

;temp_dir_indices = allindices[*,rule_dir]
;temp_rec_indices = allindices[*,rule_rec]
; *********************************************************** !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

temp_dir_indices = allindices
temp_rec_indices = allindices


;temp_allindices_dir = string(temp_dir_indices)
;temp_allindices_rec = string(temp_rec_indices)

temp_allindices_dir = string(allindices)
temp_allindices_rec = string(allindices)

hstr = STRCOMPRESS(temp_allindices_dir[0,*], /REMOVE_ALL)
kstr = STRCOMPRESS(temp_allindices_dir[1,*], /REMOVE_ALL)
lstr = STRCOMPRESS(temp_allindices_dir[2,*], /REMOVE_ALL)
labelstring_dir = hstr + ' ' + kstr + ' ' + lstr

hstr = STRCOMPRESS(temp_allindices_rec[0,*], /REMOVE_ALL)
kstr = STRCOMPRESS(temp_allindices_rec[1,*], /REMOVE_ALL)
lstr = STRCOMPRESS(temp_allindices_rec[2,*], /REMOVE_ALL)
labelstring_rec = hstr + ' ' + kstr + ' ' + lstr

; HARMONY

; *********************************************************** !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;harmony = harmony[*,rule_rec]
;alltwotheta = alltwotheta[rule_rec]
harmonicstring = StrArr(N_ELEMENTS(harmony))

;-> Store reduced set of harmonies for list generation in PRO HKL_List
;-> Store reduced set of two theta angles for list generation in PRO HKL_List
;-> Store reduced set of indices for list generation in PRO HKL_List

CASE info.crystaltrigger OF
	1: 	BEGIN
		*info.allharmonies1 = harmony
		*info.alltwotheta1 = alltwotheta
		*info.allindices1 = allindices
		*info.Vectors3D_1 = {dirvec3d:dirvec3d, recvec3d:recvec3d, allindices:allindices}
		END
	2: 	BEGIN
		*info.allharmonies2 = harmony
		*info.alltwotheta2 = alltwotheta
		*info.allindices2 = allindices
		*info.Vectors3D_2 = {dirvec3d:dirvec3d, recvec3d:recvec3d, allindices:allindices}
		END
	3: 	BEGIN
		*info.allharmonies3 = harmony
		*info.alltwotheta3 = alltwotheta
		*info.allindices3 = allindices
		*info.Vectors3D_3 = {dirvec3d:dirvec3d, recvec3d:recvec3d, allindices:allindices}
		END
	4: 	BEGIN
		*info.allharmonies4 = harmony
		*info.alltwotheta4 = alltwotheta
		*info.allindices4 = allindices
		*info.Vectors3D_4 = {dirvec3d:dirvec3d, recvec3d:recvec3d, allindices:allindices}
		END
	5: 	BEGIN
		*info.allharmonies5 = harmony
		*info.alltwotheta5 = alltwotheta
		*info.allindices5 = allindices
		*info.Vectors3D_5 = {dirvec3d:dirvec3d, recvec3d:recvec3d, allindices:allindices}
		END
ELSE:
ENDCASE

FOR i = 0, N_ELEMENTS(harmony)-1 DO BEGIN
	IF (harmony[i] GT 1) THEN harmonicstring[i] = '[' + STRCOMPRESS(STRING(harmony[i]), /REMOVE_ALL) + ']'
ENDFOR

Vectors2D = {labelstring_dir:labelstring_dir, labelstring_rec:labelstring_rec, harmonicstring:harmonicstring, dirvec2d:dirvec2d, recvec2d:recvec2d}

RETURN, Vectors2D
END

; **************************** Rotate Stereographic projections *******************************
PRO RotateStereo, event, left=left, right=right, recalculated=recalculated
widget_control, event.top, get_uvalue = info, /NO_Copy
widget_control, hourglass = 1

;-> Preparation of parameters

IF KEYWORD_SET(left) THEN rotstepsize = -info.rotstepsize * !DTOR
IF KEYWORD_SET(right) THEN rotstepsize = info.rotstepsize * !DTOR

rotatebitmask = info.rotatebitmask

axis = info.rotaxis_xyz

FOR k=1,5 DO BEGIN

	CASE k OF
		1: 	BEGIN
			current = info.allindices1
				IF (N_Elements(*current) GT 0) THEN BEGIN
				dirvec3d = (*info.Vectors3D_1).dirvec3d
				recvec3d = (*info.Vectors3D_1).recvec3d
				END
			END
		2: 	BEGIN
			current = info.allindices2
				IF (N_Elements(*current) GT 0) THEN BEGIN
				dirvec3d = (*info.Vectors3D_2).dirvec3d
				recvec3d = (*info.Vectors3D_2).recvec3d
				END
			END
		3: 	BEGIN
			current = info.allindices3
				IF (N_Elements(*current) GT 0) THEN BEGIN
				dirvec3d = (*info.Vectors3D_3).dirvec3d
				recvec3d = (*info.Vectors3D_3).recvec3d
				END
			END
		4: 	BEGIN
			current = info.allindices4
				IF (N_Elements(*current) GT 0) THEN BEGIN
				dirvec3d = (*info.Vectors3D_4).dirvec3d
				recvec3d = (*info.Vectors3D_4).recvec3d
				END
			END
		5: 	BEGIN
			current = info.allindices5
				IF (N_Elements(*current) GT 0) THEN BEGIN
				dirvec3d = (*info.Vectors3D_5).dirvec3d
				recvec3d = (*info.Vectors3D_5).recvec3d
				END
			END
	ENDCASE

IF ((rotatebitmask[k] EQ 1) AND (N_Elements(*current) GT 0)) THEN BEGIN

	IF KEYWORD_SET(recalculated) THEN BEGIN ;-> Use stored tensor for rotation after recalculation of crystal
		rottensor = info.rottensor[*,*,k-1]
	ENDIF ELSE BEGIN
		rottensor = General_ROT(axis[0], axis[1], axis[2], rotstepsize)
		info.rottensor[*,*,k-1] = rottensor # info.rottensor[*,*,k-1] ;-> Generate the new rotation tensor that is being stored
	ENDELSE

	allindices = *current ; useful for loops

;->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> SLOW

	FOR i = 0, N_ELEMENTS(allindices[1,*])-1 DO BEGIN
		IF (i EQ 0) THEN BEGIN
			tempdir = rottensor # Unit_Vector(dirvec3d[*,0])
			temprec = rottensor # Unit_Vector(recvec3d[*,0])
		ENDIF ELSE BEGIN
			temp1 = rottensor # Unit_Vector(dirvec3d[*,i])
			temp2 = rottensor # Unit_Vector(recvec3d[*,i])
			tempdir = [[tempdir], [temp1]]
			temprec = [[temprec], [temp2]]
		ENDELSE
	ENDFOR

; and store it all

CASE k OF
		1: 	BEGIN
			(*info.Vectors3D_1).dirvec3d = tempdir
			(*info.Vectors3D_1).recvec3d = temprec
			info.crystaltrigger = 1 						;-> info.crystaltrigger has to be set to current crystal ... here it's 1
			*info.Vectors2D_1 = BuildVectors2D(info)
			END
		2: 	BEGIN
			(*info.Vectors3D_2).dirvec3d = tempdir
			(*info.Vectors3D_2).recvec3d = temprec
			info.crystaltrigger = 2
			*info.Vectors2D_2 = BuildVectors2D(info)
			END
		3: 	BEGIN
			(*info.Vectors3D_3).dirvec3d = tempdir
			(*info.Vectors3D_3).recvec3d = temprec
			info.crystaltrigger = 3
			*info.Vectors2D_3 = BuildVectors2D(info)
			END
		4: 	BEGIN
			(*info.Vectors3D_4).dirvec3d = tempdir
			(*info.Vectors3D_4).recvec3d = temprec
			info.crystaltrigger = 4
			*info.Vectors2D_4 = BuildVectors2D(info)
			END
		5: 	BEGIN
			(*info.Vectors3D_5).dirvec3d = tempdir
			(*info.Vectors3D_5).recvec3d = temprec
			info.crystaltrigger = 5
			*info.Vectors2D_5 = BuildVectors2D(info)
			END
	ENDCASE
ENDIF
ENDFOR

widget_control, event.top, set_uvalue = info, /NO_Copy

END

; ******************************** Show selected Figures **************************************
PRO ShowMain, event, zoomin = zoomin
widget_control, event.top, get_uvalue = info, /NO_Copy

info.updatetrigger = 0
widget_control, hourglass = 1

colorlist = *info.colorlist

thisDevice=!D.Name
 IF (N_Elements(*info.xmin) NE 0) THEN BEGIN
xmin = *info.xmin
xmax = *info.xmax
ymin = *info.ymin
ymax = *info.ymax
delta = 1.01 ;-> more range in case of postscript output for that labels aren't clipped
ENDIF ELSE BEGIN
xmin = -1
xmax = 1
ymin = -1
ymax = 1
ENDELSE

;-> In case of PostScript output, ISOTROPIC output has to be generated by an invisible plot ***** **** *** ** *
IF (!D.Name EQ 'PS') THEN BEGIN
	IF (info.modetrigger EQ 1) THEN BEGIN
		; **** Draw Background for PostScript Output
		d = [[xmin,ymin],[xmin,ymax],[xmax,ymax],[xmax,ymin]]
		plot, d[0,*], d[1,*], xstyle = 4, ystyle=4, xrange=[xmin*delta, xmax*delta], yrange=[ymin*delta, ymax*delta],$
			/ISOTROPIC, /NOERASE, /NODATA, color = colorlist[2]
	ENDIF

ENDIF ELSE BEGIN
	;-> First perform all plots on the invisible Pixmap window but
	;-> don't enable double buffering in case of postscript output ...
	WSET, info.pixID
ENDELSE

bitmask = info.showbitmask

;-> 2D MODE
IF (info.dimtrigger EQ 1) THEN BEGIN

;-> Erase Pole Figure
	IF (bitmask[0] EQ 0) THEN ERASE

;-> Erase Pole Figure on switching to epitaxial mode
	IF ((info.modetrigger EQ 1) AND (!D.Name NE 'PS')) THEN BEGIN
		void = Dummyplot(xmin, xmax, ymin, ymax)
	ENDIF

	IF (info.zoomtrigger EQ 5) THEN BEGIN ;-> means reset
		*info.xmin = -1
		*info.xmax = 1
		*info.ymin = -1
		*info.ymax = 1
		info.zoomtrigger = 3
		void = Dummyplot(-1,1,-1,1)
	ENDIF

;-> plot zoomed epitaxial relationships 					 ***** **** *** ** *
;-> info.zoomtrigger EQ 4 is set if zoomed in epitaxial mode ***** **** *** ** *

	IF (info.zoomtrigger EQ 4) THEN BEGIN

		start = CONVERT_COORD(info.clickx1, info.clicky1, /DEVICE, /TO_DATA)
		ende = CONVERT_COORD(info.clickx2, info.clicky2, /DEVICE, /TO_DATA)

			IF (info.clickx1 LE info.clickx2) THEN BEGIN
				xmin = start[0]
				xmax = ende[0]
			ENDIF ELSE BEGIN
				xmin = ende[0]
				xmax = start[0]
			ENDELSE

			IF (info.clicky1 LE info.clicky2) THEN BEGIN
				ymin = start[1]
				ymax = ende[1]
			ENDIF ELSE BEGIN
				ymin = ende[1]
				ymax = start[1]
			ENDELSE

		*info.xmin = xmin
		*info.xmax = xmax
		*info.ymin = ymin
		*info.ymax = ymax
		info.zoomtrigger = 3
		void = Dummyplot(xmin, xmax, ymin, ymax)
	ENDIF

;-> Draw Pole Figure
	IF ((bitmask[0] EQ 1) AND (info.modetrigger EQ 0)) THEN BEGIN
		widget_control, event.top, set_uvalue = info, /NO_Copy
		IF KEYWORD_SET(zoomin) THEN	Zeichnen_xyz, event, /zoomin ELSE Zeichnen_xyz, event
		widget_control, event.top, get_uvalue = info, /NO_Copy
	ENDIF

;-> Plot the Stereographic crystal projections ***** **** *** ** *

FOR k=1,5 DO BEGIN 								;-> (V2)
	IF (bitmask[k] EQ 1) THEN BEGIN
		info.showbitmask[k] = 2
		widget_control, event.top, set_uvalue = info, /NO_Copy
		Stereoplot, event
		widget_control, event.top, get_uvalue = info, /NO_Copy
	ENDIF
ENDFOR

;-> 3D MODE ***** **** *** ** *

ENDIF ELSE BEGIN
	widget_control, event.top, set_uvalue = info, /NO_Copy
	Zeichnen_3d, event
	widget_control, event.top, get_uvalue = info, /NO_Copy
ENDELSE

;- Plot the desired net ***** **** *** ** *
	modetrigger = info.modetrigger

IF (info.nettrigger EQ 1) THEN BEGIN
	widget_control, event.top, set_uvalue = info, /NO_Copy
	IF (modetrigger EQ 0) THEN PolarNet, event ELSE PolarNet, event, /epitaxial
	widget_control, event.top, get_uvalue = info, /NO_Copy
ENDIF ELSE BEGIN
	widget_control, event.top, set_uvalue = info, /NO_Copy
	StereoNet, event, colorlist
	widget_control, event.top, get_uvalue = info, /NO_Copy
ENDELSE

;-> Then copy all to the Draw Widget Window unless in postscript output mode ***** **** *** ** *
IF (!D.Name NE 'PS') THEN BEGIN
WSET, info.plotwindowind
DEVICE, COPY=[0, 0, !D.X_Size, !D.Y_SIZE, 0, 0, info.pixID]
ENDIF

widget_control, event.top, set_uvalue = info, /NO_Copy
END

; ************************** Peak Integration ********************************
PRO Peak_Integration, event
widget_control, event.top, get_uvalue = info, /NO_Copy

	radius = Float(info.peak_range)/180
	x = *info.x
	y =	*info.y
	z = *info.z
	maxi = Max(z)

	peakxy = CONVERT_COORD(*info.peakx, *info.peaky, /DEVICE, /TO_DATA)

;-> Calculate array of data points inside the circle
	distarray = SQRT((x-peakxy[0])^2 + (y-peakxy[1])^2)
	inrange = WHERE(distarray[0,*] lt radius)

IF (N_ELEMENTS(inrange) GT 2) THEN BEGIN

;-> Mark range around desired point by a circle
	PlotS, CIRCLE(peakxy[0], peakxy[1], radius), /NOCLIP

	data = Transpose([ [Transpose(x)],[Transpose(y)],[Transpose(z)] ])
	data_inside = data[*,inrange]

;-> To get a valid representation of the volume of the peak, irregularly spaced data has to be avoided,
;-> because the density of dara points depends on psi. Therefore a Delaunay triangulation is being performed.
;-> resulting in an array of N x N points.

	TRIANGULATE, data_inside[0,*], data_inside[1,*], tr
	regular = TRIGRID(data_inside[0,*],data_inside[1,*],data_inside[2,*], tr, /quintic, NX=80, NY=80)
	regular = regular/maxi ; maximum intensity is set to 1

;-> Because the TRIGRID function returns a regular grid of interpolated Z values, the sum over the intensities is
;-> very well proportional to the volume of the peak.

	IF (*info.temp EQ 1) THEN BEGIN
		*info.peak_volume = *info.peak_volume + TOTAL(regular)
		result = Dialog_Message('Sum of Peak Volumes: ' + STRCOMPRESS(String(*info.peak_volume), /REMOVE_ALL),$
			TITLE = 'Stereopole - [Shift pressed] ', /INFORMATION)
	ENDIF ELSE BEGIN
		*info.peak_volume = TOTAL(regular)
		result = Dialog_Message('Single Peak Volume: ' + STRCOMPRESS(String(*info.peak_volume), /REMOVE_ALL),$
			TITLE = 'Stereopole', /INFORMATION)
	ENDELSE

;-> Uncomment this to see the result of the Delaunay triangulation in comparison with the original data points
;->
;->	Contour, regular, nlevels=30, /FILL, /ISOTROPIC
;->	wait, 2
;->	Contour, data_inside[2,*],data_inside[0,*],data_inside[1,*], /irregular, nlevels=30, /FILL, /ISOTROPIC
ENDIF
widget_control, event.top, set_uvalue = info, /NO_Copy
END

; ************************** Plot Stereogramm ********************************
PRO Stereoplot, event
widget_control, event.top, get_uvalue = info, /NO_Copy

;-> Define current vectors for plot, specific current crystal is marked with bit = 2
;-> and set bit back to 1 again

FOR k=1,5 DO BEGIN
	IF (info.showbitmask[k] EQ 2) THEN BEGIN
		CASE k OF
			1: BEGIN
					current2D = info.Vectors2D_1
					currentbutton = (info.showopt).showopt2
					j=k
				END
			2: BEGIN
					current2D = info.Vectors2D_2
					currentbutton = (info.showopt).showopt3
					j=k
				END
			3: BEGIN
					current2D = info.Vectors2D_3
					currentbutton = (info.showopt).showopt4
					j=k
				END
			4: BEGIN
					current2D = info.Vectors2D_4
					currentbutton = (info.showopt).showopt5
					j=k
				END
			5: BEGIN
					current2D = info.Vectors2D_5
					currentbutton = (info.showopt).showopt6
					j=k
				END
		ENDCASE
	ENDIF
ENDFOR

;-> (hkl) and [uvw] of current crystal

IF (N_ELEMENTS(*current2D) EQ 0) THEN BEGIN
	errormsg = dialog_message('Please set crystal data first!', /ERROR)
	widget_control, currentbutton, set_button = 0
	info.showbitmask[j] = 0
	widget_control, event.top, set_uvalue = info, /NO_Copy
	RETURN
ENDIF ELSE BEGIN
	IF (info.modetrigger EQ 0) THEN BEGIN
		vectors = (*current2D).recvec2d
		labels =  (*current2D).labelstring_rec + (*current2D).harmonicstring
		info.showbitmask[j] = 1
		info.uvwcolor = j
	ENDIF ELSE BEGIN
		vectors = (*current2D).dirvec2d
		labels =  (*current2D).labelstring_dir
		info.showbitmask[j] = 1
		info.uvwcolor = j
	ENDELSE
ENDELSE

;-> Calculate Positions of directions [uvw] for the Epitaxial Realtionships

IF (info.modetrigger EQ 1) THEN BEGIN

	;-> Discard directions with psi out of desired range
		range = Float(info.epirange) / 90

	;-> Generate a bitmask, where all values of at indices of vectors[0,*] and vectors[1,*]
	;-> are zeroed if the values of vectors[0,*] and vectors[1,*] are out of range

	vectors_distance = SQRT(vectors[0,*]^2 + vectors[1,*]^2)
	innerbitmask = WHERE((vectors_distance GT (1 - range)) AND (vectors_distance LT (1 + range))) ;-> bitmask for those within the borders

	IF (innerbitmask[0] EQ -1) THEN BEGIN

		dummy = [0.0,0.0]
		dummylabel = 'no [uvw] in range.'

	ENDIF ELSE BEGIN

		dummy = vectors(*,innerbitmask)
		dummylabel = labels(*,innerbitmask)

	;	IF (N_ELEMENTS(dummy) EQ 2) AND (dummy[0,0] EQ 0.0) AND (dummy[1,0] EQ 0.0) THEN dummylabel = 'no [uvw] in range.'
	ENDELSE

	vectors = dummy
	labels = dummylabel

ENDIF

;-> Define square symbol for plot and other parameters

pointsize = 0.8 * info.pointsize
charsize = 0.05 * info.charsize
charofset= 0.03 * info.charofset
colorlist = *info.colorlist

usymbx = [-0.5,0.5,0.5,-0.5,-0.5]
usymby = [-0.5,-0.5,0.5,0.5,-0.5]
USERSYM, usymbx, usymby, /FILL

CASE info.dropstring[0] OF
	'Square':	mypsym=8
	'Plus':		mypsym=1
	'Asterisk':	mypsym=2
	'Period':	mypsym=3
	'Diamond':	mypsym=4
	'Triangle':	mypsym=5
	'Cross':	mypsym=7
ENDCASE

;-> Set different colors for the different crystals
CASE info.uvwcolor OF
	1:  BEGIN
		desired = [info.colors[0],info.colors[1],info.colors[2]]
		desiredlist = colorlist[0]
		END
	2:  BEGIN
		desired = [info.colors[3],info.colors[4],info.colors[5]]
		desiredlist = colorlist[1]
		END
	3:  BEGIN
		desired = [info.colors[6],info.colors[7],info.colors[8]]
		desiredlist = colorlist[2]
		END
	4:  BEGIN
		desired = [info.colors[9],info.colors[10],info.colors[11]]
		desiredlist = colorlist[3]
		END
	5:  BEGIN
		desired = [info.colors[12],info.colors[13],info.colors[14]]
		desiredlist = colorlist[4]
		END
ENDCASE

;-> Calculate the nearest match for two desired colors with apropriate contrast [screen mode]
		TVLCT, Red, Green, Blue, /Get
		Red = Long(Red)
		Green = Long(Green)
		Blue = Long(Blue)
		dist = (Red-desired[0])^2 + (Green-desired[1])^2 + (Blue-desired[2])^2
		index = Where(dist EQ Min(dist))

;-> modetrigger=0 in case of pole figure mode
IF (info.modetrigger EQ 0) THEN BEGIN
	IF (!D.NAME EQ 'PS') THEN BEGIN
	oplot, vectors[0,*], vectors[1,*], THICK=10, PSYM=mypsym, SYMSIZE=pointsize, /NOCLIP, color=desiredlist
	ENDIF ELSE BEGIN
	oplot, vectors[0,*], vectors[1,*], PSYM=mypsym, SYMSIZE=pointsize, /NOCLIP, color=index[0]
	ENDELSE
	IF (info.labelon EQ 1) THEN BEGIN
		IF (!D.NAME EQ 'PS') THEN BEGIN
			XYOUTS, (vectors[0,*] + charofset), vectors[1,*], labels, charsize = charsize, color=desiredlist, font=0
		ENDIF ELSE BEGIN
			XYOUTS, (vectors[0,*] + charofset), vectors[1,*], labels, charsize = charsize, color=index[0]
		ENDELSE
	ENDIF

;-> modetrigger=1 in case of epitaxial mode
ENDIF ELSE BEGIN

	IF (!D.NAME EQ 'PS') THEN BEGIN
		oplot, vectors[0,*], vectors[1,*], THICK=10, PSYM=mypsym, SYMSIZE=pointsize, color=desiredlist, /NOCLIP
		IF (info.labelon EQ 1) THEN XYOUTS, (vectors[0,*] + charofset),$
			(vectors[1,*] + charofset*0.5 - (info.uvwcolor-1)*charofset), labels, color=desiredlist, charsize = charsize, font=0
	ENDIF ELSE BEGIN
		oplot, vectors[0,*], vectors[1,*], PSYM=mypsym, SYMSIZE=pointsize, color=index[0], /NOCLIP
		IF (info.labelon EQ 1) THEN XYOUTS, (vectors[0,*] + charofset),$
			(vectors[1,*] + charofset*0.5 - (info.uvwcolor-1)*charofset), labels, color=index[0], charsize = charsize
	ENDELSE

	info.zoomtrigger = 3 ;-> reset the zoomtrigger

ENDELSE
widget_control, event.top, set_uvalue = info, /NO_Copy
END

; ********************************* Quit procedure *************************************
PRO Quit, event

Widget_Control, event.top, /Destroy

END

; ******************************* Write ASCII Data *************************************
Pro ASCIIWriteData, event
widget_control, event.top, get_uvalue = info, /NO_Copy
thisPath=info.path

Catch, theError
IF theError NE 0 THEN BEGIN
   Catch, /Cancel
   errormsg = dialog_message('Error writing ASCII File!', /ERROR)
   info.path=thisPath
   Widget_Control, event.top, Set_UValue=info, /No_Copy
   RETURN
ENDIF

IF (N_ELEMENTS(*info.daten) EQ 0) THEN BEGIN
widget_control, event.top, set_uvalue = info, /NO_Copy
RETURN
ENDIF ELSE BEGIN

nop = *info.nop
data = *info.daten

outfilename = Dialog_Pickfile(Filter='*.dat',/Write, PATH=info.path, GET_PATH=thePath)
info.path = thePath ; Store current File Path for next data input
OpenW, dat, outfilename, /Get_Lun
FOR j=0,nop-1 DO PrintF, dat, data[0,j], data[1,j], data[2,j]
Free_Lun, dat

widget_control, event.top, set_uvalue = info, /NO_Copy
ENDELSE
END

; *********************** Write SavePsiScan as ASCII **********************************
Pro SavePsiScan, event
widget_control, event.top, get_uvalue = info, /NO_Copy
thisPath=info.path

;Catch, theError
;IF theError NE 0 THEN BEGIN
;   Catch, /Cancel
;   errormsg = dialog_message('Error writing ASCII File!', /ERROR)
;   info.path=thisPath
;   Widget_Control, event.top, Set_UValue=info, /No_Copy
;   RETURN
;ENDIF

IF (N_ELEMENTS(*info.daten) EQ 0) THEN BEGIN
widget_control, event.top, set_uvalue = info, /NO_Copy
RETURN
ENDIF ELSE BEGIN

data = *info.daten
start = *info.psistart
ende = *info.psiend
step = *info.psistep
nop = (ende-start)/step

x = DBLARR(nop+1)
y = DBLARR(nop+1)

FOR i=0,nop DO BEGIN
 x[i] = start+i*step
 y[i] = TOTAL(data[2,WHERE(data[1,*] EQ start+i*step)])
ENDFOR

outfilename = Dialog_Pickfile(Filter='*.dat',/Write, PATH=info.path, GET_PATH=thePath)
info.path = thePath ; Store current File Path for next data input

OpenW, dat, outfilename, /Get_Lun
FOR j=0,nop DO PrintF, dat, x[j], y[j]
Free_Lun, dat

widget_control, event.top, set_uvalue = info, /NO_Copy
ENDELSE
END


; ***************************** Save all variables ************************************
PRO Save_Session, event
widget_control, event.top, get_uvalue = info, /NO_Copy

Catch, theError
IF theError NE 0 THEN BEGIN
   Catch, /Cancel
   errormsg = dialog_message('Session not saved!', /ERROR)
   IF N_Elements(info) NE 0 THEN BEGIN
     Widget_Control, event.top, Set_UValue=info, /No_Copy
   END
   RETURN
ENDIF

save, info, filename = Dialog_Pickfile(Filter='*.stp', /Write, PATH=info.path)

widget_control, event.top, set_uvalue = info, /NO_Copy
END

; ***************************** Load previous session *********************************
PRO Load_Session, event
widget_control, event.top, get_uvalue = info, /NO_COPY

Catch, theError
IF theError NE 0 THEN BEGIN
   Catch, /Cancel
   errormsg = dialog_message('No session loaded.', /ERROR)
   widget_control, event.top, set_uvalue = info, /NO_COPY
   RETURN
ENDIF

;-> Store current widget identifiers
current0 = info.labelID
current1 = info.stepsizewid
current2 = info.plotwindow
current3 = info.mainwindow
current4 = info.tablebuttonID1
current5 = info.labelID_int
current6 = info.minintID
current7 = info.maxintID
modeopt_current = info.modeopt
info_current = info.labelfile
rotopt_current = info.rotopt
showopt_current = info.showopt
labelswitch1 = info.labelswitch1
labelswitch2 = info.labelswitch2

filename = Dialog_Pickfile(Filter='*.stp', /Read, PATH=info.path, GET_PATH=thePath)
restore, filename

info.path = thePath ; Store current File Path for next data input

;-> Restore widget identifiers
info.labelID = current0
info.stepsizewid = current1
info.plotwindow = current2
info.mainwindow = current3
info.tablebuttonID1 = current4
info.labelID_int = current5
info.minintID = current6
info.maxintID = current7

info.modeopt = modeopt_current
info.rotopt = rotopt_current
info.showopt = showopt_current
info.labelfile = info_current
info.labelswitch1 = labelswitch1
info.labelswitch2 = labelswitch2
info.path = thePath

widget_control, (info.showopt).showopt1, set_button = info.showbitmask[0]
widget_control, (info.showopt).showopt2, set_button = info.showbitmask[1]
widget_control, (info.showopt).showopt3, set_button = info.showbitmask[2]
widget_control, (info.showopt).showopt4, set_button = info.showbitmask[3]
widget_control, (info.showopt).showopt5, set_button = info.showbitmask[4]
widget_control, (info.showopt).showopt6, set_button = info.showbitmask[5]

IF (info.modetrigger EQ 0) THEN BEGIN
	widget_control, (info.modeopt).modeopt1, set_button = 1
	widget_control, (info.modeopt).modeopt2, set_button = 0
	ENDIF ELSE BEGIN
	widget_control, (info.modeopt).modeopt2, set_button = 1
	widget_control, (info.modeopt).modeopt1, set_button = 0
ENDELSE

widget_control, (info.rotopt).rotopt1, set_button = info.rotatebitmask[0]
widget_control, (info.rotopt).rotopt2, set_button = info.rotatebitmask[1]
widget_control, (info.rotopt).rotopt3, set_button = info.rotatebitmask[2]
widget_control, (info.rotopt).rotopt4, set_button = info.rotatebitmask[3]
widget_control, (info.rotopt).rotopt5, set_button = info.rotatebitmask[4]
widget_control, (info.rotopt).rotopt6, set_button = info.rotatebitmask[5]

IF (info.labelon EQ 1) THEN BEGIN
	widget_control, labelswitch1, set_button = 1
	widget_control, labelswitch2, set_button = 0
ENDIF ELSE BEGIN
	widget_control, labelswitch2, set_button = 1
	widget_control, labelswitch1, set_button = 0
ENDELSE

IF (N_ELEMENTS(*info.daten) NE 0) THEN BEGIN
	widget_Control, info.minintID, set_value = *info.minint
	widget_Control, info.maxintID, set_value = *info.maxint
ENDIF

IF (N_ELEMENTS(*info.header) NE 0) THEN BEGIN
	header = *info.header
	headerline1 = STRSPLIT(header[1], ',', /EXTRACT)
	headerline3 = STRSPLIT(header[2], ',', /EXTRACT)
		filename = headerline1[1]
		date = headerline1[3]
		SampleName = headerline3[1]
	label0 = string('Filename: ', strcompress(filename))
	label1 = string('File Date: ', strcompress(date))
	label3 = string('Sample Name: ', strcompress(SampleName))
	widget_control, (info.labelfile).labelfilemain, set_value = label0
	widget_control, (info.labelfile).labeldatemain, set_value = label1
	widget_control, (info.labelfile).labelnamemain, set_value = label3
ENDIF ELSE BEGIN
  	label3 = string('No pole figure file loaded.')
  	widget_control, (info.labelfile).labelnamemain, set_value = label3
ENDELSE

widget_control, info.stepsizewid, set_value = info.rotstepsize

IF (N_ELEMENTS(*info.daten) EQ 0) THEN BEGIN
widget_control, event.top, set_uvalue = info, /NO_COPY
Zeichnen_XYZ_Reset, event
ENDIF ELSE BEGIN
widget_control, event.top, set_uvalue = info, /NO_COPY
ShowMain, event
ENDELSE
END

; ************************* Set Preferences for Plot ******************************
PRO Preferences, event

widget_control, event.top, get_uvalue = info
widget_control, info.plotwindow, DRAW_MOTION_EVENTS=0
widget_control, info.plotwindow, DRAW_BUTTON_EVENTS=0

;-> plot part on the left

prefwindow = widget_base(column=2, mbar=mbar, title = 'Preferences ...', TLB_FRAME_ATTR=1)
	histobaseID = widget_base(prefwindow, row=1, frame=1)
	histoplotID = Widget_Draw(histobaseID, XSize=500, YSIZE=450, uvalue = 'histoplot')

;-> options part on the right
	rightbase = widget_base(prefwindow, column=2, frame=1)
	settingbaseID = widget_base(rightbase, row=2)

			settingbase2ID = widget_base(settingbaseID,row=3)
				labelID = widget_label(settingbase2ID, value='Plot Preferences', /align_left)
				settingbase22ID = widget_base(settingbase2ID, column=1, frame=1, xsize=200, /base_align_right)
				nlevelsID = CW_Field(settingbase22ID, Title = '# of contour levels: ', uvalue = 'nlevels', value=info.numberoflevels, xsize=5, /integer, /all_events)
				netstepID = CW_Field(settingbase22ID, Title = 'PolarNet steps []: ', uvalue = 'netstep', value=info.degreestep, xsize=5, /integer, /all_events)
				charsizeID = CW_Field(settingbase22ID, Title = 'Set (hkl)-label size: ', uvalue = 'charsize', value=info.charsize, xsize=5, /integer, /all_events)
				charofsetID = CW_Field(settingbase22ID, Title = 'Set label offset: ', uvalue = 'charofset', value=info.charofset, xsize=5, /floating, /all_events)
					settingbase221ID = widget_base(settingbase22ID, column=2)
						labelID = widget_label(settingbase221ID, value='(hkl) symbol: ', /align_left)
						dropID = widget_droplist(settingbase221ID, value=info.dropstring, uvalue = 'symbolselect')
				pointsizeID = CW_Field(settingbase22ID, Title = 'Set size of symbol: ', uvalue = 'pointsize', value=info.pointsize, xsize=5, /floating, /all_events)

			nthlineID = CW_Field(settingbase22ID, Title = 'Label nth line: ', uvalue = 'nthline', value=info.nthline, xsize=5, /integer, /all_events)
			res3dID = CW_Field(settingbase22ID, Title = 'Set 3D resolution [x_pts]: ', uvalue = 'res3d', value=*info.res3d, xsize=5, /integer, /all_events)
			epirange = CW_Field(settingbase22ID, Title = 'Range at psi=90 (uvw): ', uvalue = 'epi', value=info.epirange, xsize=5, /integer, /all_events)
			intirange = CW_Field(settingbase22ID, Title = 'Diameter of peak-integration: ', uvalue = 'inti', value=info.peak_range, xsize=5, /integer, /all_events)
			qualityID = CW_Field(settingbase22ID, Title = 'Quality of inverse (hkl): ', uvalue = 'quali', value=info.quality, xsize=5, /all_events)

	settingbaseID1 = widget_base(rightbase, column=1,YSize=450, /base_align_right)

			colorsbase = widget_base(settingbaseID1, column=1, xsize=250, /base_align_right)
				label = widget_label(colorsbase, value='Set Crystal Colors', /align_left)
				colorsbase1 = widget_base(colorsbase, column=3, frame=1)
					hfieldmin = CW_Field(colorsbase1, Title = 'Crystal 1 - R:', uvalue = 'red1', value = info.colors[0], xsize=3, /integer, /all_events)
					kfieldmin = CW_Field(colorsbase1, Title = 'G:', uvalue = 'green1', value = info.colors[1], xsize=3, /integer, /all_events)
					lfieldmin = CW_Field(colorsbase1, Title = 'B:', uvalue = 'blue1', value = info.colors[2], xsize=3, /integer, /all_events)
				colorsbase2 = widget_base(colorsbase, column=3, frame=1)
					hfieldmax = CW_Field(colorsbase2, Title = 'Crystal 2 - R:', uvalue = 'red2', value = info.colors[3], xsize=3, /integer, /all_events)
					kfieldmax = CW_Field(colorsbase2, Title = 'G:', uvalue = 'green2', value = info.colors[4], xsize=3, /integer, /all_events)
					lfieldmax = CW_Field(colorsbase2, Title = 'B:', uvalue = 'blue2', value = info.colors[5], xsize=3, /integer, /all_events)
				colorsbase3 = widget_base(colorsbase, column=3, frame=1)
					hfieldmin = CW_Field(colorsbase3, Title = 'Crystal 3 - R:', uvalue = 'red3', value = info.colors[6], xsize=3, /integer, /all_events)
					kfieldmin = CW_Field(colorsbase3, Title = 'G:', uvalue = 'green3', value = info.colors[7], xsize=3, /integer, /all_events)
					lfieldmin = CW_Field(colorsbase3, Title = 'B:', uvalue = 'blue3', value = info.colors[8], xsize=3, /integer, /all_events)
				colorsbase4 = widget_base(colorsbase, column=3, frame=1)
					hfieldmax = CW_Field(colorsbase4, Title = 'Crystal 4 - R:', uvalue = 'red4', value = info.colors[9], xsize=3, /integer, /all_events)
					kfieldmax = CW_Field(colorsbase4, Title = 'G:', uvalue = 'green4', value = info.colors[10], xsize=3, /integer, /all_events)
					lfieldmax = CW_Field(colorsbase4, Title = 'B:', uvalue = 'blue4', value = info.colors[11], xsize=3, /integer, /all_events)
				colorsbase5 = widget_base(colorsbase, column=3, frame=1)
					hfieldmax = CW_Field(colorsbase5, Title = 'Crystal 5 - R:', uvalue = 'red5', value = info.colors[12], xsize=3, /integer, /all_events)
					kfieldmax = CW_Field(colorsbase5, Title = 'G:', uvalue = 'green5', value = info.colors[13], xsize=3, /integer, /all_events)
					lfieldmax = CW_Field(colorsbase5, Title = 'B:', uvalue = 'blue5', value = info.colors[14], xsize=3, /integer, /all_events)

			settingbase223ID = widget_base(settingbaseID1,column=2)
						labelID = widget_label(settingbase223ID, value='What to show ...')
						modebaseID = widget_base(settingbase223ID, row=3, frame=1, /exclusive, /base_align_right)
						modeopt1ID = widget_button(modebaseID, value='Only Contour', uvalue = 'cont')
						modeopt2ID = widget_button(modebaseID, value='Only Lines', uvalue = 'lines')
						modeopt3ID = widget_button(modebaseID, value='Both', uvalue = 'both')
			settingbase224ID = widget_base(settingbaseID1,column=1)
						labelID = widget_label(settingbase224ID, value='Devide (hkl) by Greatest Common Devisor?')
						gcdbaseID = widget_base(settingbase224ID, column=1, frame=1, /exclusive, /base_align_right, /align_right)
						gcdopt1ID = widget_button(gcdbaseID, value='yes', uvalue = 'gcd')
						gcdopt2ID = widget_button(gcdbaseID, value='no', uvalue = 'no_gcd')

;-> Set mode button
CASE info.linetrigger OF
		0:	BEGIN
				widget_control, modeopt1ID, set_button = 1
			END
		1:	BEGIN
				widget_control, modeopt2ID, set_button = 1
			END
		2:	BEGIN
				widget_control, modeopt3ID, set_button = 1
			END
	 ELSE:
ENDCASE

;-> Set GCD button
CASE info.apply_gcd OF
		0:	BEGIN
				widget_control, gcdopt2ID, set_button = 1
			END
		1:	BEGIN
				widget_control, gcdopt1ID, set_button = 1
			END
	 ELSE:
ENDCASE



;-> Realize

widget_control, prefwindow, /Realize
widget_control, histoplotID, get_value = histoplotind
WSET, histoplotind
widget_control, prefwindow, Set_UValue=info, /No_Copy

HistoPole, event

XManager, 'Preferences', prefwindow, Event_Handler = 'UpdatePreferences', Cleanup='PreferencesCleanup'

END

; **********************************
PRO UpdatePreferences, event

widget_control, event.top, get_uvalue = info, /NO_Copy
widget_control, event.id, get_uvalue = eventID

IF (eventID NE 'symbolselect') THEN widget_control, event.id, get_value = numbervalue

CASE eventID OF

	'nlevels':	BEGIN
					IF (numbervalue GE 3) THEN info.numberoflevels = numbervalue ELSE info.numberoflevels = 3
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'netstep':	BEGIN
					info.degreestep = numbervalue
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'charsize':	BEGIN
					info.charsize = numbervalue
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'charofset':	BEGIN
					info.charofset = numbervalue
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'symbolselect':	BEGIN
					info.symbolselect = event.index
					info.dropstring = SHIFT(info.dropstring, -event.index)
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'pointsize':BEGIN
					info.pointsize = numbervalue
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'cont':	BEGIN
					IF (info.linetrigger NE 0) THEN info.linetrigger = 0
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'lines':	BEGIN
					IF (info.linetrigger NE 1) THEN info.linetrigger = 1
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'both':		BEGIN
					IF (info.linetrigger NE 2) THEN info.linetrigger = 2
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'nthline':	BEGIN
					IF (numbervalue GE 0) THEN info.nthline = numbervalue
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'res3d':	BEGIN
					IF (numbervalue GT 10) THEN *info.res3d = numbervalue
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'epi':		BEGIN
					IF (numbervalue GT 0) THEN info.epirange = Float(Abs(numbervalue))
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'inti':		BEGIN
					IF (numbervalue GT 0) THEN info.peak_range = numbervalue
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'quali':		BEGIN
					IF ((numbervalue GT 0.01) AND (numbervalue LT 1.0))  THEN info.quality = numbervalue
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END

	'red1':		BEGIN
					IF ((numbervalue GE 0) AND (numbervalue LE 255)) THEN info.colors[0] = numbervalue ELSE BEGIN
						errormsg = dialog_message('Value must be between 0 and 255', /ERROR)
					ENDELSE
					*info.colorlist = SetColor(info.colors)
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'green1':	BEGIN
					IF ((numbervalue GE 0) AND (numbervalue LE 255)) THEN info.colors[1] = numbervalue ELSE BEGIN
						errormsg = dialog_message('Value must be between 0 and 255', /ERROR)
					ENDELSE
					*info.colorlist = SetColor(info.colors)
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'blue1':	BEGIN
					IF ((numbervalue GE 0) AND (numbervalue LE 255)) THEN info.colors[2] = numbervalue ELSE BEGIN
						errormsg = dialog_message('Value must be between 0 and 255', /ERROR)
					ENDELSE
					*info.colorlist = SetColor(info.colors)
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'red2':		BEGIN
					IF ((numbervalue GE 0) AND (numbervalue LE 255)) THEN info.colors[3] = numbervalue ELSE BEGIN
						errormsg = dialog_message('Value must be between 0 and 255', /ERROR)
					ENDELSE
					*info.colorlist = SetColor(info.colors)
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'green2':	BEGIN
					IF ((numbervalue GE 0) AND (numbervalue LE 255)) THEN info.colors[4] = numbervalue ELSE BEGIN
						errormsg = dialog_message('Value must be between 0 and 255', /ERROR)
					ENDELSE
					*info.colorlist = SetColor(info.colors)
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'blue2':	BEGIN
					IF ((numbervalue GE 0) AND (numbervalue LE 255)) THEN info.colors[5] = numbervalue ELSE BEGIN
						errormsg = dialog_message('Value must be between 0 and 255', /ERROR)
					ENDELSE
					*info.colorlist = SetColor(info.colors)
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'red3':		BEGIN
					IF ((numbervalue GE 0) AND (numbervalue LE 255)) THEN info.colors[6] = numbervalue ELSE BEGIN
						errormsg = dialog_message('Value must be between 0 and 255', /ERROR)
					ENDELSE
					*info.colorlist = SetColor(info.colors)
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'green3':	BEGIN
					IF ((numbervalue GE 0) AND (numbervalue LE 255)) THEN info.colors[7] = numbervalue ELSE BEGIN
						errormsg = dialog_message('Value must be between 0 and 255', /ERROR)
					ENDELSE
					*info.colorlist = SetColor(info.colors)
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'blue3':	BEGIN
					IF ((numbervalue GE 0) AND (numbervalue LE 255)) THEN info.colors[8] = numbervalue ELSE BEGIN
						errormsg = dialog_message('Value must be between 0 and 255', /ERROR)
					ENDELSE
					*info.colorlist = SetColor(info.colors)
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'red4':		BEGIN
					IF ((numbervalue GE 0) AND (numbervalue LE 255)) THEN info.colors[9] = numbervalue ELSE BEGIN
						errormsg = dialog_message('Value must be between 0 and 255', /ERROR)
					ENDELSE
					*info.colorlist = SetColor(info.colors)
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'green4':	BEGIN
					IF ((numbervalue GE 0) AND (numbervalue LE 255)) THEN info.colors[10] = numbervalue ELSE BEGIN
						errormsg = dialog_message('Value must be between 0 and 255', /ERROR)
					ENDELSE
					*info.colorlist = SetColor(info.colors)
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'blue4':	BEGIN
					IF ((numbervalue GE 0) AND (numbervalue LE 255)) THEN info.colors[11] = numbervalue ELSE BEGIN
						errormsg = dialog_message('Value must be between 0 and 255', /ERROR)
					ENDELSE
					*info.colorlist = SetColor(info.colors)
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'red5':		BEGIN
					IF ((numbervalue GE 0) AND (numbervalue LE 255)) THEN info.colors[12] = numbervalue ELSE BEGIN
						errormsg = dialog_message('Value must be between 0 and 255', /ERROR)
					ENDELSE
					*info.colorlist = SetColor(info.colors)
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'green5':	BEGIN
					IF ((numbervalue GE 0) AND (numbervalue LE 255)) THEN info.colors[13] = numbervalue ELSE BEGIN
						errormsg = dialog_message('Value must be between 0 and 255', /ERROR)
					ENDELSE
					*info.colorlist = SetColor(info.colors)
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'blue5':	BEGIN
					IF ((numbervalue GE 0) AND (numbervalue LE 255)) THEN info.colors[14] = numbervalue ELSE BEGIN
						errormsg = dialog_message('Value must be between 0 and 255', /ERROR)
					ENDELSE
					*info.colorlist = SetColor(info.colors)
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'gcd':		BEGIN
					info.apply_gcd = 1
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'no_gcd':	BEGIN
					info.apply_gcd = 0
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	ELSE :
	ENDCASE
END

; ***************
PRO PreferencesCleanup, prefwindow
widget_control, prefwindow, get_uvalue = info, /NO_Copy

plotwindowind = info.plotwindowind
plotwindow = info.plotwindow
info.updatetrigger = 1

widget_control, info.mainwindow, set_uvalue = info, /No_Copy
WSET, plotwindowind
widget_control, plotwindow, DRAW_MOTION_EVENTS=1
widget_control, plotwindow, DRAW_BUTTON_EVENTS=1
END

; *************************** Show Table and set indices ******************************
PRO stereopoleTable, event
widget_control, event.top, get_uvalue = info
widget_control, info.plotwindow, DRAW_MOTION_EVENTS=0
widget_control, info.plotwindow, DRAW_BUTTON_EVENTS=0

;-> Set trigger to redraw stereogramm after input
info.updatetrigger = 2
info.axistrigger = 0

columntitle = ['h[u]','k[v]','l[w]','foldness','harmony']
;
i = info.crystaltrigger
i=i-1
man = REFORM(info.manualindex[*,*,i]) ; (V2) Extract manual indices belonging to the certain crystal

;middlehkl = info.middlehkl[i]
;info.rotaxis_xyz

vector = [0.0,0.0,1.0]
quality = info.quality

CASE info.crystaltrigger OF
	1: IF ((N_ELEMENTS(*info.daten) NE 0) AND (N_ELEMENTS(*info.allindices1) NE 0)) THEN middlehkl = CalculateBack(info, vector, quality, i) ELSE middlehkl = info.middlehkl[*,i]
	2: IF ((N_ELEMENTS(*info.daten) NE 0) AND (N_ELEMENTS(*info.allindices2) NE 0)) THEN middlehkl = CalculateBack(info, vector, quality, i) ELSE middlehkl = info.middlehkl[*,i]
	3: IF ((N_ELEMENTS(*info.daten) NE 0) AND (N_ELEMENTS(*info.allindices3) NE 0)) THEN middlehkl = CalculateBack(info, vector, quality, i) ELSE middlehkl = info.middlehkl[*,i]
	4: IF ((N_ELEMENTS(*info.daten) NE 0) AND (N_ELEMENTS(*info.allindices4) NE 0)) THEN middlehkl = CalculateBack(info, vector, quality, i) ELSE middlehkl = info.middlehkl[*,i]
	5: IF ((N_ELEMENTS(*info.daten) NE 0) AND (N_ELEMENTS(*info.allindices5) NE 0)) THEN middlehkl = CalculateBack(info, vector, quality, i) ELSE middlehkl = info.middlehkl[*,i]
ENDCASE

;-> table part on the left

crystalwindow =	widget_base(column=3, mbar=mbar, title = 'Crystal Data', TLB_FRAME_ATTR=1)
tablebase = widget_base(crystalwindow, row=2, frame=1)
tablelabel = widget_label(tablebase, value='Netplanes and Directions to Show - manual input mode')
tableID = WIDGET_TABLE(tablebase, /Editable, /All_Events, column_labels = columntitle,$
 	 x_scroll_size = 5, y_scroll_size = 20, COLUMN_WIDTHS=50,$
 	 value=man, uvalue = 'infotable')

;-> options part on the right

crystalparmsbase = widget_base(crystalwindow, row=5, frame=1)

			crystalparmsbase1 = widget_base(crystalparmsbase, row=2, xsize=170)
			label = widget_label(crystalparmsbase1, value='Unit Cell', /align_left)
			unitcellbase = widget_base(crystalparmsbase1, column=2, frame=1)
			crystalcell1 = widget_base(unitcellbase, row=3)
				afield = CW_Field(crystalcell1, Title = 'a:', uvalue = 'a', value=info.a[i], xsize=7, /floating, /all_events)
				bfield = CW_Field(crystalcell1, Title = 'b:', uvalue = 'b', value=info.b[i], xsize=7, /floating, /all_events)
				cfield = CW_Field(crystalcell1, Title = 'c:', uvalue = 'c', value=info.c[i], xsize=7, /floating, /all_events);
			crystalcell2 = widget_base(unitcellbase, row=3, /align_right)
				alphafield = CW_Field(crystalcell2, Title = 'al: ', uvalue = 'alpha', value=info.alpha[i], xsize=7, /floating, /all_events)
				betafield = CW_Field(crystalcell2, Title = 'be:', uvalue = 'beta', value=info.beta[i], xsize=7, /floating, /all_events)
				gammafield = CW_Field(crystalcell2, Title = 'ga:', uvalue = 'gamma', value=info.gamma[i], xsize=7, /floating, /all_events)


			crystalparmsbase2 = widget_base(crystalparmsbase, column=1, xsize=170)
			label = widget_label(crystalparmsbase2, value='Current Orientation (center - hkl)', /align_left)

			modebase = widget_base(crystalparmsbase2, row=2, frame=1, /exclusive)
				modekeep = widget_button(modebase, value='Keep Orientation', uvalue = 'keep')
				modeset = widget_button(modebase, value='Set new Orientation', uvalue = 'setnew')

			middlecell = widget_base(crystalparmsbase2, row=1, frame=1)
			middlecell1 = widget_base(middlecell, column=3)
				afield = CW_Field(middlecell1, Title = 'h(u): ', uvalue = 'middleh', value=middlehkl[0], xsize=2, /integer, /all_events)
				bfield = CW_Field(middlecell1, Title = 'k(v):', uvalue = 'middlek', value=middlehkl[1], xsize=2, /integer, /all_events)
				cfield = CW_Field(middlecell1, Title = 'l(w):', uvalue = 'middlel', value=middlehkl[2], xsize=2, /integer, /all_events)


			crystalparmsbase3 = widget_base(crystalparmsbase, column=1, xsize=170)
			label = widget_label(crystalparmsbase3, value='Define Index Range', /align_left)
			rangebase = widget_base(crystalparmsbase3, row=4, frame=1)
			label = widget_label(rangebase, value='Lower bound of indices', /align_left)
			rangebase1 = widget_base(rangebase, column=3, frame=1)
				hfieldmin = CW_Field(rangebase1, Title = 'h(u):', uvalue = 'hminin', value = info.hmin[i], xsize=2, /integer, /all_events)
				kfieldmin = CW_Field(rangebase1, Title = 'k(v):', uvalue = 'kminin', value = info.kmin[i], xsize=2, /integer, /all_events)
				lfieldmin = CW_Field(rangebase1, Title = 'l(w):', uvalue = 'lminin', value = info.lmin[i], xsize=2, /integer, /all_events)
			label = widget_label(rangebase, value='Upper bound of indices', /align_left)
			rangebase2 = widget_base(rangebase, column=3, frame=1)
				hfieldmax = CW_Field(rangebase2, Title = 'h(u):', uvalue = 'hmaxin', value = info.hmax[i], xsize=2, /integer, /all_events)
				kfieldmax = CW_Field(rangebase2, Title = 'k(v):', uvalue = 'kmaxin', value = info.kmax[i], xsize=2, /integer, /all_events)
				lfieldmax = CW_Field(rangebase2, Title = 'l(w):', uvalue = 'lmaxin', value = info.lmax[i], xsize=2, /integer, /all_events)

			crystalparmsbase4 = widget_base(crystalparmsbase, column=1, xsize=170)
			label = widget_label(crystalparmsbase4, value='Index construction mode', /align_left)
			modebase = widget_base(crystalparmsbase4, row=3, frame=1, /exclusive)

			modeopt1 = widget_button(modebase, value='insert manually in table', uvalue = 'manually')
			modeopt2 = widget_button(modebase, value='all indices in range', uvalue = 'byrange')
			modeopt3 = widget_button(modebase, value='both', uvalue = 'both')

;-> Set mode button
CASE info.autoindextrigger[i] OF
		1:	widget_control, modeopt1, set_button = 1
		2:	widget_control, modeopt2, set_button = 1
		3:	widget_control, modeopt3, set_button = 1
	 ELSE:
ENDCASE

widget_control, modekeep, set_button = 1
WIDGET_CONTROL, afield, SENSITIVE=0
WIDGET_CONTROL, bfield, SENSITIVE=0
WIDGET_CONTROL, cfield, SENSITIVE=0
*info.temp = [afield,bfield,cfield,middlehkl[0],middlehkl[1],middlehkl[2]] ;-> Store widget IDs for enabling input in event handler

info.keep = 1 ;-> Keep Orientation by default, if orentation is altered, a new rotation tenser has to be calculated

crystalparmsbase_0 = widget_base(crystalwindow, row=4)
	label = widget_label(crystalparmsbase_0, value='Experimental Parameters', /align_left)
crystalparmsbase_1 = widget_base(crystalparmsbase_0, row=6, frame=1)
crystalparmsbase11 = widget_base(crystalparmsbase_1, column = 1, frame=1, xsize=170, /base_align_right)
	xfield = CW_Field(crystalparmsbase11, Title = 'X-ray wavelength:', uvalue = 'wavelength', value=info.wavelength, xsize=7, /floating, /all_events)
	thetafield = CW_Field(crystalparmsbase11, Title = '2 Theta Angle []:', uvalue = 'twotheta', value=*info.twotheta, xsize=7, /floating, /all_events)
	label = widget_label(crystalparmsbase11, value='(Set 2 Theta to 0 for no validation)', /align_right)
	label = widget_label(crystalparmsbase_1, value='Select range around 2 Theta', /align_left)
crystalparmsbase12 = widget_base(crystalparmsbase_1, column = 1, frame=1, xsize=170, /base_align_right)
	rangefield = CW_Field(crystalparmsbase12, Title = 'Range []:', uvalue = 'range', value=info.twothetarange[i], xsize=7, /floating, /all_events)
	label = widget_label(crystalparmsbase_1, value='Calculate higher harmonics?', /align_left)
;crystalparmsbase13 = widget_base(crystalparmsbase_1, column = 1, frame=1, /base_align_left)
crystalparmsbase13 = widget_base(crystalparmsbase_1, column = 1, frame=1, xsize=170, /exclusive)
		harm1 = widget_button(crystalparmsbase13, value='No', uvalue = 'L1')
		harm2 = widget_button(crystalparmsbase13, value='up to Lambda/2', uvalue = 'L2')
		harm3 = widget_button(crystalparmsbase13, value='up to Lambda/3', uvalue = 'L3')
		harm4 = widget_button(crystalparmsbase13, value='up to Lambda/4', uvalue = 'L4')
		harm5 = widget_button(crystalparmsbase13, value='up to Lambda/5', uvalue = 'L5')
		harm6 = widget_button(crystalparmsbase13, value='up to Lambda/6', uvalue = 'L6')
		harm7 = widget_button(crystalparmsbase13, value='up to Lambda/7', uvalue = 'L7')

crystalparmsbase14 = widget_base(crystalparmsbase_1, column = 1, frame=1, xsize=170, /base_align_right)
	labelfile = widget_label(crystalparmsbase14, value = ' ', /align_left, /dynamic_resize)
	labeldate = widget_label(crystalparmsbase14, value = ' ', /align_left, /dynamic_resize)
	labelname = widget_label(crystalparmsbase14, value = ' ', /align_left, /dynamic_resize)
crystalparmsbase15 = widget_base(crystalparmsbase_0)
	calculate = widget_button(crystalparmsbase15, value='Calculate the Crystal',$
						uvalue = 'calculate', xsize=170, /align_center)

;-> Realize

widget_Control, crystalwindow, /Realize

CASE info.higherharm[i] OF
	1: widget_control, harm1, set_button = 1
	2: widget_control, harm2, set_button = 1
	3: widget_control, harm3, set_button = 1
	4: widget_control, harm4, set_button = 1
	5: widget_control, harm5, set_button = 1
	6: widget_control, harm6, set_button = 1
	7: widget_control, harm7, set_button = 1
	ELSE:
ENDCASE

IF ((N_ELEMENTS(*info.header) ne 0) AND (info.rwatrigger EQ 1)) THEN BEGIN
	header = *info.header
	headerline1 = STRSPLIT(header[1], ',', /EXTRACT)
	headerline3 = STRSPLIT(header[2], ',', /EXTRACT)
		filename = headerline1[1]
		date = headerline1[3]
		SampleName = headerline3[1]
	label0 = string('Filename: ', strcompress(filename))
	label1 = string('File Date: ', strcompress(date))
	label3 = string('Sample Name: ', strcompress(SampleName))
	widget_control, labelfile, set_value = label0
	widget_control, labeldate, set_value = label1
	widget_control, labelname, set_value = label3
ENDIF

IF ((N_ELEMENTS(*info.header) eq 0) AND (info.rwatrigger EQ 1)) THEN BEGIN
  	label3 = string('No pole figure file loaded.')
  	widget_control, labelname, set_value = label3
ENDIF

IF (info.rwatrigger EQ 0) THEN BEGIN
  	label3 = string('No Philips *.RWA file loaded.')
  	widget_control, labelname, set_value = label3
ENDIF

widget_control, crystalwindow, Set_UValue=info, /No_Copy

XManager, 'stereopoleTable', crystalwindow, Event_Handler = 'UpdateFromTable', Cleanup='UpdateFromTableCleanup'
END

; ***********************************
PRO UpdateFromTable, event

widget_control, event.top, get_uvalue = info, /NO_Copy
widget_control, event.id, get_uvalue = eventID
widget_control, event.id, get_value = numbervalue

i = info.crystaltrigger - 1

CASE eventID OF
	'wavelength':	BEGIN
					info.wavelength = numbervalue
					widget_control, event.top, set_uvalue = info, /NO_Copy
					END
	'twotheta':	BEGIN
					*info.twotheta = numbervalue
					widget_control, event.top, set_uvalue = info, /NO_Copy
					END
	'range':	BEGIN
					info.twothetarange[i] = numbervalue
					widget_control, event.top, set_uvalue = info, /NO_Copy
					END
	'infotable':BEGIN
					widget_control, event.ID, get_value = man
					info.manualindex[*,*,i] = man ; V2
					; IF (i EQ 0) THEN info.manualindex1 = man ELSE info.manualindex2 = man ; V1
					widget_control, event.top, Set_UValue = info, /No_Copy
				END
	'hminin':	BEGIN
					info.hmin[i] = numbervalue
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'kminin':	BEGIN
					info.kmin[i] = numbervalue
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'lminin':	BEGIN
					info.lmin[i] = numbervalue
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'hmaxin':	BEGIN
					info.hmax[i] = numbervalue
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'kmaxin':	BEGIN
					info.kmax[i] = numbervalue
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'lmaxin':	BEGIN
					info.lmax[i] = numbervalue
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'a':		BEGIN
					IF (numbervalue GT 0) THEN BEGIN
						info.a[i] = numbervalue
					ENDIF ELSE BEGIN
					   info.a[i] = 1
					ENDELSE
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'b':		BEGIN
					IF (numbervalue GT 0) THEN BEGIN
						info.b[i] = numbervalue
					ENDIF ELSE BEGIN
						info.b[i] = 1
					ENDELSE
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'c':		BEGIN
				   IF (numbervalue GT 0) THEN BEGIN
						info.c[i] = numbervalue
					ENDIF ELSE BEGIN
						info.c[i] = 1
					ENDELSE
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'alpha':	BEGIN
				  IF (numbervalue GT 0) THEN BEGIN
						info.alpha[i] = numbervalue
					ENDIF ELSE BEGIN
						info.alpha[i] = 90
					ENDELSE
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'beta':		BEGIN
				    IF (numbervalue GT 0) THEN BEGIN
						info.beta[i] = numbervalue
					ENDIF ELSE BEGIN
						info.beta[i] = 90
					ENDELSE
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'gamma':	BEGIN
					IF (numbervalue GT 0) THEN BEGIN
						info.gamma[i] = numbervalue
					ENDIF ELSE BEGIN
						info.gamma[i] = 90
					ENDELSE
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'middleh':	BEGIN
					temp = *info.temp
					IF temp[3] NE numbervalue THEN BEGIN
						info.rottensor[*,*,i] =  DBLARR(3,3) + [[1,0,0],[0,1,0],[0,0,1]]
						info.middlehkl[0,i] = numbervalue
					ENDIF
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'middlek':	BEGIN
					temp = *info.temp
					IF temp[4] NE numbervalue THEN BEGIN
						info.rottensor[*,*,i] =  DBLARR(3,3) + [[1,0,0],[0,1,0],[0,0,1]]
						info.middlehkl[1,i] = numbervalue
					ENDIF
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'middlel':	BEGIN
					temp = *info.temp
					IF temp[5] NE numbervalue THEN BEGIN
						info.rottensor[*,*,i] =  DBLARR(3,3) + [[1,0,0],[0,1,0],[0,0,1]]
						info.middlehkl[2,i] = numbervalue
					ENDIF
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'manually':	BEGIN
					IF (info.autoindextrigger[i] NE 1) THEN info.autoindextrigger[i] = 1
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'byrange':	BEGIN
					IF (info.autoindextrigger[i] NE 2) THEN info.autoindextrigger[i] = 2
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'both':		BEGIN
					IF (info.autoindextrigger[i] NE 3) THEN info.autoindextrigger[i] = 3
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'L1':		BEGIN
					IF (info.higherharm[i] NE 1) THEN info.higherharm[i] = 1
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'L2':		BEGIN
					IF (info.higherharm[i] NE 2) THEN info.higherharm[i] = 2
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'L3':		BEGIN
					IF (info.higherharm[i] NE 3) THEN info.higherharm[i] = 3
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'L4':		BEGIN
					IF (info.higherharm[i] NE 4) THEN info.higherharm[i] = 4
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'L5':		BEGIN
					IF (info.higherharm[i] NE 5) THEN info.higherharm[i] = 5
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'L6':		BEGIN
					IF (info.higherharm[i] NE 6) THEN info.higherharm[i] = 6
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'L7':		BEGIN
					IF (info.higherharm[i] NE 7) THEN info.higherharm[i] = 7
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'keep':		BEGIN
					info.keep = 1
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'setnew':	BEGIN
					info.keep = 0
					temp = *info.temp
					afield = temp[0]
					bfield = temp[1]
					cfield = temp[2]
					WIDGET_CONTROL, afield, SENSITIVE=1
					WIDGET_CONTROL, bfield, SENSITIVE=1
					WIDGET_CONTROL, cfield, SENSITIVE=1
					widget_control, event.top, set_uvalue = info, /NO_Copy
				END
	'calculate':BEGIN

; IF (i EQ 0) THEN man = info.manualindex1 ELSE man = info.manualindex2 ; V1
man = REFORM(info.manualindex[*,*,i]) ; V2; Extract manual indices belonging to the certain crystal
trigger = info.autoindextrigger[i]

;-> If 'both' selected, no values in table cause no problem ...
;-> Error handling and generation of indices like in autoindex mode ...
;-> 'trigger' is local and valid for error handling, info.autoindextrigger is
;-> global and triggers generation in PRO UpdateFromTableCleanup

IF (man[0,0] EQ 0) AND (man[1,0] EQ 0) AND (man[2,0] EQ 0) THEN info.autoindextrigger[i]=2
IF (trigger EQ 3) THEN trigger = 2

	CASE trigger OF

	1:	BEGIN
		;-> manual input and values in table
		IF (NOT((man[0,0] EQ 0) AND (man[1,0] EQ 0) AND (man[2,0] EQ 0))) THEN BEGIN
			widget_control, info.mainwindow, set_uvalue = info
			widget_control, event.top, set_uvalue = info, /NO_Copy
			widget_control, event.top, /DESTROY
		BREAK
		ENDIF

		;-> manual input and nothing in table
		IF ((man[0,0] EQ 0) AND (man[1,0] EQ 0) AND (man[2,0] EQ 0)) THEN BEGIN
			errormsg = dialog_message('You have to insert at least one index', /ERROR)
		ENDIF
		widget_control, event.top, set_uvalue = info, /NO_Copy
		BREAK
		END

	2:	BEGIN
		;-> auto mode correct form
		IF (($
			((info.hmin[i] LE info.hmax[i]) AND (info.kmin[i] LE info.kmax[i]) AND (info.lmin[i] LE info.lmax[i])) AND $
			(NOT(((info.hmin[i] EQ 0) AND (info.kmin[i] EQ 0) AND (info.lmin[i] EQ 0)) OR $
			((info.hmax[i] EQ 0) AND (info.kmax[i] EQ 0) AND (info.lmax[i] EQ 0)))))) THEN BEGIN
				widget_control, info.mainwindow, set_uvalue = info
				widget_control, event.top, set_uvalue = info, /NO_Copy
				widget_control, event.top, /DESTROY
				BREAK
		ENDIF

		;-> auto mode illegal form, any (hkl)bound = 0
		IF (((info.hmin[i] EQ 0) AND (info.kmin[i] EQ 0) AND (info.lmin[i] EQ 0)) OR $
			((info.hmax[i] EQ 0) AND (info.kmax[i] EQ 0) AND (info.lmax[i] EQ 0))) THEN BEGIN
		errormsg = dialog_message('(hkl) bound with (000) not allowed', /ERROR)
		ENDIF

		;-> auto mode illegal form, at least one (hkl)min > (hkl)max
		IF ((info.hmin[i] GT info.hmax[i]) OR (info.kmin[i] GT info.kmax[i]) OR (info.lmin[i] GT info.lmax[i])) THEN BEGIN
		errormsg = dialog_message('(hkl)min > (hkl)max not allowed', /ERROR)
		ENDIF
		widget_control, event.top, set_uvalue = info, /NO_Copy
		BREAK
		END

	ELSE : print, event
	ENDCASE
	END
	ELSE :
	ENDCASE
END

; ***************
PRO UpdateFromTableCleanup, CrystalWindow

widget_control, hourglass = 1

vartype = SIZE(CrystalWindow)
;-> If called from Table procedure [if widget dies] vartype = 2
;-> If called from outside, CrystalWindow is the info structure
;-> and vartype = 8

IF (vartype[2] NE 8) THEN BEGIN
	widget_control, CrystalWindow, get_uvalue = info, /NO_Copy
	maybeupdatemore = 0 ; call doesn't come from outside
	IF N_ELEMENTS(*info.daten) EQ 0 THEN info.zoomtrigger = 5
ENDIF ELSE BEGIN
	event = CrystalWindow
	widget_control, event.top, get_uvalue = info, /NO_Copy
	maybeupdatemore = info.updatetrigger ;is set to 3 here

;-> To recalculate the already declared crystals if the call of UpdateFromTableCleanupis coming
;-> from the outside, it has to be checked which crystals are actually declared

	declaredbitmask = [0,0,0,0,0]
	IF (N_ELEMENTS(*info.allindices1) NE 0) THEN declaredbitmask = declaredbitmask + [1,0,0,0,0]
	IF (N_ELEMENTS(*info.allindices2) NE 0) THEN declaredbitmask = declaredbitmask + [0,1,0,0,0]
	IF (N_ELEMENTS(*info.allindices3) NE 0) THEN declaredbitmask = declaredbitmask + [0,0,1,0,0]
	IF (N_ELEMENTS(*info.allindices4) NE 0) THEN declaredbitmask = declaredbitmask + [0,0,0,1,0]
	IF (N_ELEMENTS(*info.allindices5) NE 0) THEN declaredbitmask = declaredbitmask + [0,0,0,0,1]
	IF N_ELEMENTS(*info.daten) EQ 0 THEN info.zoomtrigger = 5
	declaredsave = declaredbitmask

;-> if no indices are defined -> EXIT
cond = WHERE(declaredbitmask)
	IF (cond[0] LT 0) THEN BEGIN
		info.updatetrigger = 2
		widget_control, event.top, set_uvalue = info, /NO_Copy
		RETURN
	ENDIF
ENDELSE

;-> If widget died by closing the window, contents of manual indices have to be validated.
i = info.crystaltrigger - 1

IF (info.autoindextrigger[i] EQ 1) THEN BEGIN
	man = info.manualindex[*,*,i] ; V2
		IF ((man[0,0] EQ 0) AND (man[1,0] EQ 0) AND (man[2,0] EQ 0)) THEN BEGIN
			errormsg = dialog_message('You have to insert at least one index', /ERROR)
		GOTO, JUMPPOINT2
	ENDIF
ENDIF

;-> Now input is done ... calculate indices and store them

JUMPPOINT:

CASE info.autoindextrigger[i] OF

;-> Transformation of manual input
	1:	BEGIN
				temp = ManualGenerateHKL(info)
				IndicesStruct = ValidateHKLRange(info, temp)
				alltwotheta_local = IndicesStruct.all_twotheta
				allindices_local = IndicesStruct.IndicesInRange
				allharmonies_local = IndicesStruct.harmony
		END

;->Automatic generation

	2:	BEGIN
				temp = AutoGenerateHKL(info)
				IndicesStruct = ValidateHKLRange(info, temp)
				alltwotheta_local = IndicesStruct.all_twotheta
				allindices_local = IndicesStruct.IndicesInRange
				allharmonies_local = IndicesStruct.harmony
		END

;-> Both modes at one time

	3:  BEGIN
				manualpart = ManualGenerateHKL(info)
				autopart = AutoGenerateHKL(info)
				temp = [[manualpart], [autopart]]
				IndicesStruct = ValidateHKLRange(info, temp)
				alltwotheta_local = IndicesStruct.all_twotheta
				allindices_local = IndicesStruct.IndicesInRange
				allharmonies_local = IndicesStruct.harmony
		END
	ELSE : print, 'error'
ENDCASE

;-> Now store the data into the appropriate pointer variables

CASE info.crystaltrigger OF
1:	BEGIN
		*info.allindices1 = allindices_local
		*info.allharmonies1 = allharmonies_local
		*info.alltwotheta1 = alltwotheta_local
		*info.Vectors3D_1 = BuildVectors3D(info)
		*info.Vectors2D_1 = BuildVectors2D(info)

	END
2:	BEGIN
		*info.allindices2 = allindices_local
		*info.allharmonies2 = allharmonies_local
		*info.alltwotheta2 = alltwotheta_local
		*info.Vectors3D_2 = BuildVectors3D(info)
		*info.Vectors2D_2 = BuildVectors2D(info)
	END
3:	BEGIN
		*info.allindices3 = allindices_local
		*info.allharmonies3 = allharmonies_local
		*info.alltwotheta3 = alltwotheta_local
		*info.Vectors3D_3 = BuildVectors3D(info)
		*info.Vectors2D_3 = BuildVectors2D(info)
	END
4:	BEGIN
		*info.allindices4 = allindices_local
		*info.allharmonies4 = allharmonies_local
		*info.alltwotheta4 = alltwotheta_local
		*info.Vectors3D_4 = BuildVectors3D(info)
		*info.Vectors2D_4 = BuildVectors2D(info)
	END
5:	BEGIN
		*info.allindices5 = allindices_local
		*info.allharmonies5 = allharmonies_local
		*info.alltwotheta5 = alltwotheta_local
		*info.Vectors3D_5 = BuildVectors3D(info)
		*info.Vectors2D_5 = BuildVectors2D(info)
	END
ENDCASE

;-> If the call of the procedure UpdateFromTableCleanup is comming from ReadDataFile_FirstPlot,
;-> then info.updatetrigger = 3 and therefore maybeupdatemore too.
;-> In this case it is possible that already both crystals are declared, so that after having loaded
;-> a new pole figure both crystals have to be recalculated. By means of info.crystaltrigger the first one
;-> can be 1 or 2 has been calculated at that point so that the other has to follow

IF (maybeupdatemore EQ 3) THEN BEGIN
	declaredbitmask[i] = 0
	current = WHERE(declaredbitmask)
	IF current[0] GE 0 THEN BEGIN ; only the case if there are nonzero elements in declaredbitmask
		i = current[0] ; set the crystaltrigger to the lowest element
		info.crystaltrigger = i+1
		GOTO, JUMPPOINT ; recalculate for this specific crystal
	ENDIF ELSE BEGIN
		maybeupdatemore = 0
	ENDELSE
ENDIF

JUMPPOINT2:

IF (info.updatetrigger EQ 3) THEN BEGIN

;-> After recalculation of the crystals for the new pole figure, the projections
;-> have to be rotated to the former position

	stepsize = info.rotstepsize
	bitmask = info.rotatebitmask

	info.showbitmask[0] = 1

	FOR k=0,4 DO BEGIN
		IF (declaredsave[k] EQ 1) THEN BEGIN
				bitmask_temp = INTARR(6)
				bitmask_temp[k+1] = 1
				info.rotatebitmask = bitmask_temp
				widget_control, event.top, set_uvalue = info, /No_Copy
				RotateStereo, event, /recalculated
				widget_control, event.top, get_uvalue = info, /No_Copy
		ENDIF
	ENDFOR

	info.rotstepsize = stepsize
	info.rotatebitmask = bitmask
	info.updatetrigger = 0

	widget_control, info.plotwindow, DRAW_MOTION_EVENTS=1
	widget_control, info.plotwindow, DRAW_BUTTON_EVENTS=1
	widget_control, event.top, set_uvalue = info, /No_Copy

ENDIF ELSE BEGIN

info.updatetrigger = 2

widget_control, info.plotwindow, DRAW_MOTION_EVENTS=1
widget_control, info.plotwindow, DRAW_BUTTON_EVENTS=1
widget_control, info.mainwindow, set_uvalue = info, /No_Copy

ENDELSE
END

; *************************** Prepare Data for plot ***********************************
Pro PrepareData, event
widget_control, event.top, get_uvalue = info, /NO_Copy

daten = *info.daten
numberoflevels = info.numberoflevels

*info.maxint = MAX(daten[2,*])
*info.minint = MIN(daten[2,*])
psiend = MAX(daten[1,*])

phiwerte = daten[0,*]*!PI/180
psiwerte = daten[1,*]*!PI/180
intwerte = daten[2,*]

X = Sin(psiwerte) * Cos(phiwerte) / (1 + Cos(psiwerte))
Y = Sin(psiwerte) * Sin(phiwerte) / (1 + Cos(psiwerte))
Z = daten[2,*]

*info.psiend = psiend
*info.X = X
*info.Y = Y
*info.Z = Z
*info.radius = MAX(X)

widget_control, event.top, set_uvalue = info, /NO_Copy
END

; ******************************** Rotate Pole Figure ***********************************
PRO RotatePoleFigure, event, left=left, right=right, samefigure=samefigure
widget_control, event.top, get_uvalue = info, /NO_Copy

IF (N_ELEMENTS(*info.daten) EQ 0) OR (info.rotatebitmask[0] EQ 0) THEN BEGIN
	widget_control, event.top, set_uvalue = info, /NO_Copy
	RETURN
ENDIF

dimtrigger = info.dimtrigger
daten = *info.daten

;-> rotation of
IF KEYWORD_SET(left) THEN BEGIN
	rotangle = info.rotstepsize
	IF KEYWORD_SET(samefigure) THEN BEGIN
		info.nettoangle[0] = info.nettoangle[0] - info.rotstepsize
	ENDIF ELSE BEGIN
		rotangle = -info.nettoangle[0]
	ENDELSE
ENDIF

IF KEYWORD_SET(right) THEN BEGIN
	rotangle = -info.rotstepsize
	IF KEYWORD_SET(samefigure) THEN BEGIN
		info.nettoangle[0] = info.nettoangle[0] + info.rotstepsize
	ENDIF ELSE BEGIN
		rotangle = -info.nettoangle[0]
	ENDELSE
ENDIF

;-> Now the phi values of the data are de/increased by rotstepsize, where
;-> negative degrees in phi have to be avoided [by IDL array operations]
phi = daten[0,*] + rotangle
length = N_ELEMENTS(daten[0,*])
zerophi = intarr(length)
maskLT = daten[0,*] LT zerophi
maskGE = daten[0,*] GE zerophi
workarray = 360 * maskLT
daten[0,*] = (workarray + (maskLT * phi)) OR (maskGE * phi)

*info.daten = daten
zoomed = info.zoomed

widget_control, event.top, set_uvalue = info, /NO_Copy
PrepareData, event
;ShowMain, event
END

; ****************************** Display Pole Figure 2D *********************************
PRO Zeichnen_xyz_reset, event
widget_control, event.top, get_uvalue = info, /NO_Copy

info.dimtrigger = 1

IF ((info.modetrigger EQ 1) OR (N_ELEMENTS(*info.daten) EQ 0)) THEN BEGIN
	info.zoomtrigger = 5 ;means unzoom the epitaxial mode plot (or no pole figure loaded)
	widget_control, event.top, set_uvalue = info, /NO_Copy
ENDIF ELSE BEGIN
	IF (N_ELEMENTS(*info.daten) EQ 0) THEN BEGIN
		widget_control, event.top, set_uvalue = info, /NO_Copy
		RETURN
	ENDIF

	*info.xmin = -MAX(*info.X)
	*info.xmax = MAX(*info.X)
	*info.ymin = -MAX(*info.Y)
	*info.ymax = MAX(*info.Y)
	*info.minint = MIN(*info.Z)
	*info.maxint = MAX(*info.Z)

	widget_Control, info.minintID, set_value = *info.minint
	widget_Control, info.maxintID, set_value = *info.maxint

	widget_control, (info.showopt).showopt1, set_button = 1
	info.showbitmask[0] = 1
	widget_control, event.top, set_uvalue = info, /NO_Copy
ENDELSE

ShowMain, event
END

; *************
PRO Zeichnen_xyz, event, zoomin = zoomin
widget_control, event.top, get_uvalue = info, /NO_Copy
widget_control, info.plotwindow, DRAW_MOTION_EVENTS=1
widget_control, info.plotwindow, DRAW_BUTTON_EVENTS=1

IF (N_ELEMENTS(*info.daten) EQ 0) THEN BEGIN
widget_control, event.top, set_uvalue = info, /NO_Copy
RETURN
ENDIF

daten = *info.daten
X = *info.X
Y = *info.Y
Z = *info.Z
nettrigger = info.nettrigger
info.dimtrigger = 1
numberoflevels = info.numberoflevels

xmin = *info.xmin
xmax = *info.xmax
ymin = *info.ymin
ymax = *info.ymax

fitfunct=(FINDGEN(numberoflevels)+1)/(numberoflevels+1)
IF (N_ELEMENTS(*info.minint) EQ 0) THEN minint = MIN(Z) ELSE minint = *info.minint
IF (N_ELEMENTS(*info.maxint) EQ 0) THEN maxint = MAX(Z) ELSE maxint = *info.maxint
levels=[minint,fitfunct*(maxint - minint)+minint,maxint]
*info.levels = levels

;-> plot zoomed
IF KEYWORD_SET(zoomin) THEN BEGIN

start = CONVERT_COORD(info.clickx1, info.clicky1, /DEVICE, /TO_DATA)
ende = CONVERT_COORD(info.clickx2, info.clicky2, /DEVICE, /TO_DATA)

	IF (info.clickx1 LE info.clickx2) THEN BEGIN
		xmin = start[0]
		xmax = ende[0]
	ENDIF ELSE BEGIN
		xmin = ende[0]
		xmax = start[0]
	ENDELSE

	IF (info.clicky1 LE info.clicky2) THEN BEGIN
		ymin = start[1]
		ymax = ende[1]
	ENDIF ELSE BEGIN
		ymin = ende[1]
		ymax = start[1]
	ENDELSE

*info.xmin = xmin
*info.xmax = xmax
*info.ymin = ymin
*info.ymax = ymax

ENDIF

;- alternating labeling of contour levels if desired
	IF (info.nthline EQ 0) THEN c_labels = [0] ELSE BEGIN
		c_labels = INTARR(info.numberoflevels+2)
		FOR i=0,(info.numberoflevels+1), info.nthline DO C_LABELS[i]=1
	ENDELSE

;-> plot pole figure - contours
IF (info.linetrigger NE 1) THEN BEGIN
	IF (!D.Name EQ 'PS') THEN BEGIN
		Contour, z,x,y, /irregular, levels=levels,$
				xrange=[xmin,xmax],yrange=[ymin,ymax],$
				xcharsize=4, ycharsize=4, xstyle=4, ystyle=4, /CELL_FILL, /NoClip, /ISOTROPIC
				; Keyword CELL_FILL has to be used in PostScript mode to handle missing [: clipped]
				; data correctly. Using /FILL yields filling of these areas with the lowest color index.
	ENDIF ELSE BEGIN
		Contour, z,x,y, /irregular, levels=levels,$
				xrange=[xmin,xmax],yrange=[ymin,ymax],$
				xcharsize=4, ycharsize=4, xstyle=4, ystyle=4, /FILL, /NoClip, /ISOTROPIC
	ENDELSE
ENDIF

;-> plot pole figure - only lines
IF (info.linetrigger EQ 1) THEN BEGIN

	Contour, z,x,y, /irregular, levels=levels, c_labels=c_labels,$
			xrange=[xmin,xmax],yrange=[ymin,ymax],$
			xcharsize=4, ycharsize=4, xstyle=4, ystyle=4, /NoClip, /ISOTROPIC
ENDIF

;-> overplot it with contour lines if desired - plotting both
IF (info.linetrigger EQ 2) THEN BEGIN

	Contour, z,x,y, /irregular, levels=levels, c_labels=c_labels,$
			xrange=[xmin,xmax],yrange=[ymin,ymax], /overplot, /NoClip, /ISOTROPIC
ENDIF

;-> Draw Net

widget_control, (info.showopt).showopt1, set_button = 1
info.showbitmask[0] = 1

widget_control, event.top, set_uvalue = info, /NO_Copy

END

; ************************************ Draw Polar Net  ****************************************
PRO PolarNet, event, epitaxial=epitaxial
widget_control, event.top, get_uvalue = info, /NO_Copy

colorlist = *info.colorlist

IF (N_ELEMENTS(*info.daten) EQ 0) AND $
	(N_ELEMENTS(*info.allindices1) EQ 0) AND $
	(N_ELEMENTS(*info.allindices2) EQ 0) THEN BEGIN
	widget_control, event.top, set_uvalue = info, /NO_Copy
	RETURN
ENDIF

degreestep = info.degreestep

IF (KEYWORD_SET(epitaxial) OR (N_ELEMENTS(*info.daten) EQ 0)) THEN BEGIN
	range = CONVERT_COORD(1, 1, /NORMAL, /TO_DATA)
	r = (indgen(90/degreestep)+1)*degreestep
	r = Sin(r*!PI/180)/(1+Cos(r*!PI/180))

	i=0
	IF (!D.NAME EQ 'PS') THEN BEGIN
		FOR i=0, N_ELEMENTS(r)-1 DO BEGIN
			PlotS, CIRCLE(0,0,r[i]), /NOCLIP, color=colorlist[5], THICK=5
		ENDFOR
		;-> Plot a cross throu (0,0)
		PlotS, [-1,1], [0,0], /NOCLIP, color=colorlist[5], THICK=5
		PlotS, [0,0], [-1,1], /NOCLIP, color=colorlist[5], THICK=5
	ENDIF ELSE BEGIN
		FOR i=0, N_ELEMENTS(r)-1 DO BEGIN
			PlotS, CIRCLE(0,0,r[i]), /NOCLIP
		ENDFOR
		PlotS, [-1,1], [0,0], /NOCLIP
		PlotS, [0,0], [-1,1], /NOCLIP
	ENDELSE

ENDIF ELSE BEGIN

	psiend = *info.psiend
	X = *info.X

	r = (indgen(90/degreestep)+1)*degreestep
	r = Sin(r*!PI/180)/(1+Cos(r*!PI/180))

	;-> do not draw circles outside of the maximum x,y in the file
	IF (*info.xmax GT MAX(*info.X)) THEN *info.xmax = MAX(*info.X)
	IF (*info.xmin LT -MAX(*info.X)) THEN *info.xmin = -MAX(*info.X)
	IF (*info.ymax GT MAX(*info.Y)) THEN *info.ymax = MAX(*info.Y)
	IF (*info.xmax LT -MAX(*info.Y)) THEN *info.xmin = -MAX(*info.Y)

	dummy = Abs([*info.xmax, *info.ymax, *info.xmin, *info.ymin])
	dummyold = [MAX(*info.X), MAX(*info.Y)]
	rmax = Max(dummy)
	rmaxold = Max(dummyold)

	;-> Plot outer circle if visible
	IF (!D.NAME EQ 'PS') THEN BEGIN
		IF (rmax GT rmaxold*0.99) THEN PlotS, CIRCLE(0,0,MAX(*info.X)), /NOCLIP, color=colorlist[5], THICK=5
	ENDIF ELSE BEGIN
		IF (rmax GT rmaxold*0.99) THEN PlotS, CIRCLE(0,0,MAX(*info.X)), /NOCLIP
	ENDELSE

	rmax = Max(dummy)*2

	i=0
	IF (!D.NAME EQ 'PS') THEN BEGIN
		WHILE ((r[i] lt rmax) AND (r[i] lt rmaxold)) DO BEGIN
		PlotS, CIRCLE(0,0,r[i]), /NOCLIP, color=colorlist[5], THICK=5
		i=i+1
		ENDWHILE
		;-> Plot a cross throu (0,0)
		PlotS, [-Max(X),Max(X)], [0,0], /NOCLIP, color=colorlist[5], THICK=5
		PlotS, [0,0], [-Max(X),Max(X)], /NOCLIP, color=colorlist[5], THICK=5
	ENDIF ELSE BEGIN
		WHILE ((r[i] lt rmax) AND (r[i] lt rmaxold)) DO BEGIN
		PlotS, CIRCLE(0,0,r[i]), /NOCLIP
		i=i+1
		ENDWHILE
		;-> Plot a cross throu (0,0)
		PlotS, [-Max(X),Max(X)], [0,0], /NOCLIP
		PlotS, [0,0], [-Max(X),Max(X)], /NOCLIP
	ENDELSE

ENDELSE

widget_control, event.top, set_uvalue = info, /NO_Copy
END

; ************************ Functions for Wulff Net *********************
FUNCTION BIGCIRCLE, t
grosskreis = Transpose([[0*t], [Cos(t)], [Sin(t)]])
RETURN, grosskreis
END

; **************
FUNCTION SMALLCIRCLE, i, t
IF (i GT 0) THEN j = 1 ELSE j = -1
kleinkreis = ([[Sin(Abs(i)) * Cos(t)],$
						[j * Cos(Abs(i)) + 0 * t],$
			 			[Sin(Abs(i)) * Sin(t)]])
RETURN, kleinkreis
END

; **************
PRO STEREONET, event, colorlist

a = (indgen(181))*!DTOR
b = BIGCIRCLE(a) ;[3D points of a big circle]

bigarr = FltArr(181,3,19) ;180 points with 3 dimensions per one of the 19 lines
FOR i=0,18 DO BEGIN
	FOR j=0,180 DO BEGIN
		bigarr[j,*,i] = YROT((i-9)*10*!DTOR) # b[*,j]
	ENDFOR
ENDFOR
bigarr2d = [[bigarr[*,0,*]/(1+bigarr[*,2,*])], [bigarr[*,1,*]/(1+bigarr[*,2,*])]]
; ***

smallarr = FltArr(181,3,18)
k=0
FOR i = -80,80,10 DO BEGIN
	dummy = i*!DTOR
	tmp = SMALLCIRCLE(dummy,a)
	smallarr[*,*,k] = tmp
	k=k+1
ENDFOR

;-> Add a Circle through phi=0
thruzero = [[Cos(a)],[0*a],[Sin(a)]]
smallarr = [[[smallarr]],[[thruzero]]]
smallarr2d = [[smallarr[*,0,*]/(1+smallarr[*,2,*])], [smallarr[*,1,*]/(1+smallarr[*,2,*])]]

IF (!D.NAME EQ 'PS') THEN BEGIN
	FOR i=0,18 DO BEGIN
		plots, Transpose(bigarr2D[*,*,i]), /NOCLIP, color = colorlist[5], THICK=5
	ENDFOR
	FOR i=0,18 DO BEGIN
		plots, Transpose(smallarr2d[*,*,i]), /NOCLIP, color = colorlist[5], THICK=5
	ENDFOR
ENDIF ELSE BEGIN
	FOR i=0,18 DO BEGIN
		plots, Transpose(bigarr2D[*,*,i]), /NOCLIP
	ENDFOR
	FOR i=0,18 DO BEGIN
		plots, Transpose(smallarr2d[*,*,i]), /NOCLIP
	ENDFOR
ENDELSE
END

; ************************************* Zoom Procedure ****************************************
PRO Zoom, event

Showmain, event
XYOUTS, 15, 15, 'Mark rectangle to zoom by two clicks ...', /Device

widget_control, event.top, get_uvalue = info, /NO_Copy
info.zoomtrigger = 0
DEVICE, CURSOR_STANDARD = 32513
widget_control, event.top, set_uvalue = info, /NO_Copy

END

; ************
PRO PerformZoom, event
widget_control, event.top, get_uvalue = info, /NO_Copy

IF (info.zoomtrigger EQ 2) THEN BEGIN
;-> Draw square bevore zoom in for one second ...
PlotS,	[info.clickx1,info.clickx1,info.clickx2,info.clickx2,info.clickx1],$
		[info.clicky1,info.clicky2,info.clicky2,info.clicky1,info.clicky1],$
		color=255, /device
wait, 1
IF ((info.modetrigger EQ 1) OR (N_ELEMENTS(*info.daten) EQ 0)) THEN info.zoomtrigger = 4 ELSE info.zoomtrigger = 3
info.zoomed = 1
widget_control, event.top, set_uvalue = info, /NO_Copy
DEVICE, /CURSOR_CROSSHAIR

ShowMain, event, /zoomin
ENDIF
END

; *********************************** Change Color Table **************************************
PRO Colortable, event

xloadct, group=event.top, updatecallback='Colortable_redraw', updatecbdata = event

END

; **************
PRO Colortable_redraw, data=event

widget_control, event.top, get_uvalue = info, /NO_Copy
dimtrigger = info.dimtrigger
widget_control, event.top, set_uvalue = info, /NO_Copy
ShowMain, event
END

;***************
PRO CallPrintwindow, event

widget_control, event.top, get_uvalue = info, /NO_Copy
wid=info.plotwindowind
widget_control, event.top, set_uvalue = info, /NO_Copy

PRINTWINDOW, wid, PageSize='A4'
END

; ********************************* Display Pole Figure 3D ************************************
PRO Zeichnen_3d, event
widget_control, event.top, get_uvalue = info, /NO_Copy

IF (N_ELEMENTS(*info.daten) EQ 0) THEN BEGIN
widget_control, event.top, set_uvalue = info, /NO_Copy
RETURN
ENDIF

daten = *info.daten
X = *info.X
Y = *info.Y
Z = *info.Z
res3d = *info.res3d

info.dimtrigger = 2

phiwerte = daten[0,*]*!PI/180
psiwerte = daten[1,*]*!PI/180
intwerte = daten[2,*]

j=Max(X)
X=X-j
Y=Y-j

;-> Derive the Delaunay triangulation of the irregularly-gridded data

TRIANGULATE,X,Y,TR

;-> Calculate regular grid of interpolated Z values
;-> This is only required for Shade_Surf procedure i.e. 3D view

regular = TRIGRID(x,y,z,tr,/quintic, nx=res3d, ny=res3d)

IF (!D.NAME EQ 'PS') THEN BEGIN
	TVLCT, r, g, b, /Get
	;-> Color Table has to be reverted for PostScript Output
	TVLCT, Reverse(r), Reverse(g), Reverse(b)
	;-> Create Shaded Surface representation of pole figure for Postscript
	IF (N_ELEMENTS(*info.maxint) EQ 0) THEN maxint = MAX(Z) ELSE maxint = *info.maxint
	Shade_SURF, regular, zrange=[0,maxint], THICK = 10, CHARSIZE=2, CHARTHICK=5, /NORMAL
	;-> Color Table is reverted to original values again
	TVLCT, r, g, b
ENDIF ELSE BEGIN
	;-> Create Shaded Surface representation of pole figure - NonPostscript Output
	IF (N_ELEMENTS(*info.maxint) EQ 0) THEN maxint = MAX(Z) ELSE maxint = *info.maxint
	Shade_SURF, regular, zrange=[0,maxint]
ENDELSE
widget_control, event.top, set_uvalue = info, /NO_Copy
END

; ********************************** Image Histogram ************************************
PRO HistoPole, event
widget_control, event.top, get_uvalue = info, /NO_Copy

IF (N_ELEMENTS(*info.daten) EQ 0) THEN BEGIN
	widget_control, event.top, set_uvalue = info, /NO_Copy
	RETURN
ENDIF ELSE BEGIN

widget_control, info.plotwindow, DRAW_MOTION_EVENTS=0
widget_control, info.plotwindow, DRAW_BUTTON_EVENTS=0

IF (N_ELEMENTS(*info.minint) EQ 0) THEN minimum = MIN(*info.Z) ELSE minimum = *info.minint
IF (N_ELEMENTS(*info.maxint) EQ 0) THEN maximum = MAX(*info.Z) ELSE maximum = *info.maxint

y = Histogram(*info.Z, min = minimum, max = maximum, nbins = info.numberoflevels) ;binsize=(maximum-minimum)/info.numberoflevels)
x = lindgen(info.numberoflevels+1) * ((maximum-minimum)/info.numberoflevels) +  minimum

plot, x,y, title='Pole Figure Intensity Histogram', xrange=[MIN(x), MAX(x)], xtitle='Intensity [cps]', ytitle='amount', psym=10, xstyle=1

widget_control, event.top, set_uvalue = info, /NO_Copy
ENDELSE
END

; ********************************** Virtual PSI Scan ************************************
PRO VirtualPsiScan, event
widget_control, event.top, get_uvalue = info, /NO_Copy

IF (N_ELEMENTS(*info.daten) EQ 0) THEN BEGIN
	widget_control, event.top, set_uvalue = info, /NO_Copy
	RETURN
ENDIF ELSE BEGIN

info.dimtrigger = 3

widget_control, info.plotwindow, DRAW_MOTION_EVENTS=0
widget_control, info.plotwindow, DRAW_BUTTON_EVENTS=0

data = *info.daten
start = *info.psistart
ende = *info.psiend
step = *info.psistep

nop = (ende-start)/step

x = FLTARR(nop+1)
y = FLTARR(nop+1)

FOR i=0,nop DO BEGIN
 x[i] = start+i*step
 y[i] = TOTAL(data[2,WHERE(data[1,*] EQ start+i*step)])
ENDFOR

plot, x,y, title='Integrated PSI Scan of Pole Figure', xrange=[MIN(data[1,*]), MAX(data[1,*])], yrange=[MIN(y), MAX(y)], xtitle='PSI []', ytitle='Intensity [cps]', psym=1, xstyle=1
oplot, x,y

widget_control, event.top, set_uvalue = info, /NO_Copy
ENDELSE
END


; **************************** Save Window as PostScript *************************************
PRO OutputPS, event

widget_control, event.top, get_uvalue = info, /NO_Copy
widget_control, info.plotwindow, DRAW_MOTION_EVENTS=0
widget_control, info.plotwindow, DRAW_BUTTON_EVENTS=0
thisDevice=info.thisDevice
thisPath=info.path
widget_control, event.top, set_uvalue = info, /NO_Copy

Set_Plot, 'PS', /COPY

Catch, theError
IF theError NE 0 THEN BEGIN
	Catch, /Cancel
  	Set_Plot, thisDevice
  	widget_control, event.top, get_uvalue = info, /NO_Copy
	widget_control, info.plotwindow, DRAW_MOTION_EVENTS=1
	widget_control, info.plotwindow, DRAW_BUTTON_EVENTS=1
	info.path=thisPath
	widget_control, event.top, set_uvalue = info, /NO_Copy
  	RETURN
ENDIF

Device, filename=Dialog_Pickfile(Filter='*.ps',/Write, PATH=thisPath, GET_PATH=thePath), xsize=100, ysize=100
Device, /helvetica, font_size=40
Device, /color, bits_per_pixel=8

ShowMain, event

Device, /Close_File
Set_Plot, thisDevice

widget_control, event.top, get_uvalue = info, /NO_Copy
widget_control, info.plotwindow, DRAW_MOTION_EVENTS=1
widget_control, info.plotwindow, DRAW_BUTTON_EVENTS=1
info.path = thePath ; Store current File Path for next data input
widget_control, event.top, set_uvalue = info, /NO_Copy

END

; ************************************ SaveImage *****************************************
PRO SaveImageAs, event

Device, Get_Visual_Depth = VisDepth, Get_Decomposed = DecopState
widget_control, event.id, get_value = ButtonValue, get_uvalue = FileExtension

widget_control, event.top, get_uvalue = info, /No_Copy
daten = *info.daten
thisPath=info.path
thisDevice=info.thisDevice
widget_control, event.top, set_uvalue = info, /No_Copy

Catch, theError
IF theError NE 0 THEN BEGIN
	Catch, /Cancel
  	Set_Plot, thisDevice
  	widget_control, event.top, get_uvalue = info, /NO_Copy
	widget_control, info.plotwindow, DRAW_MOTION_EVENTS=1
	widget_control, info.plotwindow, DRAW_BUTTON_EVENTS=1
	info.path=thisPath
	widget_control, event.top, set_uvalue = info, /NO_Copy
  	RETURN
ENDIF

IF (N_ELEMENTS(daten) NE 0) THEN BEGIN

	InitFilename = '*' + FileExtension

	;** Postscript **

	IF ButtonValue EQ 'Postscript File' THEN BEGIN
		OutputPS, event
		RETURN
	ENDIF ELSE BEGIN
		filename = Dialog_Pickfile(File=InitFilename, Filter = FileExtension, /WRITE, PATH=thisPath)
		IF filename EQ "" THEN RETURN
	ENDELSE

	;** everything is ok ... write TIFF or JPEG File **

	widget_control, event.top, get_uvalue = info, /No_Copy
    info.path=thisPath

	CASE ButtonValue OF
		'TIFF File':BEGIN
						IF (VisDepth GT 8) THEN BEGIN
							; color indices will be interpreted as 3, 8-bit color indices [rgb]
							Device, Decomposed=1
							; Take a snapshot of the true color graphics window
							image24 = TVRD(True=1)
							Device, Decomposed=DecopState
						ENDIF ELSE BEGIN
							; Take a snapshot in 8 bit color mode
							image8 = TVRD()
							; return RGB values from the internal color table into the r,g,b variables
							TVLCT, r,g,b, /Get
							; now create a 24-bit image out of the image array and the color vectors
							s = Size(image8, /Dimensions)
							image24 = BytArr(3,s[0],s[1])
							image24[0,*,*] = r[image8]
							image24[1,*,*] = g[image8]
							image24[2,*,*] = b[image8]
						ENDELSE
						Write_TIFF, filename, Reverse(Temporary(image24),3)
					END

		'JPEG File':BEGIN
							IF (VisDepth GT 8) THEN BEGIN
							Device, Decomposed=1
							image24 = TVRD(True=1)
							Device, Decomposed=DecopState
						ENDIF ELSE BEGIN
							image8 = TVRD()
							TVLCT, r,g,b, /Get
							s = Size(image8, /Dimensions)
							image24 = BytArr(3,s[0],s[1])
							image24[0,*,*] = r[image8]
							image24[1,*,*] = g[image8]
							image24[2,*,*] = b[image8]
						ENDELSE
						Write_JPEG, filename, image24, True=1, Quality=100
					END
		ELSE: print,'ELSE !!!'
	ENDCASE

ENDIF
widget_control, event.top, set_uvalue = info, /No_Copy

END

; **************************** Load Crystal Data ***********************
PRO LoadCrystal, event
widget_control, event.top, get_uvalue = info, /No_Copy
thisPath=info.path

Catch, theError
IF theError NE 0 THEN BEGIN
	Catch, /Cancel
	errormsg = dialog_message('Crystal not loaded.', /ERROR)
	info.path=thisPath
	Widget_Control, event.top, Set_UValue=info, /No_Copy
	RETURN
ENDIF

i = *info.temp-1 ; number of crystal
filename = Dialog_Pickfile(File=InitFilename, Filter = "*.cry", /READ, PATH=info.path, GET_PATH=thePath)
info.path = thePath ; Store current File Path for next data input
RESTORE, filename

info.a[i]=a
info.b[i]=b
info.c[i]=c
info.alpha[i]=alpha
info.beta[i]=beta
info.gamma[i]=gamma
info.hmin[i]=hmin
info.kmin[i]=kmin
info.lmin[i]=lmin
info.hmax[i]=hmax
info.kmax[i]=kmax
info.lmax[i]=lmax
info.manualindex[*,*,i]=manualindex
info.middlehkl[*,i]=middlehkl
info.middleuvw[*,i]=middleuvw

info.crystaltrigger = i+1
widget_control, event.top, set_uvalue = info, /No_Copy
stereopoleTable, event

END

; **************************** Save Crystal Data ***********************
PRO SaveCrystal, event
widget_control, event.top, get_uvalue = info, /No_Copy
thisPath=info.path

Catch, theError
IF theError NE 0 THEN BEGIN
	Catch, /Cancel
	errormsg = dialog_message('Crystal not saved.', /ERROR)
	info.path=thisPath
	Widget_Control, event.top, Set_UValue=info, /No_Copy
	RETURN
ENDIF
i = *info.temp-1 ; number of crystal
InitFilename = '*.cry'

a=info.a[i]
b=info.b[i]
c=info.c[i]
alpha=info.alpha[i]
beta=info.beta[i]
gamma=info.gamma[i]
hmin=info.hmin[i]
kmin=info.kmin[i]
lmin=info.lmin[i]
hmax=info.hmax[i]
kmax=info.kmax[i]
lmax=info.lmax[i]
manualindex=info.manualindex[*,*,i]
middlehkl=info.middlehkl[*,i]
middleuvw=info.middleuvw[*,i]

SAVE,	a,b,c,alpha,beta,gamma,hmin,kmin,lmin,hmax,kmax,lmax,$
		manualindex,middlehkl,middleuvw,$
		filename = Dialog_Pickfile(File=InitFilename, Filter = "*.cry", /WRITE, PATH=info.path)

widget_control, event.top, set_uvalue = info, /No_Copy
END

; **************************** Show current HKL and 2 Theta angles ***********************
PRO HKL_list, event

widget_control, event.top, get_uvalue = info

listwindow = widget_base(mbar=mbar, title = 'HKL List of Indices in Range')

textID = widget_text(listwindow, xsize=60, ysize=40, Font='Courier',/EDITABLE, /SCROLL)
widget_Control, listwindow, /Realize

l1=' HKL List - Crystal 1'
l6=' HKL List - Crystal 2'
l7=' HKL List - Crystal 3'
l8=' HKL List - Crystal 4'
l9=' HKL List - Crystal 5'
l2=' --------------------'
l3=' '
l4=' 2 Theta [] | (hkl) | harmony | psi[] | phi[]'
l5=' -----------------------------------------------'
l10=' STEREOPOLE - HKL-List
l11=' (hkl) devided by its greatest common devisor,'
l12=' deselect at [Preferences -> GCD] to avoid this.'

header0 = ''
header1 = [[l1],[l2],[l3],[l4],[l5]]
header2 = [[l3],[l3],[l6],[l2],[l3],[l4],[l5]]
header3 = [[l3],[l3],[l7],[l2],[l3],[l4],[l5]]
header4 = [[l3],[l3],[l8],[l2],[l3],[l4],[l5]]
header5 = [[l3],[l3],[l9],[l2],[l3],[l4],[l5]]
gcdheader = [[l10],[l5],[l11],[l12],[l5],[l3]]
text = ''
notdefinedtrigger = 0 ; means crystal is expected as defined as default
hkllist = ''
IF info.apply_gcd EQ 1 THEN header0 = [gcdheader] ELSE header0 = [[l10],[l5],[l3]]

FOR m=1,5 DO BEGIN
	CASE m OF
		1:	BEGIN
				hkllist = ''
				IF (N_ELEMENTS(*info.allindices1) NE 0) THEN BEGIN
					recvectors = (*info.Vectors2D_1).recvec2d
					allindices = *info.allindices1
					allharmonies = *info.allharmonies1
					all_twotheta = *info.alltwotheta1
					recmetric = info.recmetric[*,*,0]
					notdefinedtrigger = 0
				ENDIF ELSE BEGIN
					notdefinedtrigger = 1
				ENDELSE
				header = [[header0],[header1]]
				notdefinedstring = ' Crystal 1 or range not defined yet.'
			END
		2:	BEGIN
				hkllist = ''
				IF (N_ELEMENTS(*info.allindices2) NE 0) THEN BEGIN
					recvectors = (*info.Vectors2D_2).recvec2d
					allindices = *info.allindices2
					allharmonies = *info.allharmonies2
					all_twotheta = *info.alltwotheta2
					recmetric = info.recmetric[*,*,1]
					notdefinedtrigger = 0
				ENDIF ELSE BEGIN
					notdefinedtrigger = 1
				ENDELSE
				header = header2
				notdefinedstring = ' Crystal 2 or range not defined yet.'
			END
		3:	BEGIN
				hkllist = ''
			   	IF (N_ELEMENTS(*info.allindices3) NE 0) THEN BEGIN
					recvectors = (*info.Vectors2D_3).recvec2d
					allindices = *info.allindices3
					allharmonies = *info.allharmonies3
					all_twotheta = *info.alltwotheta3
					recmetric = info.recmetric[*,*,2]
					notdefinedtrigger = 0
				ENDIF ELSE BEGIN
					notdefinedtrigger = 1
				ENDELSE
				header = header3
				notdefinedstring = ' Crystal 3 or range not defined yet.'
			END
		4:	BEGIN
				hkllist = ''
				IF (N_ELEMENTS(*info.allindices4) NE 0) THEN BEGIN
					recvectors = (*info.Vectors2D_4).recvec2d
					allindices = *info.allindices4
					allharmonies = *info.allharmonies4
					all_twotheta = *info.alltwotheta4
					recmetric = info.recmetric[*,*,3]
					notdefinedtrigger = 0
				ENDIF ELSE BEGIN
					notdefinedtrigger = 1
				ENDELSE
				header = header4
				notdefinedstring = ' Crystal 4 or range not defined yet.'
			END
		5:	BEGIN
				hkllist = ''
			  	IF (N_ELEMENTS(*info.allindices5) NE 0) THEN BEGIN
					recvectors = (*info.Vectors2D_5).recvec2d
					allindices = *info.allindices5
					allharmonies = *info.allharmonies5
					all_twotheta = *info.alltwotheta5
					recmetric = info.recmetric[*,*,4]
					notdefinedtrigger = 0
				ENDIF ELSE BEGIN
					notdefinedtrigger = 1
				ENDELSE
				header = header5
				notdefinedstring = ' Crystal 5 or range not defined yet.'
			END
	ENDCASE

IF (notdefinedtrigger NE 1) THEN BEGIN
	; hklliststring = hkllist

; ***** Create string output

	x = recvectors[0,*]
	y = recvectors[1,*]

	psi = ACos((1 - x^2 - y^2)/(1 + x^2 + y^2))*180/!Pi
	phi = ATan(y/x)*180/!Pi

	FOR i=0,N_ELEMENTS(x)-1 DO BEGIN
	CASE 1 OF
		;3. Quadrant
		(y[i] LT 0) AND (x[i] LE 0): phi[i]= phi[i] + 180
		;2. Quadrant
		(y[i] GT 0) AND (x[i] LT 0): phi[i]= phi[i] + 180
		;4. Quadrant
		(y[i] LE 0) AND (x[i] GT 0): phi[i]= phi[i] + 360
		ELSE :
	ENDCASE
	ENDFOR

	psistring = STRCOMPRESS(STRING(psi, FORMAT = '(F10.3)'))
	phistring = STRCOMPRESS(STRING(phi, FORMAT = '(F10.3)'))

;-> Create Psistring

	empty = STRARR(N_ELEMENTS(psistring))
	bevore =  8 - strlen(psistring)
	one = WHERE(bevore EQ 1)
	two = WHERE(bevore EQ 2)
	three = WHERE(bevore EQ 3)

	IF one[0] GE 0 THEN empty(one) = ' '
	IF two[0] GE 0 THEN empty(two) = '  '
	IF three[0] GE 0 THEN empty(three) = '  '
	psistring = empty + STRCOMPRESS(psistring, /REMOVE_ALL)

;-> Create Phistring

	empty = STRARR(N_ELEMENTS(phistring))
	bevore =  8 - strlen(phistring)
	one = WHERE(bevore EQ 1)
	two = WHERE(bevore EQ 2)
	three = WHERE(bevore EQ 3)

	IF one[0] GE 0 THEN empty(one) = ' '
	IF two[0] GE 0 THEN empty(two) = '  '
	IF three[0] GE 0 THEN empty(three) = '  '
	phistring = empty + STRCOMPRESS(phistring, /REMOVE_ALL)

;-> Create Strings - including harmony information

	rule_spacer_h = WHERE(allindices[0,*] LT 0)
	rule_spacer_k = WHERE(allindices[1,*] LT 0)
	rule_spacer_l = WHERE(allindices[2,*] LT 0)

	dummy_stringarray = STRARR(N_ELEMENTS(allindices[0,*])) + ' '

	dummy_stringarray_h = dummy_stringarray
	IF rule_spacer_h[0] NE (-1) THEN dummy_stringarray_h(rule_spacer_h) = ''

	dummy_stringarray_k = dummy_stringarray
	IF rule_spacer_k[0] NE (-1) THEN dummy_stringarray_k(rule_spacer_k) = ''

	dummy_stringarray_l = dummy_stringarray
	IF rule_spacer_l[0] NE (-1) THEN dummy_stringarray_l(rule_spacer_l) = ''

	temp = Strcompress(String(all_twotheta)) + '  ' + dummy_stringarray_h + Strcompress(String(allindices[0,*]))$
													+ dummy_stringarray_k + Strcompress(String(allindices[1,*]))$
											 		+ dummy_stringarray_l + Strcompress(String(allindices[2,*]))$
											 		+ '    (' + Strcompress(String(allharmonies[0,*]),/REMOVE_ALL) +')'$
											 		+ '     ' + psistring + ' ' + phistring

	uniques = UNIQ(temp)
	temp = temp[uniques]
	IF (SIZE(temp))[0] NE 0 THEN temp = TRANSPOSE(temp)

	;;-> Sort it ascending by 2 Theta angle
	;	temp_float = FLOAT(temp)
	;	temp = TRANSPOSE(temp(SORT(temp_float)))

	;-> Combine Strings
		text = [[text],[header],[temp]]

	ENDIF ELSE BEGIN
		text = [[text],[header],[notdefinedstring]]
	ENDELSE

ENDFOR

widget_control, textID, set_value=text

xmanager, 'HKL_list', listwindow

END

; ****
PRO HKL_List_event, event ; dummy handler
END

; ************************************** CleanUp *****************************************
PRO Stereopole_Cleanup, mainwindow

widget_control, mainwindow, get_uvalue = info, /NO_Copy
IF N_Elements(info) EQ 0 THEN RETURN

;-> Free all used pointers
Ptr_Free, info.header
Ptr_Free, info.infilename
Ptr_Free, info.psistart
Ptr_Free, info.psiend
Ptr_Free, info.psistep
Ptr_Free, info.phistart
Ptr_Free, info.phiend
Ptr_Free, info.nop
Ptr_Free, info.phistep
Ptr_Free, info.datenreset
Ptr_Free, info.daten
Ptr_Free, info.X
Ptr_Free, info.Y
Ptr_Free, info.Z
Ptr_Free, info.levels
Ptr_Free, info.res3d
Ptr_Free, info.temp
Ptr_Free, info.Vectors2D_2
Ptr_Free, info.Vectors2D_1
Ptr_Free, info.Vectors3D_2
Ptr_Free, info.Vectors3D_1
Ptr_Free, info.allindices2
Ptr_Free, info.allindices1
Ptr_Free, info.allharmonies2
Ptr_Free, info.allharmonies1
Ptr_Free, info.harmonicstring2
Ptr_Free, info.harmonicstring2
Ptr_Free, info.ymax
Ptr_Free, info.ymin
Ptr_Free, info.xmax
Ptr_Free, info.xmin
Ptr_Free, info.maxint
Ptr_Free, info.minint
Ptr_Free, info.datenreset
Ptr_Free, info.twotheta
Ptr_Free, info.colorlist

WDelete, info.pixID

widget_control, mainwindow, /DESTROY

END

; *************************************** Help File *******************************************
PRO Help, event
widget_control, event.top, get_uvalue = info, /No_Copy

textarray = [$
	'Help is available online on the website of Graz Technical University,',$
	'see: http://www.if.tugraz.at/amd/stereopole/index.html',$
	'', $
	'Suggestions are welcome to ingo.salzmann@physik.hu-berlin.de']

result = Dialog_Message(textarray, /INFORMATION, TITLE = 'Stereopole - Pole Figure Analysis v1.2 - Help')

widget_control, event.top, set_uvalue = info, /No_Copy
END

; *************************************** About Popup *******************************************
PRO About, event
widget_control, event.top, get_uvalue = info, /No_Copy
textarray = [$
	'Stereopole v1.2 - X-Ray Diffraction Pole Figure Analysis with IDL.',$
	' 2004-2005 by Ingo Salzmann [ingo.salzmann@physik.hu-berlin.de]', $
	'',$
	'This program is free software; you can redistribute it and/or modify',$
    'it under the terms of the GNU General Public License as published by',$
    'the Free Software Foundation; either version 2 of the License, or',$
    '(at your option) any later version.',$
	'',$
    'This program is distributed in the hope that it will be useful,',$
    'but WITHOUT ANY WARRANTY; without even the implied warranty of',$
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the',$
    'GNU General Public License for more details.',$
	'',$
    'You should have received a copy of the GNU General Public License',$
    'along with this program; if not, write to the Free Software',$
    'Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA']

result = Dialog_Message(textarray, /INFORMATION, TITLE = 'Stereopole - Pole Figure Analysis v1.2 beta')
widget_control, event.top, set_uvalue = info, /No_Copy
END

; *************************************** Reset All to Default ********************************
PRO Reset_Stereopole, event
widget_control, event.top, get_uvalue = info, /No_Copy

;-> Reset Colors
thisDevice=!D.Name
LOADCT, 3 ;-> Load Color Table 'Red Temperature' and reverse it
TVLCT, r, g, b, /Get
TVLCT, Reverse(r), Reverse(g), Reverse(b)
*info.colorlist = SetColor(info.colors)

u = DBLARR(3,3) + [[1,0,0],[0,1,0],[0,0,1]]
unit_matrices = [[[u]],[[u]],[[u]],[[u]],[[u]]]
axishkl 	= DBLARR(3)+[0,0,1] ;-> initializing variables
axisuvw 	= DBLARR(3)+[0,0,1]
info.rotaxis_xyz = DBLARR(3)+[0,0,1]
info.rotaxis_hkl = DBLARR(3)+[0,0,1]
dtrans 		= unit_matrices
rtrans 		= unit_matrices
info.rottensor 	= unit_matrices
info.rottensor_orth_hkl = unit_matrices
info.rottensor_orth_uvw = unit_matrices
anglehkl= 1D
angleuvw= 1D
info.keep = 1
info.quality  = 0.1

info.dir_rot_info = {axisuvw:axisuvw, angleuvw:angleuvw, dtrans:dtrans}
info.rec_rot_info = {axishkl:axishkl, anglehkl:anglehkl, rtrans:rtrans}

IF ((N_ELEMENTS(*info.allindices1) GT 0) OR $
	(N_ELEMENTS(*info.allindices2) GT 0) OR $
	(N_ELEMENTS(*info.allindices3) GT 0) OR $
	(N_ELEMENTS(*info.allindices4) GT 0) OR $
	(N_ELEMENTS(*info.allindices5) GT 0)) THEN BEGIN
		info.updatetrigger = 3
		info.showbitmask[0] = 0
		widget_control, event.top, set_uvalue = info, /NO_Copy
		UpdateFromTableCleanup, event
		Zeichnen_xyz_reset, event
		RETURN
ENDIF ELSE BEGIN
info.showbitmask[0] = 1
ENDELSE
widget_control, event.top, set_uvalue = info, /No_Copy
Zeichnen_xyz_reset, event
END
; ************************************** Main Program *****************************************
PRO STEREOPOLE,	header, infilename, psistart, psiend, $
				psistep, phistart, phiend, nop, phistep, daten

void = STARTUPFILE()

thisDevice=!D.Name
LOADCT, 3 ;-> Load Color Table 'Red Temperature' and reverse it
TVLCT, r, g, b, /Get
TVLCT, Reverse(r), Reverse(g), Reverse(b)

Catch, theError
IF theError NE 0 THEN BEGIN
   Catch, /Cancel
   errormsg = dialog_message('General Error in Main Procedure!')
   IF N_Elements(info) NE 0 THEN $
      Widget_Control, event.top, Set_UValue=info, /No_Copy
   RETURN
ENDIF

pixmapID = !D.Window
tablerealized1 = 0
numberoflevels = 60
nthline = 3
linetrigger = 0
degreestep = 15
charofset = 1.0
charsize = 15
pointsize = 3.0
dropstring = ['Square','Plus','Asterisk','Period','Diamond','Triangle','Cross']
symbolselect = 0
zoomed = 0
nettrigger = 1
dimtrigger = 1
zoomtrigger = 3 ;-> means no zoom
updatetrigger = 0
modetrigger = 0 ;-> means pole figure mode as default
uvwcolor = 1
colors = [255,0,0,0,0,255,0,200,0,255,150,0,255,0,255] ;-> RGB values for the five crytals, used in Function SetColor
showbitmask = [0,0,0,0,0,0]
rotatebitmask = [0,0,0,0,0,0]
crystaltrigger = 1 ;-> means crystal 1
axistrigger = 0
integrattrigger = 0
rotstepsize = 10
apply_gcd = 0

axis 		= [0,0,1]
u = DBLARR(3,3) + [[1,0,0],[0,1,0],[0,0,1]]
unit_matrices = [[[u]],[[u]],[[u]],[[u]],[[u]]]
axishkl 	= DBLARR(3)+[0,0,1] ;-> initializing variables
axisuvw 	= DBLARR(3)+[0,0,1]
rotaxis_xyz = DBLARR(3)+[0,0,1]
rotaxis_hkl = DBLARR(3)+[0,0,1]
dtrans 		= unit_matrices
rtrans 		= unit_matrices
rottensor 	= unit_matrices
rottensor_orth_hkl = unit_matrices
rottensor_orth_uvw = unit_matrices
recmetric = unit_matrices
anglehkl= 1D
angleuvw= 1D
keep = 1
quality  = 0.1

clickx1 = 0
clicky1 = 0
clickx2 = 0
clicky2 = 0
labelon = 1
epirange = 10
; manualindex1 = intarr(3,100)
foldness1 = intarr(1,100)+1
harmony1 = intarr(1,100)+1
manualindex = INTARR(5,100,5)	; V2: create array for indices of all 5 crystals
manualindex[3:4,*,*]=1			; set foldnesses and harmonies to 1 as default
autoindextrigger = INTARR(5)+1 ; V2: manual: 1, auto: 0
hmin = INTARR(5)-5
kmin = INTARR(5)-5
lmin = INTARR(5)-5
hmax = INTARR(5)+5
kmax = INTARR(5)+5
lmax = INTARR(5)+5
middlehkl = INTARR(3,5)+1
middleuvw = INTARR(3,5)+1
nettoangle = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0] ; 1st item: PoleFigure, 2nd: crystal_1, 3rd: crystal_2, ...
a = FLTARR(5) + 5.0
b = FLTARR(5) + 5.0
c = FLTARR(5) + 5.0
alpha = FLTARR(5) + 90.0
beta = FLTARR(5) + 90.0
gamma = FLTARR(5) + 90.0
wavelength = 2.28975 ; Cr wavelength
higherharm = INTARR(5)+1
twothetarange = FLTARR(5)+1.0
hkllist1 = Ptr_New(hkllist1)
hkllist2 = Ptr_New(hkllist2)
hkllist3 = Ptr_New(hkllist3)
hkllist4 = Ptr_New(hkllist4)
hkllist5 = Ptr_New(hkllist5)
hkllist = {hkllist1:hkllist1, hkllist2:hkllist2, hkllist3:hkllist3, hkllist4:hkllist4, hkllist5:hkllist5}
rwatrigger = 1
peak_range = 4 ; diameter of circle around double click for peak integration
version = 2
path = !DIR ; Set initial File Path to IDL root path

mainwindow = widget_base(column=1, mbar=mbar, uvalue='mainwindow', $
			 title='STEREOPOLE v1.2 - XRD Pole Figure Analysis', /TLB_SIZE_EVENTS) ; v0.99c

;-> Create menu buttons in menu bar base

filemenu = widget_button(mbar, value='File')
fileopt1 = widget_button(filemenu, value='Open *.rwa ...',$
				uvalue='ReadData')
fileopt2 = widget_button(filemenu, value='Load ASCII...',$
				Event_Pro='LoadASCII')
fileopt3 = widget_button(filemenu, value='Save ASCII...',$
				Event_Pro='ASCIIWriteData')
fileopt4 = widget_button(filemenu, value='Load Session',$
				Event_Pro='Load_Session')
fileopt5 = widget_button(filemenu, value='Save Session',$
				Event_Pro='Save_Session')
fileopt6 = widget_button(filemenu, value='Load Crystal', /Menu, /separator)
	button = widget_button(fileopt6, Value='Crystal 1', UValue='crystal1_load')
	button = widget_button(fileopt6, Value='Crystal 2', UValue='crystal2_load')
	button = widget_button(fileopt6, Value='Crystal 3', UValue='crystal3_load')
	button = widget_button(fileopt6, Value='Crystal 4', UValue='crystal4_load')
	button = widget_button(fileopt6, Value='Crystal 5', UValue='crystal5_load')

fileopt7 = widget_button(filemenu, value='Save Crystal', /Menu)
	button = widget_button(fileopt7, Value='Crystal 1', UValue='crystal1_save')
	button = widget_button(fileopt7, Value='Crystal 2', UValue='crystal2_save')
	button = widget_button(fileopt7, Value='Crystal 3', UValue='crystal3_save')
	button = widget_button(fileopt7, Value='Crystal 4', UValue='crystal4_save')
	button = widget_button(fileopt7, Value='Crystal 5', UValue='crystal5_save')
fileopt8 = widget_button(filemenu, value='Save Image As',$
				Event_Pro='SaveImageAs', /Menu)
button = widget_button(fileopt8, Value='TIFF File', UValue='.tif')
button = widget_button(fileopt8, Value='JPEG File', UValue='.jpg')
button = widget_button(fileopt8, Value='Postscript File', UValue='.ps')

fileopt9 = widget_button(filemenu, value='Save PSI Scan',$
				Event_Pro='SavePsiScan')
fileopt10 = widget_button(filemenu, value='Exit', Event_Pro='Quit', /separator)

editmenu = widget_button(mbar, value='Edit')
editopt1 = widget_button(editmenu, value='Zoom ...',$
				Event_Pro='Zoom')
editopt2 = widget_button(editmenu, value='Preferences...', uvalue='Preferences', /separator)
editopt3 = widget_button(editmenu, value='Reset', Event_Pro='Reset_Stereopole')

analysismenu = widget_button(mbar, value='Analysis')
editopt2 = widget_button(analysismenu, value='Integrated PSI-scan', Event_Pro='VirtualPsiScan')
editopt3 = widget_button(analysismenu, value='Intensity Histogram', Event_Pro='HistoPole')
editopt4 = widget_button(analysismenu, value='Set new Rotation axis ...', uvalue='newAxis')
editopt5 = widget_button(analysismenu, value='Peak Integration ...', uvalue='Integration')
editopt6 = widget_button(analysismenu, Value='Show (hkl) list ...', uvalue='hkl_list' )
editopt7 = widget_button(analysismenu, Value='Calculate interplanar angles ...', /Menu)
	button = widget_button(editopt7, Value='Crystal 1', UValue='angles1')
	button = widget_button(editopt7, Value='Crystal 2', UValue='angles2')
	button = widget_button(editopt7, Value='Crystal 3', UValue='angles3')
	button = widget_button(editopt7, Value='Crystal 4', UValue='angles4')
	button = widget_button(editopt7, Value='Crystal 5', UValue='angles5')
editopt8 = widget_button(analysismenu, value='(hkl) by click ...', /Menu)
	button = widget_button(editopt8, Value='Crystal 1', UValue='crystal1_hkl')
	button = widget_button(editopt8, Value='Crystal 2', UValue='crystal2_hkl')
	button = widget_button(editopt8, Value='Crystal 3', UValue='crystal3_hkl')
	button = widget_button(editopt8, Value='Crystal 4', UValue='crystal4_hkl')
	button = widget_button(editopt8, Value='Crystal 5', UValue='crystal5_hkl')

helpmenu = widget_button(mbar, value='Help')
helpopt1 = widget_button(helpmenu, value='Help Index', Event_Pro='Help')
helpopt2 = widget_button(helpmenu, value='About...', Event_Pro='About', /separator)


iconbigbase = widget_base(mainwindow, column=3)

;-> Create bitmap buttons in row-aligned base

iconbase = widget_base(iconbigbase, row=1, frame=1)
subdir   = 'resource/bitmaps'
root = 'stereopole'
mapfile  = filepath('open.bmp', subdir=subdir)
iconopt1 = widget_button(iconbase, value=mapfile,$
				Event_Pro='Load_Session', /bitmap)
mapfile  = filepath('importf.bmp', subdir=subdir)
iconopt2 = widget_button(iconbase, value=mapfile,$
				uvalue='ReadData', /bitmap)
mapfile  = filepath('save.bmp', subdir=subdir)
iconopt3 = widget_button(iconbase, value=mapfile,$
				Event_Pro='Save_Session', /bitmap)
mapfile  = filepath('print1.bmp', subdir=subdir)
iconopt4 = widget_button(iconbase, value=mapfile,$
				Event_Pro='CallPrintwindow', /bitmap)
mapfile  = filepath('ps.bmp', subdir=root)
iconopt5 = widget_button(iconbase, value=mapfile, uvalue='ps', /bitmap)


iconbase1 = widget_base(iconbigbase, row=1, frame=1)
mapfile  = filepath('zoom.bmp', subdir=root)
iconopt5 = widget_button(iconbase1, value=mapfile, Event_Pro='Zoom', /bitmap)

mapfile  = filepath('contour.bmp', subdir=subdir)
iconopt1 = widget_button(iconbase1, value=mapfile, /bitmap,$
				Event_Pro='Zeichnen_xyz_reset')
mapfile  = filepath('surface.bmp', subdir=subdir)
iconopt2 = widget_button(iconbase1, value=mapfile, /bitmap,$
				Event_Pro='Zeichnen_3d')
mapfile  = filepath('color.bmp', subdir=root)
iconopt3 = widget_button(iconbase1, value=mapfile, /bitmap,$
				Event_Pro='Colortable')
mapfile  = filepath('hist.bmp', subdir=subdir)
iconopt4 = widget_button(iconbase1, value=mapfile, /bitmap,$
				Event_Pro='HistoPole')
mapfile  = filepath('profile.bmp', subdir=subdir)
iconopt5 = widget_button(iconbase1, value=mapfile, /bitmap,$
				Event_Pro='VirtualPsiScan')
mapfile  = filepath('wulff.bmp', subdir=root)
iconopt6 = widget_button(iconbase1, value=mapfile, /bitmap,$
				uvalue='wulff')
mapfile  = filepath('polarnet.bmp', subdir=root)
iconopt7 = widget_button(iconbase1, value=mapfile, /bitmap,$
				uvalue='polar')

iconbase2 = widget_base(iconbigbase, row=1, frame=1)
mapfile  = filepath('table1.bmp', subdir=root)
tablebuttonID1 = widget_button(iconbase2, value=mapfile, /bitmap, uvalue='tablerealized1' )
mapfile  = filepath('table2.bmp', subdir=root)
tablebuttonID2 = widget_button(iconbase2, value=mapfile, /bitmap, uvalue='tablerealized2' )
mapfile  = filepath('table3.bmp', subdir=root)
tablebuttonID3 = widget_button(iconbase2, value=mapfile, /bitmap, uvalue='tablerealized3' )
mapfile  = filepath('table4.bmp', subdir=root)
tablebuttonID4 = widget_button(iconbase2, value=mapfile, /bitmap, uvalue='tablerealized4' )
mapfile  = filepath('table5.bmp', subdir=root)
tablebuttonID5 = widget_button(iconbase2, value=mapfile, /bitmap, uvalue='tablerealized5' )
mapfile  = filepath('hkl.bmp', subdir=root)
tablebuttonID6 = widget_button(iconbase2, value=mapfile, /bitmap, uvalue='hkl_list' )

;-> Create draw widget
mainoutput = widget_base(mainwindow, column=2, frame=1)
	plotbase = widget_base(mainoutput)
		plotwindow = Widget_Draw(plotbase, XSize=800, YSIZE=600,$
		/button_events, /motion_events, uvalue = 'plotwindow')

;		plotwindow = Widget_Draw(plotbase, XSize=800, YSize=600, X_Scroll_Size=800, Y_Scroll_SIZE=600,$
;		/button_events, /motion_events, /SCROLL, RETAIN=2, uvalue = 'plotwindow')

;-> Create options widget
	optionsbase = widget_base(mainoutput, column=1)

;-> Toggle Mode
	label = widget_label(optionsbase, value='Select Mode', /align_left)
		modebase = widget_base(optionsbase, row=5, frame=1, /exclusive)
			modeopt1 = widget_button(modebase, value='Pole Figure and Planes', uvalue = 'planes')
			modeopt2 = widget_button(modebase, value='Epitaxial Relationships', uvalue = 'directions')

;-> Toggle displayed plane and pole figure plots
	label = widget_label(optionsbase, value='Select items to show:', /align_left)
		showbase = widget_base(optionsbase, column=2, frame=1, /nonexclusive)
			showopt1 = widget_button(showbase, value='Pole Figure', uvalue = 'poleshow')
			showopt2 = widget_button(showbase, value='Crystal 1', uvalue = 'show1')
			showopt3 = widget_button(showbase, value='Crystal 2', uvalue = 'show2')
			showopt4 = widget_button(showbase, value='Crystal 3', uvalue = 'show3')
			showopt5 = widget_button(showbase, value='Crystal 4', uvalue = 'show4')
			showopt6 = widget_button(showbase, value='Crystal 5', uvalue = 'show5')

;-> Toggle whether to show index labels
	label = widget_label(optionsbase, value='Show index labels', /align_left)
	labelonbase = widget_base(optionsbase, column=2, frame=1, /exclusive)
		labelswitch1 = widget_button(labelonbase, value='On', uvalue = 'labelon')
		labelswitch2 = widget_button(labelonbase, value='Off', uvalue = 'labeloff')

;-> position part
	label = widget_label(optionsbase, value='Current position', /align_left)
		currentposbase = widget_base(optionsbase, row=2, frame=1)
			labelID = widget_label(currentposbase, value = ' ', /align_left, /dynamic_resize)
			labelID_int = widget_label(currentposbase, value = ' ', /align_left, /dynamic_resize)

;-> rotational part
	label = widget_label(optionsbase, value='Rotation', /align_left)
	rotatebase = widget_base(optionsbase, row=4, frame=1)
		label = widget_label(rotatebase, value='Select items to rotate:', /align_left)
		rotselectbase = widget_base(rotatebase, column=2, frame=1, /nonexclusive)
			rotopt1 = widget_button(rotselectbase, value='Pole Figure', uvalue = 'rotatepole')
			rotopt2 = widget_button(rotselectbase, value='Crystal 1', uvalue = 'rotatecryst1')
			rotopt3 = widget_button(rotselectbase, value='Crystal 2', uvalue = 'rotatecryst2')
			rotopt4 = widget_button(rotselectbase, value='Crystal 3', uvalue = 'rotatecryst3')
			rotopt5 = widget_button(rotselectbase, value='Crystal 4', uvalue = 'rotatecryst4')
			rotopt6 = widget_button(rotselectbase, value='Crystal 5', uvalue = 'rotatecryst5')

		stepsizebase = widget_base(rotatebase, row = 1)
			stepsizewid =	CW_Field(stepsizebase, Title = 'Toggle step size []:',$
						value = 10, uvalue = 'rotstepsize', /integer, xsize=3, /all_events)

		buttonbase = widget_base(rotatebase, row = 1, /ALIGN_CENTER)
			leftbutton = widget_button(buttonbase, value='left', uvalue = 'rotleft', xsize=55)
			rightbutton = widget_button(buttonbase, value='right', uvalue = 'rotright', xsize=55)

;-> Set Intensity Range for Pole Figure
		label = widget_label(optionsbase, value='Intensity Range to show:', /align_left)
		intensityrangebase = widget_base(optionsbase, column = 1, frame=1, xsize=170,/base_align_right)
			minintID = CW_Field(intensityrangebase, Title = 'Minimum Int.: ', uvalue = 'minint', xsize=5, /LONG, /return_events)
			maxintID = CW_Field(intensityrangebase, Title = 'Maximum Int.: ', uvalue = 'maxint', xsize=5, /LONG, /return_events)

;-> File Info
		infobase = widget_base(optionsbase,  column = 1, frame=1, xsize=170, /base_align_right)
			labelfilemain = widget_label(infobase, value = ' ', /align_left, /dynamic_resize)
			labeldatemain = widget_label(infobase, value = ' ', /align_left, /dynamic_resize)
			labelnamemain = widget_label(infobase, value = ' ', /align_left, /dynamic_resize)

;-> crystal
	crystalparmsbase = widget_base(optionsbase, row=1)
			calculate = widget_button(crystalparmsbase, value='Set Plot Preferences',$
						uvalue = 'Preferences', xsize=170, /align_center)

;-> Get the sizes of the widgets other than the draw widget
UpperRowGeom = Widget_Info(iconbigbase, /Geometry)
dy = UpperRowGeom.ysize
LeftColumnGeom = Widget_Info(optionsbase, /Geometry)
dx = LeftColumnGeom.xsize
geometry={dx:dx, dy:dy}

;-> Create an invisible pixmap window for double buffered output
;-> All graphic output is made there and then copied to the draw widget at once
;-> what results in no flickering of the plot...
Window, XSize=800, YSize=600, /Pixmap, /Free
pixID = !D.Window

;-> Realize widgets
widget_control, mainwindow, /Realize
widget_control, plotwindow, get_value = plotwindowind

;-> Store IDs of option buttons and FileInfo field
modeopt = {modeopt1:modeopt1, modeopt2:modeopt2}
rotopt = {rotopt1:rotopt1, rotopt2:rotopt2, rotopt3:rotopt3, rotopt4:rotopt4, rotopt5:rotopt5, rotopt6:rotopt6}
showopt = {showopt1:showopt1, showopt2:showopt2, showopt3:showopt3, showopt4:showopt4, showopt5:showopt5, showopt6:showopt6}
labelfile = {labelfilemain:labelfilemain, labeldatemain:labeldatemain, labelnamemain:labelnamemain}

dir_rot_info = {axisuvw:axisuvw, angleuvw:angleuvw, dtrans:dtrans}
rec_rot_info = {axishkl:axishkl, anglehkl:anglehkl, rtrans:rtrans}

;-> Info Structure of pointers for runtime data exchange
info = {mainwindow:mainwindow,$			; ID of mainwindow
		plotwindow:plotwindow,$			; ID of plotwindow
		plotwindowind:plotwindowind,$	; index of plotwindow [used for WSET]
		pixID:pixID,$					; ID of the invisible pixmap window used for smooth output
		labelID:labelID,$				; ID of Label Widget for current position of cursor
		labelID_int:labelID_int,$		; ID of Label Widget for current intensity
		stepsizewid:stepsizewid,$		; ID of stepsize Widget
		tablebuttonID1:tablebuttonID1,$	; ID of Table Button1, uvalue = tablerealized1 (0/1)
		labelswitch1:labelswitch1,$		; ID of labelon switch
		labelswitch2:labelswitch2,$		; ID of labeloff switch
		minintID:minintID,$				; ID of Minimum Intensity CW_Field
		maxintID:maxintID,$				; ID of Maximum Intensity CW_Field
		modeopt:modeopt,$				; Array of widget IDs of mode options
		rotopt:rotopt,$					; Array of widget IDs of rotation options
		showopt:showopt,$				; Array of widget IDs of show options
		labelfile:labelfile,$			; Array of widget ID of main file info
		header:Ptr_New(header),$		; File header from Philips TEX ASCII file
		infilename:Ptr_New(infilename),$; filename
		psistart:Ptr_New(psistart),$	; lower limit of psi []
		psiend:Ptr_New(psiend),$		; upper limit of psi []
		psistep:Ptr_New(psistep),$		; step size of psi []
		phistart:Ptr_New(phistart),$	; lower limit of phi []
		phiend:Ptr_New(phiend),$		; upper limit of phi []
		nop:Ptr_New(nop),$				; number of data points
		twotheta:Ptr_New(twotheta),$	; Two Theta angle of Pole Figure File
		twothetarange:twothetarange,$	; Range around 2 Theta for possible NetPlanes
		phistep:Ptr_New(phistep),$		; step size of phi []
		daten:Ptr_New(daten),$			; array of floating point data with structure (phi,psi,intensity)
		datenreset:Ptr_New(datenreset),$; initial (phi,psi,intensity) for 'reset' function
		X:Ptr_New(X),$					; stereographical projected data (X)
		Y:Ptr_New(Y),$					; stereographical projected data (Y)
		Z:Ptr_New(Z),$					; intensity
		minint:Ptr_New(minint),$		; minimum intensity to show
		maxint:Ptr_New(maxint),$		; maximum intensity to show
		radius:Ptr_New(radius),$		; radius of the pole figure
		numberoflevels:numberoflevels,$	; number of intensity levels in 2D Plot
		levels:Ptr_New(levels),$		; Array of intensity levels
		degreestep:degreestep,$			; step size in psi for psi circles in pole figure
		charofset:charofset,$			; Offset for (hkl) labels
		charsize:charsize,$				; Character size for (hkl) labels
		pointsize:pointsize,$			; Size of UserSymbol 'Square'
		symbolselect:symbolselect,$		; Index [0 ... 6] of Symbol for stereographic points ...
										; ['Square','Plus','Asterisk','Period','Diamond','Triangle','Cross']
		dropstring:dropstring,$			; dynamically rearranged string array of symbol type names
		epirange:epirange,$				; range in degree around psi=90 in epitaxial mode
		nthline:nthline,$				; Indicates that the nth line will be labeled in contour lines
		linetrigger:linetrigger,$		; Triggers whether to plot contour lines or not
		updatetrigger:updatetrigger,$	; if set to 1: motion event in plotwindow -> redraw
		res3d:Ptr_New(res3d),$			; Number of points in x and y direction for 3D plot
		colorlist:Ptr_New(colorlist),$	; List of color indices for Postscript output
		colors:colors,$					; RGB values of the two crytal colors
		dimtrigger:dimtrigger,$			; values: 1 = 2D, 2 = 3D, 3=PSI-Scan
		nettrigger:nettrigger,$			; values: 1 = PolarNet, 2 = Wulff Net
		crystaltrigger:crystaltrigger,$	; crystal1 = 1, crystal2 = 2, ...
		rotstepsize:rotstepsize,$		; stepsize for rotation in degree (stored as integer)
		zoomtrigger:zoomtrigger,$		; trigger for zoom edges
		modetrigger:modetrigger,$		; trigger for both pole figure (0) and epitaxial (1) modes
		uvwcolor:uvwcolor,$				; color for [uvw] labels and points [1 = crystal 1], [2 = crystal 2]
		showbitmask:showbitmask,$		; position 0:pole, 1:c1, 2:c2 3:c3 4:c4, 4:c5
		rotatebitmask:rotatebitmask,$	; position 0:pole, 1:crystal1, 2:crystal2, ...
		labelon:labelon,$				; labels on in stereogramm? 1:on, 0:off
		zoomed:zoomed,$					; trigger for logical variable zoomed - 0: not zoomed, 1: zoomed
		clickx1:clickx1,$				; zoom range in _display_ coordinates
		clicky1:clicky1,$				;
		clickx2:clickx2,$				;
		clicky2:clicky2,$				;
		peakx:Ptr_New(peakx),$			; x-position of double click in plotwindow
		peaky:Ptr_New(peaky),$			; y-position of double click in plotwindow
		peak_volume:Ptr_New(peak_volume),$ ; pointer for the volume of a certain peak
		peak_range:peak_range,$			; range around double click position for peak integration
		xmin:Ptr_New(xmin),$			; zoom range in _data_ coordinates
		xmax:Ptr_New(xmax),$			;
		ymin:Ptr_New(ymin),$			;
		ymax:Ptr_New(ymax),$			;
		manualindex:manualindex,$		; V2: (4,5,100) array for indices
		autoindextrigger:autoindextrigger,$ ; manual: 1, auto: 0
		allindices1:Ptr_New(allindices1),$	; here the resulting indices of crystal 1 are stored
		allindices2:Ptr_New(allindices2),$	; here the resulting indices of crystal 2 are stored
		allindices3:Ptr_New(allindices3),$	; here the resulting indices of crystal 3 are stored
		allindices4:Ptr_New(allindices4),$	; here the resulting indices of crystal 4 are stored
		allindices5:Ptr_New(allindices5),$	; here the resulting indices of crystal 5 are stored
		allharmonies1:Ptr_New(allharmonies1),$;  indices for higher harmonic of any index in desired range [cryst1]
		allharmonies2:Ptr_New(allharmonies2),$;  indices for higher harmonic of any index in desired range [cryst2]
		allharmonies3:Ptr_New(allharmonies3),$;  indices for higher harmonic of any index in desired range [cryst3]
		allharmonies4:Ptr_New(allharmonies4),$;  indices for higher harmonic of any index in desired range [cryst4]
		allharmonies5:Ptr_New(allharmonies5),$;  indices for higher harmonic of any index in desired range [cryst5]
		alltwotheta1:Ptr_New(alltwotheta1),$;	2Theta angles in range -> For List output
		alltwotheta2:Ptr_New(alltwotheta2),$;	2Theta angles in range -> For List output
		alltwotheta3:Ptr_New(alltwotheta3),$;	2Theta angles in range -> For List output
		alltwotheta4:Ptr_New(alltwotheta4),$;	2Theta angles in range -> For List output
		alltwotheta5:Ptr_New(alltwotheta5),$;	2Theta angles in range -> For List output
		harmonicstring1:Ptr_New(harmonicstring1),$; string labels for higher harmonics [cryst 1]
		harmonicstring2:Ptr_New(harmonicstring2),$; string labels for higher harmonics [cryst 2]
		harmonicstring3:Ptr_New(harmonicstring3),$; string labels for higher harmonics [cryst 3]
		harmonicstring4:Ptr_New(harmonicstring4),$; string labels for higher harmonics [cryst 4]
		harmonicstring5:Ptr_New(harmonicstring5),$; string labels for higher harmonics [cryst 5]
		hmin:hmin,$						; range for hkl, arrays for both crystals
		kmin:kmin,$						;
		lmin:lmin,$						;
		hmax:hmax,$						;
		kmax:kmax,$						;
		lmax:lmax,$						;
		middlehkl:middlehkl,$			; (hkl) to be in the middle of the stereographic proj.
		middleuvw:middleuvw,$			; (uvw) to be in the middle of the stereographic proj.
		a:a,$							; unit cell
		b:b,$							;
		c:c,$							;
		alpha:alpha,$					;
		beta:beta,$						;
		gamma:gamma,$					;
		wavelength:wavelength,$			; Wavelength or x-ray radiation [default: Cr]
		nettoangle:nettoangle,$			; sum of angles that the items were rotated
		higherharm:higherharm,$			; Level of higher harmonics to be calculated [cryst1, cryst2]
		Vectors3D_1:Ptr_New(Vectors3D_1),$	; Structure 3D vectors in direct and reciprocal space, crystal 1
		Vectors3D_2:Ptr_New(Vectors3D_2),$	; Structure 3D vectors in direct and reciprocal space, crystal 2
		Vectors3D_3:Ptr_New(Vectors3D_3),$	; Structure 3D vectors in direct and reciprocal space, crystal 3
		Vectors3D_4:Ptr_New(Vectors3D_4),$	; Structure 3D vectors in direct and reciprocal space, crystal 4
		Vectors3D_5:Ptr_New(Vectors3D_5),$	; Structure 3D vectors in direct and reciprocal space, crystal 5
		Vectors2D_1:Ptr_New(Vectors2D_1),$	; Structure 2D vectors in direct and reciprocal space, crystal 1
		Vectors2D_2:Ptr_New(Vectors2D_2),$	; Structure 2D vectors in direct and reciprocal space, crystal 2
		Vectors2D_3:Ptr_New(Vectors2D_3),$	; Structure 2D vectors in direct and reciprocal space, crystal 3
		Vectors2D_4:Ptr_New(Vectors2D_4),$	; Structure 2D vectors in direct and reciprocal space, crystal 4
		Vectors2D_5:Ptr_New(Vectors2D_5),$	; Structure 2D vectors in direct and reciprocal space, crystal 5
		hkllist:hkllist,$				; String Array for peak information
		geometry:geometry,$				; Geometry infos for proper resizing of the top level base widget
		rwatrigger:rwatrigger,$			; Triggers whether pole figure is a Philips RWA file [0: no, 1:yes]
		thisDevice:thisDevice,$			; Standard Graphics output device ['WIN' on Windows, 'X' on X-Windows]
		path:path,$						; Input File Path
		temp:Ptr_New(temp),$			; Temporary variable

		rotaxis_xyz:rotaxis_xyz,$		; (xyz) of rotation axis [orthogonal system]
		rotaxis_hkl:rotaxis_hkl,$		; rotation axis within the crystal system
		dir_rot_info:dir_rot_info,$;structure to store history of rotation for direct space vectors during orthogonalization
		rec_rot_info:rec_rot_info,$;	; structure to store history of rotation for reciprocal space vectors during orthogonalization
		rottensor:rottensor,$			; Continousely updated rotation tensor
		rottensor_orth_hkl:rottensor_orth_hkl,$ ; Rotation Tensors generated at the point of orthogonalization
		rottensor_orth_uvw:rottensor_orth_uvw,$
		keep:keep,$						; Keep Orientational information (1) or not (0) in Setup of Crystal information
		quality:quality,$				; Quality of (hkl) calculation ... default is 0.1
		integrattrigger:integrattrigger,$;1: Double Click yields Peak Integration
		axistrigger:axistrigger,$		; 1-5: Single Click = Set axis for crystal of index axistrigger, default: 0
		recmetric:recmetric,$			; array of metric tensors for the five crystals
		apply_gcd:apply_gcd,$			; Toggle whether to apply devision of (hkl) by its greatest common devisor to avoid overplotting ...
		version:version}				; version 1 or 2 [manually]

*info.res3d = 100
*info.temp = 0
;*info.minint = minint
;*info.maxint = maxint ;LONG64(*info.minint)

;-> Set the draw widget 'plotwindow' as graphical output window
info.plotwindow = plotwindow
info.plotwindowind = plotwindowind

WSET, info.plotwindowind
ERASE
WSET, info.pixID

widget_control, showopt1, set_button = info.showbitmask[0]
widget_control, showopt2, set_button = info.showbitmask[1]
widget_control, showopt3, set_button = info.showbitmask[2]
widget_control, showopt4, set_button = info.showbitmask[3]
widget_control, showopt5, set_button = info.showbitmask[4]

widget_control, modeopt1, set_button = 1

widget_control, rotopt1, set_button = info.rotatebitmask[0]
widget_control, rotopt2, set_button = info.rotatebitmask[1]
widget_control, rotopt3, set_button = info.rotatebitmask[2]

widget_control, labelfilemain, set_value = 'No pole figure file loaded.'

IF (info.labelon EQ 1) THEN BEGIN
	widget_control, labelswitch1, set_button = 1
ENDIF ELSE BEGIN
	widget_control, labelswitch2, set_button = 1
ENDELSE

;-> Define orange and blue as default postscript colors for the two crystals

*info.colorlist = SetColor(info.colors)

;-> Copy and store info structure in the top-level base (mainwindow)
;-> /No_Copy keyword leads to undefined local variable 'info', but user
;-> value of 'mainwindow' is defined

widget_control, mainwindow, Set_UValue=info, /No_Copy

;-> Managing events
xmanager, 'stereopole', mainwindow, Event_Handler = 'StereopoleEvent',$
	Cleanup='Stereopole_Cleanup';, /No_Block;, Group_Leader=group_leader

END

; ********************************* Event handling ***************************************
PRO StereopoleEvent, event

;-> return a scalar string that contains the name of the structure
actualEvent = Tag_Names(event, /Structure_Name)

; ***** Resize Event Handling

IF (actualEvent EQ 'WIDGET_BASE') THEN BEGIN
	widget_control, event.top, get_uvalue = info, /NO_COPY
	dy=info.geometry.dy
	dx=info.geometry.dx
	Widget_Control, info.plotwindow, Draw_XSize=(event.x-dx-16), Draw_YSize=(event.y-dy-16), Units=0

	;-> Also the invisible window for double buffering has to be resized in the same way
	;-> Therfore it has to be deleted and defined again

	WDelete, info.pixID
	Window, XSize=(event.x-dx-16), YSize=(event.y-dy-16), /Pixmap, /Free
	info.pixID = !D.Window
	Widget_Control, event.top, Set_uvalue=info, /No_Copy

	ShowMain, event
	RETURN
ENDIF ELSE BEGIN

; ***** Draw Widget Event Handling

widget_control, event.top, get_uvalue = info, /NO_COPY
widget_control, event.id, get_uvalue = eventid

Case eventid OF
	'ReadData':	BEGIN
				widget_control, event.top, set_uvalue = info, /NO_Copy
				ReadDataFile_FirstPlot, event
				END

	'planes':	BEGIN
				IF (info.modetrigger NE 0) THEN BEGIN
					info.nettrigger = 1 ;-> polar net
					info.modetrigger = 0
					info.updatetrigger = 3 ;-> recalculate all hkl
					widget_control, event.top, set_uvalue = info, /NO_Copy
					UpdateFromTableCleanup, event
					Zeichnen_xyz_reset, event
				ENDIF ELSE BEGIN
					widget_control, event.top, set_uvalue = info, /NO_Copy
					RETURN
				ENDELSE
				END

	'directions': BEGIN
				IF (N_ELEMENTS(*info.daten) EQ 0) THEN BEGIN
					errormsg = dialog_message('Please load Pole Figure first!', /ERROR)
					widget_control, (info.modeopt).modeopt2, set_button = 0
					widget_control, (info.modeopt).modeopt1, set_button = 1
					widget_control, event.top, set_uvalue = info, /NO_Copy
					RETURN
				ENDIF ELSE BEGIN

;				IF (*info.twotheta EQ 0) THEN BEGIN
;					errormsg = dialog_message('Please set Two-Theta Angle or load Pole Figure first!', /ERROR)
;					widget_control, (info.modeopt).modeopt2, set_button = 0
;					widget_control, (info.modeopt).modeopt1, set_button = 1
;					widget_control, event.top, set_uvalue = info, /NO_Copy
;					RETURN
;				ENDIF ELSE BEGIN

				info.modetrigger = 1
				info.nettrigger = 2 ;-> wulff net
				info.showbitmask[0] = 0
				info.updatetrigger = 3
				*info.xmin = -1
				*info.xmax = 1
				*info.ymin = -1
				*info.ymax = 1
				dimtrigger = info.dimtrigger

				widget_control, event.top, set_uvalue = info, /NO_Copy
				UpdateFromTableCleanup, event
				ShowMain, event
				ENDELSE
				END

	'poleshow':	BEGIN
				current = info.showbitmask[0]
				IF (N_ELEMENTS(*info.daten) EQ 0) THEN BEGIN
					errormsg = dialog_message('Please Load Pole Figure first!', /ERROR)
					info.showbitmask[0] = 0
					widget_control, (info.showopt).showopt1, set_button = 0
					widget_control, event.top, set_uvalue = info, /NO_Copy
					RETURN
				ENDIF ELSE BEGIN

				IF (current EQ 0) THEN BEGIN
					info.showbitmask[0] = 1
					widget_control, event.top, set_uvalue = info, /NO_Copy
					ShowMain, event
				ENDIF ELSE BEGIN

					info.showbitmask[0] = 0
				;-> Erase content of the draw widget
				;	ERASE
					widget_control, (info.showopt).showopt1, set_button = 0
					widget_control, event.top, set_uvalue = info, /NO_Copy
					ShowMain, event
				ENDELSE
				ENDELSE
				END

	'show1':	BEGIN
					current = info.showbitmask[1]
					IF (current EQ 0) THEN BEGIN
						info.showbitmask[1] = 1
						widget_control, event.top, set_uvalue = info, /NO_Copy
						ShowMain, event
					ENDIF ELSE BEGIN
						info.showbitmask[1] = 0
						widget_control, event.top, set_uvalue = info, /NO_Copy
						ShowMain, event
					ENDELSE
				END

	'show2':	BEGIN
					current = info.showbitmask[2]
					IF (current EQ 0) THEN BEGIN
						info.showbitmask[2] = 1
						widget_control, event.top, set_uvalue = info, /NO_Copy
						ShowMain, event
					ENDIF ELSE BEGIN
						info.showbitmask[2] = 0
						widget_control, event.top, set_uvalue = info, /NO_Copy
						ShowMain, event
					ENDELSE
				END

	'show3':	BEGIN
					current = info.showbitmask[3]
					IF (current EQ 0) THEN BEGIN
						info.showbitmask[3] = 1
						widget_control, event.top, set_uvalue = info, /NO_Copy
						ShowMain, event
					ENDIF ELSE BEGIN
						info.showbitmask[3] = 0
						widget_control, event.top, set_uvalue = info, /NO_Copy
						ShowMain, event
					ENDELSE
			 	END

	'show4':	BEGIN
					current = info.showbitmask[4]
					IF (current EQ 0) THEN BEGIN
						info.showbitmask[4] = 1
						widget_control, event.top, set_uvalue = info, /NO_Copy
						ShowMain, event
					ENDIF ELSE BEGIN
						info.showbitmask[4] = 0
						widget_control, event.top, set_uvalue = info, /NO_Copy
						ShowMain, event
					ENDELSE
			 	END

	'show5':	BEGIN
					current = info.showbitmask[5]
					IF (current EQ 0) THEN BEGIN
						info.showbitmask[5] = 1
						widget_control, event.top, set_uvalue = info, /NO_Copy
						ShowMain, event
					ENDIF ELSE BEGIN
						info.showbitmask[5] = 0
						widget_control, event.top, set_uvalue = info, /NO_Copy
						ShowMain, event
					ENDELSE
			 	END

	'wulff':	BEGIN
				IF (info.nettrigger NE 2) THEN BEGIN
				info.nettrigger = 2
				dimtrigger = info.dimtrigger
				widget_control, event.top, set_uvalue = info, /NO_Copy

	;-> do not plot wulff net in 3D mode
				IF (dimtrigger EQ 1) THEN ShowMain, event
				ENDIF ELSE BEGIN
				widget_control, event.top, set_uvalue = info, /NO_Copy
				RETURN
				ENDELSE
				END

	'polar':	BEGIN
				IF (info.nettrigger NE 1) THEN BEGIN
				info.nettrigger = 1
				dimtrigger = info.dimtrigger
				widget_control, event.top, set_uvalue = info, /NO_Copy

	;-> do not plot polar net in 3D mode
				IF (dimtrigger EQ 1) THEN ShowMain, event
				ENDIF ELSE BEGIN
				widget_control, event.top, set_uvalue = info, /NO_Copy
				RETURN
				ENDELSE
				END

	'labelon':	BEGIN
				info.labelon = 1
				widget_control, event.top, set_uvalue = info, /NO_Copy
				ShowMain, event
				END

	'labeloff':	BEGIN
				info.labelon = 0
				widget_control, event.top, set_uvalue = info, /NO_Copy
				ShowMain, event
				END

	'rotatepole': BEGIN
					current = info.rotatebitmask[0]
					IF (current EQ 0) THEN BEGIN
						info.rotatebitmask[0] = 1
						widget_control, event.top, set_uvalue = info, /NO_Copy
					ENDIF ELSE BEGIN
						info.rotatebitmask[0] = 0
						widget_control, event.top, set_uvalue = info, /NO_Copy
					ENDELSE
				END

	'rotatecryst1':	BEGIN
						current = info.rotatebitmask[1]
						IF (current EQ 0) THEN BEGIN
						info.rotatebitmask[1] = 1
						widget_control, event.top, set_uvalue = info, /NO_Copy
					ENDIF ELSE BEGIN
						info.rotatebitmask[1] = 0
						widget_control, event.top, set_uvalue = info, /NO_Copy
					ENDELSE
				END

	'rotatecryst2':	BEGIN
						current = info.rotatebitmask[2]
						IF (current EQ 0) THEN BEGIN
						info.rotatebitmask[2] = 1
						widget_control, event.top, set_uvalue = info, /NO_Copy
					ENDIF ELSE BEGIN
						info.rotatebitmask[2] = 0
						widget_control, event.top, set_uvalue = info, /NO_Copy
					ENDELSE
				END

	'rotatecryst3':	BEGIN
						current = info.rotatebitmask[3]
						IF (current EQ 0) THEN BEGIN
						info.rotatebitmask[3] = 1
						widget_control, event.top, set_uvalue = info, /NO_Copy
					ENDIF ELSE BEGIN
						info.rotatebitmask[3] = 0
						widget_control, event.top, set_uvalue = info, /NO_Copy
					ENDELSE
				END

	'rotatecryst4':	BEGIN
						current = info.rotatebitmask[4]
						IF (current EQ 0) THEN BEGIN
						info.rotatebitmask[4] = 1
						widget_control, event.top, set_uvalue = info, /NO_Copy
					ENDIF ELSE BEGIN
						info.rotatebitmask[4] = 0
						widget_control, event.top, set_uvalue = info, /NO_Copy
					ENDELSE
				END

	'rotatecryst5':	BEGIN
						current = info.rotatebitmask[5]
						IF (current EQ 0) THEN BEGIN
						info.rotatebitmask[5] = 1
						widget_control, event.top, set_uvalue = info, /NO_Copy
					ENDIF ELSE BEGIN
						info.rotatebitmask[5] = 0
						widget_control, event.top, set_uvalue = info, /NO_Copy
					ENDELSE
				END

	'rotleft':	BEGIN
					widget_control, event.top, set_uvalue = info, /NO_Copy
					RotatePoleFigure, event, /left, /samefigure
					RotateStereo, event, /left
					ShowMain, event ; V2
				END

	'rotright' :BEGIN
					widget_control, event.top, set_uvalue = info, /NO_Copy
					RotatePoleFigure, event, /right, /samefigure
					RotateStereo, event, /right
					ShowMain, event ; V2
				END

	'rotstepsize' :	BEGIN
						widget_control, event.id, get_value = numbervalue
						info.rotstepsize = numbervalue
						widget_control, event.top, set_uvalue = info, /NO_Copy
					END
	'tablerealized1':	BEGIN
							info.crystaltrigger = 1
							widget_control, event.top, set_uvalue = info, /NO_Copy
							stereopoleTable, event
						END
	'tablerealized2':	BEGIN
							info.crystaltrigger = 2
							widget_control, event.top, set_uvalue = info, /NO_Copy
							stereopoleTable, event
						END
	'tablerealized3':	BEGIN
							info.crystaltrigger = 3
							widget_control, event.top, set_uvalue = info, /NO_Copy
							stereopoleTable, event
						END
	'tablerealized4':	BEGIN
							info.crystaltrigger = 4
							widget_control, event.top, set_uvalue = info, /NO_Copy
							stereopoleTable, event
						END
	'tablerealized5':	BEGIN
							info.crystaltrigger = 5
							widget_control, event.top, set_uvalue = info, /NO_Copy
							stereopoleTable, event
						END
	'Preferences'	:	BEGIN
						IF (N_ELEMENTS(*info.daten) EQ 0) THEN BEGIN
							errormsg = dialog_message('Please Load Pole Figure first!', /ERROR)
							widget_control, event.top, set_uvalue = info, /NO_Copy
						RETURN
						ENDIF ELSE BEGIN
							widget_control, event.top, set_uvalue = info, /NO_Copy
							Preferences, event
						ENDELSE
						END
	'ps':				BEGIN
							widget_control, event.top, set_uvalue = info, /NO_Copy
							OutputPS, event
						END
	'hkl_list':			BEGIN
						IF (info.modetrigger EQ 0) AND (N_ELEMENTS(*info.daten) NE 0) THEN BEGIN
							IF ((N_Elements(*info.allindices1) GT 0) OR (N_Elements(*info.allindices2) GT 0)) THEN BEGIN
								widget_control, event.top, set_uvalue = info, /NO_Copy
								HKL_list, event
							ENDIF ELSE BEGIN
								widget_control, event.top, set_uvalue = info, /NO_Copy
								RETURN
							ENDELSE
						ENDIF ELSE BEGIN
							widget_control, event.top, set_uvalue = info, /NO_Copy
							RETURN
						ENDELSE
						END

	'crystal1_load':	BEGIN
							*info.temp = 1
							widget_control, event.top, set_uvalue = info, /NO_Copy
							LoadCrystal, event
						END
	'crystal2_load':	BEGIN
							*info.temp = 2
							widget_control, event.top, set_uvalue = info, /NO_Copy
							LoadCrystal, event
						END
	'crystal3_load':	BEGIN
							*info.temp = 3
							widget_control, event.top, set_uvalue = info, /NO_Copy
							LoadCrystal, event
						END
	'crystal4_load':	BEGIN
							*info.temp = 4
							widget_control, event.top, set_uvalue = info, /NO_Copy
							LoadCrystal, event
						END
	'crystal5_load':	BEGIN
							*info.temp = 5
							widget_control, event.top, set_uvalue = info, /NO_Copy
							LoadCrystal, event
						END
	'crystal1_save':	BEGIN
							*info.temp = 1
							widget_control, event.top, set_uvalue = info, /NO_Copy
							SaveCrystal, event
						END
	'crystal2_save':	BEGIN
							*info.temp = 2
							widget_control, event.top, set_uvalue = info, /NO_Copy
							SaveCrystal, event
						END
	'crystal3_save':	BEGIN
							*info.temp = 3
							widget_control, event.top, set_uvalue = info, /NO_Copy
							SaveCrystal, event
						END
	'crystal4_save':	BEGIN
							*info.temp = 4
							widget_control, event.top, set_uvalue = info, /NO_Copy
							SaveCrystal, event
						END
	'crystal5_save':	BEGIN
							*info.temp = 5
							widget_control, event.top, set_uvalue = info, /NO_Copy
							SaveCrystal, event
						END
	'crystal1_hkl':		BEGIN
							IF N_ELEMENTS(*info.allindices1) GT 0 THEN info.axistrigger = 1 ELSE errormsg = dialog_message('Please set crystal data first!', /ERROR)
							widget_control, event.top, set_uvalue = info, /NO_Copy
						END
	'crystal2_hkl':		BEGIN
							IF N_ELEMENTS(*info.allindices2) GT 0 THEN info.axistrigger = 2 ELSE errormsg = dialog_message('Please set crystal data first!', /ERROR)
							widget_control, event.top, set_uvalue = info, /NO_Copy
						END
	'crystal3_hkl':		BEGIN
							IF N_ELEMENTS(*info.allindices3) GT 0 THEN info.axistrigger = 3 ELSE errormsg = dialog_message('Please set crystal data first!', /ERROR)
							widget_control, event.top, set_uvalue = info, /NO_Copy
						END
	'crystal4_hkl':		BEGIN
							IF N_ELEMENTS(*info.allindices4) GT 0 THEN info.axistrigger = 4 ELSE errormsg = dialog_message('Please set crystal data first!', /ERROR)
							widget_control, event.top, set_uvalue = info, /NO_Copy
						END
	'crystal5_hkl':		BEGIN
							IF N_ELEMENTS(*info.allindices5) GT 0 THEN info.axistrigger = 5 ELSE errormsg = dialog_message('Please set crystal data first!', /ERROR)
							widget_control, event.top, set_uvalue = info, /NO_Copy
						END
	'newAxis':			BEGIN
							info.axistrigger = 6
							widget_control, event.top, set_uvalue = info, /NO_Copy
							Showmain, event
							XYOUTS, 15, 15, 'Click on desired position to set new axis of rotation ...', /Device
						END
	'Integration':		BEGIN
							info.axistrigger = 7
							widget_control, event.top, set_uvalue = info, /NO_Copy
							Showmain, event
							XYOUTS, 15, 15, 'Double click on desired peak for integration, press <Shift> for sum of several peaks ...', /Device
						END
	'angles1':			BEGIN
							IF N_ELEMENTS(*info.allindices1) GT 0 THEN info.axistrigger = 8 ELSE errormsg = dialog_message('Please set crystal data first!', /ERROR)
							*info.temp = [[0,0,0],[0,0,0]]
							widget_control, event.top, set_uvalue = info, /NO_Copy
							Showmain, event
							XYOUTS, 15, 15, 'Click on two positions to calculate interplanar angles for crystal 1 ...', /Device
						END
	'angles2':			BEGIN
							IF N_ELEMENTS(*info.allindices2) GT 0 THEN info.axistrigger = 9 ELSE errormsg = dialog_message('Please set crystal data first!', /ERROR)
							*info.temp = [[0,0,0],[0,0,0]]
							widget_control, event.top, set_uvalue = info, /NO_Copy
							Showmain, event
							XYOUTS, 15, 15, 'Click on two positions to calculate interplanar angles for crystal 2 ...', /Device
						END
	'angles3':			BEGIN
							IF N_ELEMENTS(*info.allindices3) GT 0 THEN info.axistrigger = 10 ELSE errormsg = dialog_message('Please set crystal data first!', /ERROR)
							*info.temp = [[0,0,0],[0,0,0]]
							widget_control, event.top, set_uvalue = info, /NO_Copy
							Showmain, event
							XYOUTS, 15, 15, 'Click on two positions to calculate interplanar angles for crystal 3 ...', /Device
						END
	'angles4':			BEGIN
							IF N_ELEMENTS(*info.allindices4) GT 0 THEN info.axistrigger = 11 ELSE errormsg = dialog_message('Please set crystal data first!', /ERROR)
							*info.temp = [[0,0,0],[0,0,0]]
							widget_control, event.top, set_uvalue = info, /NO_Copy
							Showmain, event
							XYOUTS, 15, 15, 'Click on two positions to calculate interplanar angles for crystal 4 ...', /Device
						END
	'angles5':			BEGIN
							IF N_ELEMENTS(*info.allindices5) GT 0 THEN info.axistrigger = 12 ELSE errormsg = dialog_message('Please set crystal data first!', /ERROR)
							*info.temp = [[0,0,0],[0,0,0]]
							widget_control, event.top, set_uvalue = info, /NO_Copy
							Showmain, event
							XYOUTS, 15, 15, 'Click on two positions to calculate interplanar angles for crystal 5 ...', /Device
						END
	'minint':			BEGIN
							widget_control, event.id, get_value = numbervalue
							*info.minint = FLOAT(numbervalue)
							IF (*info.minint GE *info.maxint) THEN *info.minint = *info.maxint - 10
							IF *info.minint LT 0 THEN *info.minint = 0
							widget_Control, info.minintID, set_value = *info.minint
							widget_control, event.top, Set_UValue = info, /No_Copy
							Showmain, event
						END
	'maxint':			BEGIN
							widget_control, event.id, get_value = numbervalue
							*info.maxint = FLOAT(numbervalue)
							IF (*info.maxint LE *info.minint) THEN *info.maxint = *info.minint + 10
							widget_Control, info.maxintID, set_value = *info.maxint
							widget_control, event.top, set_uvalue = info, /NO_Copy
							Showmain, event
						END

;-> Cursor and Zoom events

	'plotwindow'  :	BEGIN

;-> After change of Preferences
					IF (info.updatetrigger EQ 1) THEN BEGIN
						widget_control, event.top, set_uvalue = info, /NO_Copy
						ShowMain, event
						RETURN
					ENDIF

;-> After recalculation of crystal
					IF (info.updatetrigger EQ 2) THEN BEGIN
						bitmask = info.rotatebitmask
						bitmask_temp = INTARR(6)
						bitmask_temp[info.crystaltrigger] = 1
						info.rotatebitmask = bitmask_temp
						widget_control, event.top, set_uvalue = info, /No_Copy
						RotateStereo, event, /recalculated											; maybe avoid second computation ?
						widget_control, event.top, get_uvalue = info, /NO_Copy
						info.rotatebitmask = bitmask
						widget_control, event.top, set_uvalue = info, /NO_Copy
						ShowMain, event
						RETURN
					ENDIF

;-> Give current position

						zoomtrigger = info.zoomtrigger
						cursorx = event.x
						cursory = event.y
						current = CONVERT_COORD(cursorx, cursory, /DEVICE, /TO_DATA)

					IF ((N_ELEMENTS(*info.daten) NE 0) AND (info.dimtrigger EQ 1)) THEN BEGIN
						d = SQRT((*info.x-current[0])^2 + (*info.y-current[1])^2)
						curr_radius = SQRT((current[0])^2 + (current[1])^2)
						result = WHERE(d EQ MIN(d))
						intense = LONG64((*info.z)[result])
						IF ((curr_radius LE *info.radius) AND (info.modetrigger EQ 0)) THEN BEGIN
							label_int = string('Int.[cps]: ', strcompress(intense[0]))
						ENDIF ELSE BEGIN
							label_int = ' '
						ENDELSE
						widget_control, info.labelID_int, set_value = label_int
					ENDIF ELSE BEGIN
						label_int = ' '
						widget_control, info.labelID_int, set_value = label_int
					ENDELSE

						curr_psi =	ACos((1 - current[0]^2 - current[1]^2)/$
										(1 + current[0]^2 + current[1]^2))*180/!Pi

						curr_phi = ATan(current[1]/current[0])*180/!Pi

						CASE 1 OF
							;3. Quadrant
							(current[1] LT 0) AND (current[0] LE 0): curr_phi= curr_phi + 180
							;2. Quadrant
							(current[1] GT 0) AND (current[0] LT 0): curr_phi= curr_phi + 180
							;4. Quadrant
							(current[1] LE 0) AND (current[0] GT 0): curr_phi= curr_phi + 360
							ELSE :
						ENDCASE

						IF ((N_ELEMENTS(*info.daten) ne 0) AND (info.dimtrigger EQ 1)) THEN BEGIN
						curr_phi  = STRING(curr_phi, FORMAT = '(F10.2)')
						curr_psi  = STRING(curr_psi, FORMAT = '(F10.2)')
						label = string('Psi: ', strcompress(curr_psi),',  Phi: ', strcompress(curr_phi))
						widget_control, info.labelID, set_value = label
						ENDIF ELSE BEGIN
						label = ' '
						widget_control, info.labelID, set_value = label
						ENDELSE

;-> ZoomIn if desired [zoomtrigger set to 0]

						IF (event.press EQ 1) AND (zoomtrigger EQ 0) THEN BEGIN
								info.clickx1 = cursorx
								info.clicky1 = cursory
								info.zoomtrigger = 1 ;-> value 1 means first corner marked ...

						ENDIF ELSE BEGIN
							IF (event.press EQ 1) AND (zoomtrigger EQ 1) THEN BEGIN
								info.clickx2 = cursorx
								info.clicky2 = cursory
								info.zoomtrigger = 2 ;-> value 2 means rectangle ready -> zoom in!

								widget_control, event.top, set_uvalue = info, /NO_Copy

								PerformZoom, event
								RETURN
							ENDIF
						ENDELSE

;-> Give current position of single click
						IF ((N_ELEMENTS(*info.daten) NE 0) AND (info.dimtrigger EQ 1) AND (info.axistrigger GT 0) AND (info.axistrigger LT 6) AND (info.modetrigger EQ 0)) THEN BEGIN
							IF (event.press EQ 1) THEN BEGIN
								current_click = CONVERT_COORD(cursorx, cursory, /DEVICE, /TO_DATA)
								current = Project_to_3d(current_click[0], current_click[1]) 	;-> rotation axis within the orthogonal system
								hkl = CalculateBack(info, current, info.quality, info.axistrigger-1)
								hkl_string = STRING(STRCOMPRESS(hkl))
								hkl_string[0] = STRCOMPRESS(hkl_string[0],/REMOVE_ALL)
	                         	MyString = 'Selected (hkl) for Crystal ' + STRING(STRCOMPRESS(info.axistrigger)) + ': (' + hkl_string[0] + hkl_string[1] + hkl_string[2] +')'
	                         	widget_control, event.top, set_uvalue = info, /NO_Copy
								ShowMain, event
								widget_control, event.top, get_uvalue = info, /NO_Copy
								XYOUTS, 15, 15, MyString, /Device
							ENDIF
						ENDIF

;-> Set new global axis for rotation
						IF ((N_ELEMENTS(*info.daten) NE 0) AND (info.dimtrigger EQ 1) AND (info.axistrigger EQ 6) AND (info.modetrigger EQ 0)) THEN BEGIN
							IF (event.press EQ 1) THEN BEGIN
								current_click = CONVERT_COORD(cursorx, cursory, /DEVICE, /TO_DATA)
								info.rotaxis_xyz = Project_to_3d(current_click[0], current_click[1]) 	;-> rotation axis within the orthogonal system
								IF (N_ELEMENTS(*info.allindices1) NE 0) THEN BEGIN
									hkl = CalculateBack(info, info.rotaxis_xyz, info.quality, 0)
									hkl_string = STRING(STRCOMPRESS(hkl))
									hkl_string[0] = STRCOMPRESS(hkl_string[0],/REMOVE_ALL)
		                         	MyString = 'Selected new axis of rotation equals ('+ hkl_string[0] + hkl_string[1] + hkl_string[2] +') of Crystal 1.'
		                         	widget_control, event.top, set_uvalue = info, /NO_Copy
									ShowMain, event
									widget_control, event.top, get_uvalue = info, /NO_Copy
									XYOUTS, 15, 15, MyString, /Device
								ENDIF
							ENDIF
						ENDIF

;-> Calculate Interplanar angle ...
						IF ((N_ELEMENTS(*info.daten) NE 0) AND (info.dimtrigger EQ 1) AND (info.axistrigger GE 8) AND (info.modetrigger EQ 0)) THEN BEGIN
							temp = *info.temp
							IF ((event.press EQ 1) AND ARRAY_EQUAL(temp[*,0],[0,0,0])) THEN BEGIN
								current_click = CONVERT_COORD(cursorx, cursory, /DEVICE, /TO_DATA)
								current = Project_to_3d(current_click[0], current_click[1]) 	;-> rotation axis within the orthogonal system
								hkl = CalculateBack(info, current, info.quality, info.axistrigger-8)
								temp[*,0] = hkl
								*info.temp = temp
								hkl_string = STRING(STRCOMPRESS(hkl))
								hkl_string[0] = STRCOMPRESS(hkl_string[0],/REMOVE_ALL)
	                         	MyString = 'Crystal' + STRING(STRCOMPRESS(info.axistrigger-7)) + ': Angle between planes (' + hkl_string[0] + hkl_string[1] + hkl_string[2] +')'
	                         	widget_control, event.top, set_uvalue = info, /NO_Copy
								ShowMain, event
								XYOUTS, 15, 15, MyString, /Device
								RETURN
							ENDIF
							IF ((event.press EQ 1) AND ARRAY_EQUAL(temp[*,1],[0,0,0])) THEN BEGIN
								current_click = CONVERT_COORD(cursorx, cursory, /DEVICE, /TO_DATA)
								current = Project_to_3d(current_click[0], current_click[1]) 	;-> rotation axis within the orthogonal system
								hkl = CalculateBack(info, current, info.quality, info.axistrigger-8)
								temp[*,1] = hkl
								*info.temp = temp
								hkl_string = STRING(STRCOMPRESS(hkl))
								hkl_string[0] = STRCOMPRESS(hkl_string[0],/REMOVE_ALL)
 								temp_string = STRING(STRCOMPRESS(temp[*,0]))
								temp_string[0] = STRCOMPRESS(temp_string[0],/REMOVE_ALL)
	                         	MyString = 'Crystal' + STRING(STRCOMPRESS(info.axistrigger-7)) + ': Angle between planes (' + temp_string[0] + temp_string[1] + temp_string[2] +') ' + 'and (' + hkl_string[0] + hkl_string[1] + hkl_string[2] +'):'
	                         	widget_control, event.top, set_uvalue = info, /NO_Copy
								ShowMain, event
								widget_control, event.top, get_uvalue = info, /NO_Copy
								angle = InterplanarAngle(info, temp, info.axistrigger-8)
								info.axistrigger = 0
								XYOUTS, 15, 15, MyString + STRCOMPRESS(STRING(angle, FORMAT = '(F10.3)'))+'', /Device
							ENDIF

						ENDIF

;-> Give current position of mouse click for peak integration [double click]
						IF ((event.press EQ 1) AND (event.clicks EQ 2) AND (N_ELEMENTS(*info.daten) NE 0) AND (info.axistrigger EQ 7)) THEN BEGIN
						*info.peakx = cursorx
						*info.peaky = cursory
						*info.temp = event.modifiers

							;-> Delete circles if Shift not pressed
							IF (event.modifiers NE 1) THEN BEGIN
								widget_control, event.top, set_uvalue = info, /NO_Copy
								Zeichnen_xyz_reset, event
								widget_control, event.top, get_uvalue = info, /NO_Copy
								XYOUTS, 15, 15, 'Double click on desired peak for integration, press <Shift> for sum of several peaks.', /Device
							ENDIF

							widget_control, event.top, set_uvalue = info, /NO_Copy
							Peak_Integration, event
							widget_control, event.top, get_uvalue = info, /NO_Copy
						ENDIF

						widget_control, event.top, set_uvalue = info, /NO_Copy
					END
	Else : widget_control, event.top, set_uvalue = info, /NO_Copy
Endcase
ENDELSE
END

; **************** Printing by Procedure Printwindow **************************
;+
; NAME
;   PRINTWINDOW
;
;    This program sends the contents of the specified
;    window to the default printer. The current window
;    is used if a window index number is not provided.
;
; AUTHOR:
;
;   FANNING SOFTWARE CONSULTING
;   David Fanning, Ph.D.
;   1645 Sheely Drive
;   Fort Collins, CO 80526 USA
;   Phone: 970-221-0438
;   E-mail: davidf@dfanning.com
;   Coyote's Guide to IDL Programming: http://www.dfanning.com/
;
; CATEGORY:
;
;  Graphics
;
; CALLING SEQUENCE:
;
;  IDL> PrintWindow, wid
;
; OPTIONAL POSITIONAL PARAMETERS:
;
;   WID       The window index number of the window to send to the
;             printer. !D.Window used by default.
;
; KEYWORD PARAMETERS:
;
;   LANDSCAPE  If this keyword is set, the output is in Landscape
;              mode. Otherwise, Portrait mode is used.
;
;   PAGESIZE: Set this keyword to a string indicating the type
;             of PostScript page size you want. Current values are "LETTER",
;             "LEGAL", and "A4". Default is "LETTER".
;
; MODIFICATION HISTORY:
;
; Written by David Fanning based on previous PRINT_IT program. 29 July 2000.
;-
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; Copyright  2000 Fanning Software Consulting
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################


FUNCTION PRINTWINDOW_PSWINDOW_ASPECT, aspectRatio, MARGIN=margin, WindowAspect=wAspectRatio

ON_ERROR, 1

   ; Check for aspect ratio parameter and possibilities.

IF N_PARAMS() EQ 0 THEN aspectRatio = 1.0

IF aspectRatio EQ 0 THEN BEGIN
   MESSAGE, 'Aspect Ratio of 0. Changing to 1...', /Informational
   aspectRatio = 1.0
ENDIF

s = SIZE(aspectRatio)
IF s(s(0)+1) NE 4 THEN $
   MESSAGE, 'Aspect Ratio is not a FLOAT. Take care...', /Informational

   ; Check for margins.

IF N_ELEMENTS(margin) EQ 0 THEN margin = 0.15

   ; Error checking.

IF margin LT 0 OR margin GE 0.5 THEN $
   MESSAGE, 'The MARGIN keyword value must be between 0.0 and 0.5.'

   ; Calculate the aspect ratio of the current window.

IF N_Elements(wAspectRatio) EQ 0 THEN wAspectRatio = FLOAT(!D.Y_VSIZE) / !D.X_VSIZE

   ; Calculate normalized positions in window.

IF (aspectRatio LE wAspectRatio) THEN BEGIN
   xstart = margin
   ystart = 0.5 - (0.5 - margin) * (aspectRatio / wAspectRatio)
   xend = 1.0 - margin
   yend = 0.5 + (0.5 - margin) * (aspectRatio / wAspectRatio)
ENDIF ELSE BEGIN
   xstart = 0.5 - (0.5 - margin) * (wAspectRatio / aspectRatio)
   ystart = margin
   xend = 0.5 + (0.5 - margin) * (wAspectRatio / aspectRatio)
   yend = 1.0 - margin
ENDELSE

position = [xstart, ystart, xend, yend]

RETURN, position
END ; ----------------------------------------------------------------------------------



FUNCTION PRINTWINDOW_PSWINDOW, LANDSCAPE=landscape, CM=cm, MARGIN=margin, $
   PageSize=pagesize, Printer=printer, Fudge=fudge, XFudge=xfudge, YFudge=yfudge

   ; Set up default values.

landscape = Keyword_Set(landscape)
cm = Keyword_Set(cm)
printer = Keyword_Set(printer)
inches = 1

   ; Set up printer fudge factors, if necessary.

IF N_Elements(fudge) NE 0 THEN BEGIN
   xfudge = fudge
   yfudge = fudge
ENDIF
IF N_Elements(xfudge) EQ 0 THEN xfudge = 0.0
IF N_Elements(yfudge) EQ 0 THEN yfudge = 0.0

   ; Get the page size.

IF N_Elements(pagesize) EQ 0 THEN pagesize = 'LETTER' $
   ELSE pagesize = StrUpCase(pagesize)
CASE pagesize OF
   'LETTER': BEGIN
      shortside = 8.5
      longside = 11.0
      ENDCASE
   'LEGAL': BEGIN
      shortside = 8.5
      longside = 14.0
      ENDCASE
    'A4': BEGIN
      shortside = 8.27
      longside = 11.7
      ENDCASE
    ELSE: BEGIN
      Message, 'Unknown page size. Using LETTER...', /Informational
      shortside = 8.5
      longside = 11.0
      ENDCASE
ENDCASE

   ; Need measurements in centimeters?

IF KEYWORD_SET(cm) THEN BEGIN
      shortside = shortside * 2.54
      longside = longside * 2.54
      inches = 0
ENDIF

   ; Determine the margin of the window on the page.

IF N_ELEMENTS(margin) EQ 0 THEN margin=0.15

   ; Get the aspect ratio of the current display window. Aspect ratio
   ; is ratio of xsize/ysize.

aspectRatio = FLOAT(!D.Y_VSIZE) / !D.X_VSIZE

   ; Get the aspect ratio of the page.

IF Keyword_Set(landscape) THEN pAspectRatio = shortside / longside $
   ELSE pAspectRatio = longside / shortside

   ; Get the position on the page for this window.

pos = PRINTWINDOW_PSWindow_Aspect(aspectRatio, Margin=margin, WindowAspect=pAspectRatio)

   ; Convert normalized position coordinates to size units.

IF KEYWORD_SET(landscape) THEN BEGIN
   IF printer THEN BEGIN
      xsize = (pos[2] - pos[0]) * longside
      ysize = (pos[3] - pos[1]) * shortside
      yoffset = pos[1] * shortside - yfudge
      xoffset = pos[0] * longside - xfudge
      landscape = 1
      portrait = 0
   ENDIF ELSE BEGIN
      xsize = (pos[2] - pos[0]) * longside
      ysize = (pos[3] - pos[1]) * shortside
      xoffset = pos[1] * shortside
      yoffset = longside - (pos[0] * longside)
      landscape = 1
      portrait = 0
   ENDELSE
ENDIF ELSE BEGIN
   xsize = (pos[2] - pos[0]) * shortside
   ysize = (pos[3] - pos[1]) * longside
   IF printer THEN BEGIN
      xoffset = pos[0] * shortside - xfudge
      yoffset = pos[1] * longside - yfudge
   ENDIF ELSE BEGIN
      xoffset = pos[0] * shortside
      yoffset = pos[1] * longside
   ENDELSE
   landscape = 0
   portrait = 1
ENDELSE

   ; Return the proper DEVICE data structure.

RETURN, {PSWINDOW_STRUCT, XSIZE:xsize, YSIZE:ysize, $
   XOFFSET:xoffset, YOFFSET:yoffset, INCHES:inches, $
   PORTRAIT:portrait, LANDSCAPE:landscape}

END ;--------------------------------------------------------------------------------------


FUNCTION PrintWindow_Error, theMessage, Traceback=traceback, NoName=noName

On_Error, 2

   ; Check for presence and type of message.

IF N_Elements(theMessage) EQ 0 THEN theMessage = !Error_State.Msg
s = Size(theMessage)
messageType = s[s[0]+1]
IF messageType NE 7 THEN BEGIN
   Message, "The message parameter must be a string.", _Extra=extra
ENDIF

   ; Get the call stack and the calling routine's name.

Help, Calls=callStack
callingRoutine = (Str_Sep(StrCompress(callStack[1])," "))[0]

   ; Are widgets supported? Doesn't matter in IDL 5.3 and higher.

widgetsSupported = ((!D.Flags AND 65536L) NE 0) OR Float(!Version.Release) GE 5.3
IF widgetsSupported THEN BEGIN
   IF Keyword_Set(noName) THEN answer = Dialog_Message(theMessage, _Extra=extra) ELSE BEGIN
      IF StrUpCase(callingRoutine) EQ "$MAIN$" THEN answer = Dialog_Message(theMessage, _Extra=extra) ELSE $
         answer = Dialog_Message(StrUpCase(callingRoutine) + ": " + theMessage)
   ENDELSE
ENDIF ELSE BEGIN
      Message, theMessage, /Continue, /NoPrint, /NoName, /NoPrefix, _Extra=extra
      Print, '%' + callingRoutine + ': ' + theMessage
      answer = 'OK'
ENDELSE

   ; Provide traceback information if requested.

IF Keyword_Set(traceback) THEN BEGIN
   Help, /Last_Message, Output=traceback
   Print,''
   Print, 'Traceback Report from Error_Message:'
   Print, ''
   FOR j=0,N_Elements(traceback)-1 DO Print, "     " + traceback[j]
ENDIF

RETURN, answer
END ;-----------------------------------------------------------------------------------


PRO PrintWindow, wid, Landscape=landscape, PageSize=pageSize

   ; Check parameters.

IF N_Params() EQ 0 THEN wid = !D.Window
landscape = Keyword_Set(landscape)
IF N_Elements(pagesize) EQ 0 THEN pagesize = 'LETTER' $
   ELSE pagesize = StrUpCase(pagesize)
CASE pagesize OF
   'LETTER': BEGIN
      shortside = 8.5
      longside = 11.0
      ENDCASE
   'LEGAL': BEGIN
      shortside = 8.5
      longside = 14.0
      ENDCASE
    'A4': BEGIN
      shortside = 8.27
      longside = 11.7
      ENDCASE
    ELSE: BEGIN
      Message, 'Unknown page size. Using LETTER...', /Informational
      shortside = 8.5
      longside = 11.0
      ENDCASE
ENDCASE

   ; Are we running on a 24-bit device?

Catch, theError
IF theError NE 0 THEN BEGIN
   theDepth = 8
   GOTO, testDepth
ENDIF
Device, Get_Visual_Depth=theDepth
testDepth:
IF theDepth GT 8 THEN truecolor = 1 ELSE truecolor = 0

   ; Main error handler.

Catch, theError
IF theError NE 0 THEN BEGIN
   Catch, /Cancel
   ok = PrintWindow_Error(!Error_State.Msg)
   RETURN
ENDIF

   ; Valid window ID?

IF wid LT 0 THEN Message, 'No graphics window currently open.', /NoName

   ; Get information about printer.

ok = Dialog_PrinterSetup()
IF NOT ok THEN RETURN

   ; Make the window current. Get contents.

thisWindow = !D.Window
WSet, wid
contents = TVRD(True=truecolor)

   ; Need a window on printer with same aspect ratio as current window.
   ; Your fudge factor (to account for offsets calculated from the printable
   ; edge of the page rather than the real edge of the page) may be different.

keywords = PRINTWINDOW_PSWindow(/Printer, Landscape=landscape, Fudge=0.25, PageSize=pagesize, Margin=0)

   ; Change the current device to PRINTER. Copy color table.

thisDevice = !D.Name

Set_Plot, 'PRINTER', /Copy

   ; Reset the PRINTER for proper calculations. Landscape mode
   ; has to be set separately from sizes. I don't know why.

Device, Scale_Factor=1, Portrait=1
Device, Landscape = keywords.landscape

   ; Configure the printer.

Device, _Extra=keywords

   ; Print it.

TV, contents, True=truecolor, XSize=keywords.xsize, $
   YSize=keywords.ysize, Inches=keywords.inches
Plots, [0,0,1,1,0], [0,1,1,0,0], /Normal
Device, /Close_Document

   ; Clean up.

Set_Plot, thisDevice
WSet, thisWindow

END

; *************************** End of Procedure PRINTWINDOW **************************